@@ -143,6 +143,7 @@ conversation_element_list_name(wmem_allocator_t *allocator, conversation_element
         "string",
         "uint",
         "uint64",
+        "int",
     };
     char *sep = "";
     wmem_strbuf_t *conv_hash_group = wmem_strbuf_new(allocator, "");
@@ -195,6 +196,9 @@ static char* conversation_element_list_values(conversation_element_t *elements)
         case CE_UINT64:
             g_string_append_printf(value_str, "%" G_GUINT64_FORMAT, cur_el->uint64_val);
             break;
+        case CE_INT:
+            g_string_append_printf(value_str, "%d", cur_el->int_val);
+            break;
         }
     }
     return g_string_free(value_str, FALSE);
@@ -368,6 +372,11 @@ conversation_hash_element_list(gconstpointer v)
             tmp_addr.data = &element->uint64_val;
             hash_val = add_address_to_hash(hash_val, &tmp_addr);
             break;
+        case CE_INT:
+            tmp_addr.len = (int) sizeof(element->int_val);
+            tmp_addr.data = &element->int_val;
+            hash_val = add_address_to_hash(hash_val, &tmp_addr);
+            break;
         case CE_CONVERSATION_TYPE:
             tmp_addr.len = (int) sizeof(element->conversation_type_val);
             tmp_addr.data = &element->conversation_type_val;
@@ -426,6 +435,11 @@ conversation_match_element_list(gconstpointer v1, gconstpointer v2)
                 return FALSE;
             }
             break;
+        case CE_INT:
+            if (element1->int_val != element2->int_val) {
+                return FALSE;
+            }
+            break;
         case CE_CONVERSATION_TYPE:
             if (element1->conversation_type_val != element2->conversation_type_val) {
                 return FALSE;
@@ -1912,6 +1926,8 @@ conversation_type conversation_pt_to_conversation_type(port_type pt)
             return CONVERSATION_BLUETOOTH;
         case PT_IWARP_MPA:
             return CONVERSATION_IWARP_MPA;
+        case PT_MCTP:
+            return CONVERSATION_MCTP;
     }
 
     DISSECTOR_ASSERT(FALSE);
@@ -1950,6 +1966,8 @@ endpoint_type conversation_pt_to_endpoint_type(port_type pt)
             return ENDPOINT_BLUETOOTH;
         case PT_IWARP_MPA:
             return ENDPOINT_IWARP_MPA;
+        case PT_MCTP:
+            return ENDPOINT_MCTP;
     }
 
     DISSECTOR_ASSERT(FALSE);
