@@ -1,8 +1,8 @@
 /* packet-nr-rrc-template.c
  * NR;
  * Radio Resource Control (RRC) protocol specification
- * (3GPP TS 38.331 V17.1.0 Release 17) packet dissection
- * Copyright 2018-2022, Pascal Quantin
+ * (3GPP TS 38.331 V17.5.0 Release 17) packet dissection
+ * Copyright 2018-2023, Pascal Quantin
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -408,7 +408,7 @@ nr_rrc_RSRQ_Range_fmt(gchar *s, guint32 v)
   } else if (v < 127) {
     snprintf(s, ITEM_LABEL_LENGTH, "%.1fdB <= SS-RSRQ < %.1fdB (%u)", (((float)v-1)/2)-43, ((float)v/2)-43, v);
   } else {
-    snprintf(s, ITEM_LABEL_LENGTH, "-20dB <= SS-RSRQ (127)");
+    snprintf(s, ITEM_LABEL_LENGTH, "20dB <= SS-RSRQ (127)");
   }
 }
 
@@ -744,6 +744,21 @@ dissect_nr_rrc_rrcreconfiguration_msg(tvbuff_t* tvb _U_, packet_info* pinfo _U_,
 }
 
 
+static int
+dissect_nr_rrc_rrcreconfigurationcomplete_msg(tvbuff_t* tvb _U_, packet_info* pinfo _U_, proto_tree* tree _U_, void* data _U_)
+{
+    proto_item* ti;
+    proto_tree* sub_tree;
+
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, "NR RRC");
+    col_set_str(pinfo->cinfo, COL_INFO, "RRCReconfigurationComplete");
+
+    ti = proto_tree_add_item(tree, proto_nr_rrc, tvb, 0, -1, ENC_NA);
+    sub_tree = proto_item_add_subtree(ti, ett_nr_rrc);
+    return dissect_nr_rrc_RRCReconfigurationComplete_PDU(tvb, pinfo, sub_tree, NULL);
+}
+
+
 static int
 dissect_nr_rrc_ue_capabilityrat_containerlist_msg(tvbuff_t* tvb _U_, packet_info* pinfo _U_, proto_tree* tree _U_, void* data _U_)
 {
@@ -1032,6 +1047,7 @@ proto_register_nr_rrc(void) {
   register_dissector("nr-rrc.cg_configinfo", dissect_nr_rrc_cg_configinfo_msg, proto_nr_rrc);
   register_dissector("nr-rrc.radiobearerconfig", dissect_nr_rrc_radiobearerconfig_msg, proto_nr_rrc);
   register_dissector("nr-rrc.rrc_reconf_msg", dissect_nr_rrc_rrcreconfiguration_msg, proto_nr_rrc);
+  register_dissector("nr-rrc.rrc_reconf_compl_msg", dissect_nr_rrc_rrcreconfigurationcomplete_msg, proto_nr_rrc);
   register_dissector("nr-rrc.ue_capabilityrat_containerlist", dissect_nr_rrc_ue_capabilityrat_containerlist_msg, proto_nr_rrc);
   register_dissector("nr-rrc.ue_mrdc_cap_msg", dissect_nr_rrc_ue_mrdc_capability_msg, proto_nr_rrc);
   register_dissector("nr-rrc.ue_nr_cap_msg", dissect_nr_rrc_ue_nr_capability_msg, proto_nr_rrc);
