@@ -26,9 +26,7 @@
 
 #include <config.h>
 #include <epan/packet.h>
-#include <epan/conversation.h>
 #include <epan/proto_data.h>
-#include <epan/tvbuff.h>
 #include <string.h>
 #include "packet-tcp.h"
 
@@ -36,6 +34,8 @@
 void proto_reg_handoff_FiveCoLegacy(void);
 void proto_register_FiveCoLegacy(void);
 
+static dissector_handle_t FiveCoLegacy_handle;
+
 /****************************************************************************/
 /* Definition declaration */
 /****************************************************************************/
@@ -44,7 +44,7 @@ void proto_register_FiveCoLegacy(void);
 #define FIVECO_LEGACY_HEADER_LENGTH 6
 #define FIVECO_LEGACY_MIN_LENGTH FIVECO_LEGACY_HEADER_LENGTH + 2 // Checksum is 16 bits
 
-#define PROTO_TAG_FIVECO "5co-legacy"
+#define PSNAME "5co-legacy"
 
 /* Global sample ports preferences */
 #define FIVECO_PORT1 8010     /* TCP port of the FiveCo protocol */
@@ -156,11 +156,11 @@ typedef struct
 {
     guint16 usParaLen;
     guint16 isReplied;
-    tvbuff_t *pData;
+    guint8 *pDataBuffer;
 } FCOSConvRequestVal;
 
 /* Conversation hash tables */
-static GHashTable *FiveCo_requests_hash = NULL;
+static wmem_map_t *FiveCo_requests_hash = NULL;
 
 /* Internal unique ID (used to match answer with question
    since some software set always 0 as packet ID in protocol header)
@@ -181,53 +181,53 @@ typedef struct
 
 /* Known (common on every product) registers */
 static FCOSRegisterDef aRegisters[] = {
-    {0x00, 4, "Register Type/Model", "5co-legacy.RegTypeModel", FT_UINT32, -1, CF_FUNC(dispType)},
-    {0x01, 4, "Register Version", "5co-legacy.RegVersion", FT_UINT32, -1, CF_FUNC(dispVersion)},
-    {0x02, 0, "Function Reset device", "5co-legacy.RegReset", FT_NONE, -1, NULL},
-    {0x03, 0, "Function Save user parameters", "5co-legacy.RegSave", FT_NONE, -1, NULL},
-    {0x04, 0, "Function Restore user parameters", "5co-legacy.RegRestore", FT_NONE, -1, NULL},
-    {0x05, 0, "Function Restore factory parameters", "5co-legacy.RegRestoreFact", FT_NONE, -1, NULL},
-    {0x06, 0, "Function Save factory parameters", "5co-legacy.SaveFact", FT_NONE, -1, NULL},
-    {0x07, 0, "Register unknown", "5co-legacy.RegUnknown07", FT_NONE, -1, NULL},
-    {0x08, 0, "Register unknown", "5co-legacy.RegUnknown08", FT_NONE, -1, NULL},
-    {0x09, 0, "Register unknown", "5co-legacy.RegUnknown09", FT_NONE, -1, NULL},
-    {0x0A, 0, "Register unknown", "5co-legacy.RegUnknown0A", FT_NONE, -1, NULL},
-    {0x0B, 0, "Register unknown", "5co-legacy.RegUnknown0B", FT_NONE, -1, NULL},
-    {0x0C, 0, "Register unknown", "5co-legacy.RegUnknown0C", FT_NONE, -1, NULL},
-    {0x0D, 0, "Register unknown", "5co-legacy.RegUnknown0D", FT_NONE, -1, NULL},
-    {0x0E, 0, "Register unknown", "5co-legacy.RegUnknown0E", FT_NONE, -1, NULL},
-    {0x0F, 0, "Register unknown", "5co-legacy.RegUnknown0F", FT_NONE, -1, NULL},
-    {0x10, 4, "Register Communication options", "5co-legacy.RegComOption", FT_UINT32, -1, NULL},
-    {0x11, 6, "Register Ethernet MAC Address", "5co-legacy.RegMAC", FT_UINT48, -1, CF_FUNC(dispMAC)},
-    {0x12, 4, "Register IP Address", "5co-legacy.RegIPAdd", FT_UINT32, -1, CF_FUNC(dispIP)},
-    {0x13, 4, "Register IP Mask", "5co-legacy.RegIPMask", FT_UINT32, -1, CF_FUNC(dispMask)},
-    {0x14, 1, "Register TCP Timeout", "5co-legacy.RegTCPTimeout", FT_UINT8, -1, CF_FUNC(dispTimeout)},
-    {0x15, 16, "Register Module name", "5co-legacy.RegName", FT_STRING, -1, NULL}};
+    {0x00, 4, "Register Type/Model", "5co_legacy.RegTypeModel", FT_UINT32, -1, CF_FUNC(dispType)},
+    {0x01, 4, "Register Version", "5co_legacy.RegVersion", FT_UINT32, -1, CF_FUNC(dispVersion)},
+    {0x02, 0, "Function Reset device", "5co_legacy.RegReset", FT_NONE, -1, NULL},
+    {0x03, 0, "Function Save user parameters", "5co_legacy.RegSave", FT_NONE, -1, NULL},
+    {0x04, 0, "Function Restore user parameters", "5co_legacy.RegRestore", FT_NONE, -1, NULL},
+    {0x05, 0, "Function Restore factory parameters", "5co_legacy.RegRestoreFact", FT_NONE, -1, NULL},
+    {0x06, 0, "Function Save factory parameters", "5co_legacy.SaveFact", FT_NONE, -1, NULL},
+    {0x07, 0, "Register unknown", "5co_legacy.RegUnknown07", FT_NONE, -1, NULL},
+    {0x08, 0, "Register unknown", "5co_legacy.RegUnknown08", FT_NONE, -1, NULL},
+    {0x09, 0, "Register unknown", "5co_legacy.RegUnknown09", FT_NONE, -1, NULL},
+    {0x0A, 0, "Register unknown", "5co_legacy.RegUnknown0A", FT_NONE, -1, NULL},
+    {0x0B, 0, "Register unknown", "5co_legacy.RegUnknown0B", FT_NONE, -1, NULL},
+    {0x0C, 0, "Register unknown", "5co_legacy.RegUnknown0C", FT_NONE, -1, NULL},
+    {0x0D, 0, "Register unknown", "5co_legacy.RegUnknown0D", FT_NONE, -1, NULL},
+    {0x0E, 0, "Register unknown", "5co_legacy.RegUnknown0E", FT_NONE, -1, NULL},
+    {0x0F, 0, "Register unknown", "5co_legacy.RegUnknown0F", FT_NONE, -1, NULL},
+    {0x10, 4, "Register Communication options", "5co_legacy.RegComOption", FT_UINT32, -1, NULL},
+    {0x11, 6, "Register Ethernet MAC Address", "5co_legacy.RegMAC", FT_UINT48, -1, CF_FUNC(dispMAC)},
+    {0x12, 4, "Register IP Address", "5co_legacy.RegIPAdd", FT_UINT32, -1, CF_FUNC(dispIP)},
+    {0x13, 4, "Register IP Mask", "5co_legacy.RegIPMask", FT_UINT32, -1, CF_FUNC(dispMask)},
+    {0x14, 1, "Register TCP Timeout", "5co_legacy.RegTCPTimeout", FT_UINT8, -1, CF_FUNC(dispTimeout)},
+    {0x15, 16, "Register Module name", "5co_legacy.RegName", FT_STRING, -1, NULL}};
 
     /* List of static header fields */
 static hf_register_info hf_base[] = {
-    {&hf_fiveco_header, {"Header", "5co-legacy.header", FT_NONE, BASE_NONE, NULL, 0x0, "Header of the packet", HFILL}},
-    {&hf_fiveco_fct, {"Function", "5co-legacy.fct", FT_UINT16, BASE_HEX, VALS(packettypenames), 0x0, "Function type", HFILL}},
-    {&hf_fiveco_id, {"Frame ID", "5co-legacy.id", FT_UINT16, BASE_DEC, NULL, 0x0, "Packet ID", HFILL}},
-    {&hf_fiveco_length, {"Data length", "5co-legacy.length", FT_UINT16, BASE_DEC, NULL, 0x0, "Parameters length of the packet", HFILL}},
-    {&hf_fiveco_data, {"Data", "5co-legacy.data", FT_NONE, BASE_NONE, NULL, 0x0, "Data (parameters)", HFILL}},
-    {&hf_fiveco_cks, {"Checksum", "5co-legacy.checksum", FT_UINT16, BASE_HEX, NULL, 0x0, "Checksum of the packet", HFILL}},
-    {&hf_fiveco_i2cadd, {"I2C Address", "5co-legacy.i2cadd", FT_UINT8, BASE_DEC, NULL, 0x0, NULL, HFILL}},
-    {&hf_fiveco_i2c2write, {"I2C number of bytes to write", "5co-legacy.i2c2write", FT_UINT8, BASE_DEC, NULL, 0x0, NULL, HFILL}},
-    {&hf_fiveco_i2cwrite, {"I2C bytes to write", "5co-legacy.i2cwrite", FT_NONE, BASE_NONE, NULL, 0x0, NULL, HFILL}},
-    {&hf_fiveco_i2c2read, {"I2C number of bytes to read", "5co-legacy.i2c2read", FT_UINT8, BASE_DEC, NULL, 0x0, NULL, HFILL}},
-    {&hf_fiveco_i2canswer, {"I2C bytes read", "5co-legacy.i2cread", FT_NONE, BASE_NONE, NULL, 0x0, NULL, HFILL}},
-    {&hf_fiveco_i2cwriteanswer, {"I2C bytes write", "5co-legacy.i2writeanswer", FT_NONE, BASE_NONE, NULL, 0x0, NULL, HFILL}},
-    {&hf_fiveco_i2cack, {"I2C ack state", "5co-legacy.i2cack", FT_UINT8, BASE_HEX, NULL, 0x0, NULL, HFILL}},
-    {&hf_fiveco_i2c2scan, {"I2C addresses to scan", "5co-legacy.i2c2scan", FT_NONE, BASE_NONE, NULL, 0x0, NULL, HFILL}},
-    {&hf_fiveco_i2cscaned, {"I2C addresses present", "5co-legacy.i2cscaned", FT_NONE, BASE_NONE, NULL, 0x0, NULL, HFILL}},
-    {&hf_fiveco_i2cerror, {"I2C error", "5co-legacy.i2cerror", FT_NONE, BASE_NONE, NULL, 0x0, NULL, HFILL}},
-    {&hf_fiveco_regread, {"Read", "5co-legacy.regread", FT_NONE, BASE_NONE, NULL, 0x0, NULL, HFILL}},
-    {&hf_fiveco_regreadunknown, {"Read Register unknown", "5co-legacy.hf_fiveco_regreadunknown", FT_NONE, BASE_NONE, NULL, 0x0, NULL, HFILL}},
-    {&hf_fiveco_regreaduk, {"Data not decoded", "5co-legacy.regreaduk", FT_NONE, BASE_NONE, NULL, 0x0, "Data not decoded because there are unable to map to a known register", HFILL}},
-    {&hf_fiveco_EasyIPMAC, {"MAC address", "5co-legacy.EasyIPMAC", FT_ETHER, BASE_NONE, NULL, 0x0, NULL, HFILL}},
-    {&hf_fiveco_EasyIPIP, {"New IP address", "5co-legacy.EasyIPIP", FT_IPv4, BASE_NONE, NULL, 0x0, NULL, HFILL}},
-    {&hf_fiveco_EasyIPSM, {"New subnet mask", "5co-legacy.EasyIPSM", FT_IPv4, BASE_NONE, NULL, 0x0, NULL, HFILL}}
+    {&hf_fiveco_header, {"Header", "5co_legacy.header", FT_NONE, BASE_NONE, NULL, 0x0, "Header of the packet", HFILL}},
+    {&hf_fiveco_fct, {"Function", "5co_legacy.fct", FT_UINT16, BASE_HEX, VALS(packettypenames), 0x0, "Function type", HFILL}},
+    {&hf_fiveco_id, {"Frame ID", "5co_legacy.id", FT_UINT16, BASE_DEC, NULL, 0x0, "Packet ID", HFILL}},
+    {&hf_fiveco_length, {"Data length", "5co_legacy.length", FT_UINT16, BASE_DEC, NULL, 0x0, "Parameters length of the packet", HFILL}},
+    {&hf_fiveco_data, {"Data", "5co_legacy.data", FT_NONE, BASE_NONE, NULL, 0x0, "Data (parameters)", HFILL}},
+    {&hf_fiveco_cks, {"Checksum", "5co_legacy.checksum", FT_UINT16, BASE_HEX, NULL, 0x0, "Checksum of the packet", HFILL}},
+    {&hf_fiveco_i2cadd, {"I2C Address", "5co_legacy.i2cadd", FT_UINT8, BASE_DEC, NULL, 0x0, NULL, HFILL}},
+    {&hf_fiveco_i2c2write, {"I2C number of bytes to write", "5co_legacy.i2c2write", FT_UINT8, BASE_DEC, NULL, 0x0, NULL, HFILL}},
+    {&hf_fiveco_i2cwrite, {"I2C bytes to write", "5co_legacy.i2cwrite", FT_NONE, BASE_NONE, NULL, 0x0, NULL, HFILL}},
+    {&hf_fiveco_i2c2read, {"I2C number of bytes to read", "5co_legacy.i2c2read", FT_UINT8, BASE_DEC, NULL, 0x0, NULL, HFILL}},
+    {&hf_fiveco_i2canswer, {"I2C bytes read", "5co_legacy.i2cread", FT_NONE, BASE_NONE, NULL, 0x0, NULL, HFILL}},
+    {&hf_fiveco_i2cwriteanswer, {"I2C bytes write", "5co_legacy.i2writeanswer", FT_NONE, BASE_NONE, NULL, 0x0, NULL, HFILL}},
+    {&hf_fiveco_i2cack, {"I2C ack state", "5co_legacy.i2cack", FT_UINT8, BASE_HEX, NULL, 0x0, NULL, HFILL}},
+    {&hf_fiveco_i2c2scan, {"I2C addresses to scan", "5co_legacy.i2c2scan", FT_NONE, BASE_NONE, NULL, 0x0, NULL, HFILL}},
+    {&hf_fiveco_i2cscaned, {"I2C addresses present", "5co_legacy.i2cscaned", FT_NONE, BASE_NONE, NULL, 0x0, NULL, HFILL}},
+    {&hf_fiveco_i2cerror, {"I2C error", "5co_legacy.i2cerror", FT_NONE, BASE_NONE, NULL, 0x0, NULL, HFILL}},
+    {&hf_fiveco_regread, {"Read", "5co_legacy.regread", FT_NONE, BASE_NONE, NULL, 0x0, NULL, HFILL}},
+    {&hf_fiveco_regreadunknown, {"Read Register unknown", "5co_legacy.hf_fiveco_regreadunknown", FT_NONE, BASE_NONE, NULL, 0x0, NULL, HFILL}},
+    {&hf_fiveco_regreaduk, {"Data not decoded", "5co_legacy.regreaduk", FT_NONE, BASE_NONE, NULL, 0x0, "Data not decoded because there are unable to map to a known register", HFILL}},
+    {&hf_fiveco_EasyIPMAC, {"MAC address", "5co_legacy.EasyIPMAC", FT_ETHER, BASE_NONE, NULL, 0x0, NULL, HFILL}},
+    {&hf_fiveco_EasyIPIP, {"New IP address", "5co_legacy.EasyIPIP", FT_IPv4, BASE_NONE, NULL, 0x0, NULL, HFILL}},
+    {&hf_fiveco_EasyIPSM, {"New subnet mask", "5co_legacy.EasyIPSM", FT_IPv4, BASE_NONE, NULL, 0x0, NULL, HFILL}}
  };
 
 /*****************************************************************************/
@@ -256,12 +256,11 @@ dissect_FiveCoLegacy(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *
     guint64 *pulInternalID = NULL;
     FCOSConvRequestKey requestKey, *pNewRequestKey;
     FCOSConvRequestVal *pRequestVal = NULL;
-    guint8 *pNewDataBuffer = NULL;
+    tvbuff_t *pRequestTvb = NULL;
     guint8 ucAdd, ucBytesToWrite, ucBytesToRead;
     guint8 ucRegAdd, ucRegSize;
     guint32 unOffset;
     guint32 unSize;
-    char *string_buf = NULL;
 
     /* Load protocol payload length (including checksum) */
     tcp_data_length = tvb_captured_length(tvb);
@@ -269,7 +268,7 @@ dissect_FiveCoLegacy(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *
         return 0;
 
     /* Display fiveco in protocol column */
-    col_set_str(pinfo->cinfo, COL_PROTOCOL, PROTO_TAG_FIVECO);
+    col_set_str(pinfo->cinfo, COL_PROTOCOL, PSNAME);
     /* Clear out stuff in the info column */
     col_clear(pinfo->cinfo, COL_INFO);
 
@@ -319,7 +318,7 @@ dissect_FiveCoLegacy(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *
         /* Get info about the request */
         requestKey.usExpCmd = header_type;
         requestKey.unInternalID = *pulInternalID;
-        pRequestVal = (FCOSConvRequestVal *)g_hash_table_lookup(FiveCo_requests_hash, &requestKey);
+        pRequestVal = (FCOSConvRequestVal *)wmem_map_lookup(FiveCo_requests_hash, &requestKey);
         if ((!pinfo->fd->visited) && (!pRequestVal) && (isRequest))
         {
             /* If unknown and if it is a request, allocate new hash element that we want to handle later in answer */
@@ -348,11 +347,14 @@ dissect_FiveCoLegacy(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *
             pRequestVal = wmem_new(wmem_file_scope(), FCOSConvRequestVal);
             pRequestVal->usParaLen = header_data_length;
             pRequestVal->isReplied = FALSE;
-            pNewDataBuffer = (guint8 *)wmem_alloc(wmem_file_scope(), header_data_length);
-            pRequestVal->pData = tvb_new_real_data(pNewDataBuffer, header_data_length, header_data_length);
-            tvb_memcpy(tvb, pNewDataBuffer, tcp_data_offset + 6, header_data_length);
+            pRequestVal->pDataBuffer = (guint8 *)wmem_alloc(wmem_file_scope(), header_data_length);
+            tvb_memcpy(tvb, pRequestVal->pDataBuffer, tcp_data_offset + 6, header_data_length);
 
-            g_hash_table_insert(FiveCo_requests_hash, pNewRequestKey, pRequestVal);
+            wmem_map_insert(FiveCo_requests_hash, pNewRequestKey, pRequestVal);
+        }
+
+        if (pRequestVal) {
+            pRequestTvb = tvb_new_child_real_data(tvb, pRequestVal->pDataBuffer, pRequestVal->usParaLen, pRequestVal->usParaLen);
         }
 
         /* Compute checksum of the packet and read one received */
@@ -521,10 +523,7 @@ dissect_FiveCoLegacy(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *
                                 aRegisters[ucRegAdd].nsWsHeaderID,
                                 tvb, i, ucRegSize,
                                 ENC_NA);
-                            string_buf = wmem_alloc(wmem_packet_scope(), (size_t)ucRegSize + 1);
-                            // Get string from tvb with an extra char for null terminaison
-                            tvb_get_raw_bytes_as_string(tvb, i, string_buf, (size_t)ucRegSize + 1);
-                            proto_item_append_text(fiveco_data_item, ": %.16s", string_buf);
+                            proto_item_append_text(fiveco_data_item, ": %s", tvb_format_text(pinfo->pool, tvb, i, ucRegSize));
                             i += ucRegSize;
                         }
                         // else display raw data in hex
@@ -572,13 +571,13 @@ dissect_FiveCoLegacy(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *
                         while ((y < pRequestVal->usParaLen) && (i < tcp_data_offset + header_data_length))
                         {
                             // I2C address in first byte of request
-                            ucAdd = tvb_get_guint8(pRequestVal->pData, y++);
+                            ucAdd = tvb_get_guint8(pRequestTvb, y++);
                             // Read number of bytes to write
-                            ucBytesToWrite = tvb_get_guint8(pRequestVal->pData, y);
+                            ucBytesToWrite = tvb_get_guint8(pRequestTvb, y);
                             // Skip number of bytes to write and those bytes
                             y += 1 + ucBytesToWrite;
                             // Read number of bytes to read
-                            ucBytesToRead = tvb_get_guint8(pRequestVal->pData, y++);
+                            ucBytesToRead = tvb_get_guint8(pRequestTvb, y++);
                             if (ucBytesToRead > 0)
                             {
                                 fiveco_data_item = proto_tree_add_item(fiveco_data_tree, hf_fiveco_i2canswer,
@@ -628,7 +627,7 @@ dissect_FiveCoLegacy(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *
                         while ((y < pRequestVal->usParaLen) && (i < tcp_data_offset + header_data_length))
                         {
                             // Register address in first byte of request
-                            ucRegAdd = tvb_get_guint8(pRequestVal->pData, y++);
+                            ucRegAdd = tvb_get_guint8(pRequestTvb, y++);
                             // If register address is known & found in answer
                             if ((ucRegAdd < array_length(aRegisters)) &&
                                 (aRegisters[ucRegAdd].unValue == ucRegAdd) &&
@@ -650,10 +649,7 @@ dissect_FiveCoLegacy(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *
 														    aRegisters[ucRegAdd].nsWsHeaderID,
 														    tvb, i, ucRegSize,
                                                             ENC_NA);
-                                    string_buf = wmem_alloc(wmem_packet_scope(), (size_t)ucRegSize + 1);
-                                    // Get string from tvb with an extra char for null terminaison
-                                    tvb_get_raw_bytes_as_string(tvb, i, string_buf, (size_t)ucRegSize + 1);
-                                    proto_item_append_text(fiveco_data_item, ": %.16s", string_buf);
+                                    proto_item_append_text(fiveco_data_item, ": %s", tvb_format_text(pinfo->pool, tvb, i, ucRegSize));
                                     i += ucRegSize;
                                 }
                                 // else display raw data in hex
@@ -694,9 +690,7 @@ dissect_FiveCoLegacy(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *
                 break;
             case FLASH_AREA_ANSWER:
                 if ( header_data_length > 1 ) {
-                    string_buf = wmem_alloc(wmem_packet_scope(), header_data_length);
-                    tvb_get_raw_bytes_as_string(tvb, tcp_data_offset, string_buf, header_data_length - 1);
-                    proto_item_append_text(fiveco_data_item, " (%s)", string_buf);
+                    proto_item_append_text(fiveco_data_item, " (%s)", tvb_format_text(pinfo->pool, tvb, tcp_data_offset, header_data_length - 1));
                 }
                 break;
 
@@ -787,16 +781,6 @@ static gint fiveco_hash_equal(gconstpointer v, gconstpointer w)
     return 0;
 }
 
-/*****************************************************************************/
-/* Protocol initialization function                                          */
-/*****************************************************************************/
-static void fiveco_protocol_init(void)
-{
-    if (FiveCo_requests_hash)
-        g_hash_table_destroy(FiveCo_requests_hash);
-    FiveCo_requests_hash = g_hash_table_new(fiveco_hash, fiveco_hash_equal);
-}
-
 /*****************************************************************************/
 /* Register the protocol with Wireshark.
  *
@@ -828,19 +812,19 @@ void proto_register_FiveCoLegacy(void)
         &ett_fiveco,
         &ett_fiveco_checksum};
 
-    /* Register the dissector */
     /* Register the protocol name and description */
     proto_FiveCoLegacy = proto_register_protocol("FiveCo's Legacy Register Access Protocol",
-                                                 PROTO_TAG_FIVECO, "5co_legacy");
+                                                 PSNAME, "5co_legacy");
 
     /* Required function calls to register the header fields and subtrees */
     proto_register_field_array(proto_FiveCoLegacy, hf, array_length(hf));
     proto_register_subtree_array(ett, array_length(ett));
 
-    /* Register hash init function
-        * Protocol hash is used to follow conversation.
-        */
-    register_init_routine(&fiveco_protocol_init);
+    /* Register the dissector */
+    FiveCoLegacy_handle = register_dissector("5co_legacy", dissect_FiveCoLegacy,
+                                                    proto_FiveCoLegacy);
+
+    FiveCo_requests_hash = wmem_map_new_autoreset(wmem_epan_scope(), wmem_file_scope(), fiveco_hash, fiveco_hash_equal);
 
     /* Set preference callback to NULL since it is not used */
     prefs_register_protocol(proto_FiveCoLegacy, NULL);
@@ -855,16 +839,9 @@ void proto_register_FiveCoLegacy(void)
 void proto_reg_handoff_FiveCoLegacy(void)
 {
     static gboolean initialized = FALSE;
-    static dissector_handle_t FiveCoLegacy_handle;
 
     if (!initialized)
     {
-        /* Use create_dissector_handle() to indicate that
-         * dissect_FiveCoLegacy() returns the number of bytes it dissected (or 0
-         * if it thinks the packet does not belong to PROTONAME).
-         */
-        FiveCoLegacy_handle = create_dissector_handle(dissect_FiveCoLegacy,
-                                                      proto_FiveCoLegacy);
         dissector_add_uint("tcp.port", FIVECO_PORT1, FiveCoLegacy_handle);
         dissector_add_uint("tcp.port", FIVECO_PORT2, FiveCoLegacy_handle);
         dissector_add_uint("udp.port", FIVECO_UDP_PORT1, FiveCoLegacy_handle);
