@@ -163,7 +163,7 @@ dissect_adb_cs(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _
             adb_service_data.session_key[1] = pinfo->destport;
             adb_service_data.session_key[2] = pinfo->srcport;
 
-            next_tvb = tvb_new_subset_length_caplen(tvb, offset, tvb_captured_length_remaining(tvb, offset), tvb_captured_length_remaining(tvb, offset));
+            next_tvb = tvb_new_subset_remaining(tvb, offset);
             call_dissector_with_data(adb_service_handle, next_tvb, pinfo, tree, &adb_service_data);
 
             return tvb_captured_length(tvb);
@@ -338,7 +338,7 @@ dissect_adb_cs(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _
         adb_service_data.session_key[1] = pinfo->destport;
         adb_service_data.session_key[2] = pinfo->srcport;
 
-        next_tvb = tvb_new_subset_length_caplen(tvb, offset, tvb_captured_length_remaining(tvb, offset), tvb_captured_length_remaining(tvb, offset));
+        next_tvb = tvb_new_subset_remaining(tvb, offset);
         call_dissector_with_data(adb_service_handle, next_tvb, pinfo, tree, &adb_service_data);
         offset = tvb_captured_length(tvb);
     } else {
