@@ -316,7 +316,9 @@ static int dissect_block_asb(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree
                     }
 
                     const gint offset_value = offset;
-                    wscbor_skip_next_item(wmem_packet_scope(), tvb, &offset);
+                    if (!wscbor_skip_next_item(wmem_packet_scope(), tvb, &offset)) {
+                        return 0;
+                    }
                     tvbuff_t *tvb_value = tvb_new_subset_length(tvb, offset_value, offset - offset_value);
 
                     dissector_handle_t value_dissect = NULL;
@@ -375,7 +377,9 @@ static int dissect_block_asb(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree
                         }
 
                         const gint offset_value = offset;
-                        wscbor_skip_next_item(wmem_packet_scope(), tvb, &offset);
+                        if (!wscbor_skip_next_item(wmem_packet_scope(), tvb, &offset)) {
+                            return 0;
+                        }
                         tvbuff_t *tvb_value = tvb_new_subset_length(tvb, offset_value, offset - offset_value);
 
                         dissector_handle_t value_dissect = NULL;
@@ -475,19 +479,15 @@ static void reinit_bpsec(void) {
 
 /// Overall registration of the protocol
 void proto_register_bpsec(void) {
-    proto_bpsec = proto_register_protocol(
-        "DTN Bundle Protocol Security", /* name */
-        "BPSec", /* short name */
-        "bpsec" /* abbrev */
-    );
+    proto_bpsec = proto_register_protocol("DTN Bundle Protocol Security", "BPSec", "bpsec");
 
     proto_register_field_array(proto_bpsec, fields, array_length(fields));
     proto_register_subtree_array(ett, array_length(ett));
     expert_module_t *expert = expert_register_protocol(proto_bpsec);
     expert_register_field_array(expert, expertitems, array_length(expertitems));
 
-    param_dissectors = register_custom_dissector_table("bpsec.param", "BPSec Parameter", proto_bpsec, bpsec_id_hash, bpsec_id_equal);
-    result_dissectors = register_custom_dissector_table("bpsec.result", "BPSec Result", proto_bpsec, bpsec_id_hash, bpsec_id_equal);
+    param_dissectors = register_custom_dissector_table("bpsec.param", "BPSec Parameter", proto_bpsec, bpsec_id_hash, bpsec_id_equal, g_free);
+    result_dissectors = register_custom_dissector_table("bpsec.result", "BPSec Result", proto_bpsec, bpsec_id_hash, bpsec_id_equal, g_free);
 
     prefs_register_protocol(proto_bpsec, reinit_bpsec);
 }
@@ -498,14 +498,14 @@ void proto_reg_handoff_bpsec(void) {
     /* Packaged extensions */
     {
         guint64 *key = g_new(guint64, 1);
-        *key = 11;
-        dissector_handle_t hdl = create_dissector_handle(dissect_block_bib, proto_bpsec);
+        *key = BP_BLOCKTYPE_BIB;
+        dissector_handle_t hdl = create_dissector_handle_with_name(dissect_block_bib, proto_bpsec, "Block Integrity Block");
         dissector_add_custom_table_handle("bpv7.block_type", key, hdl);
     }
     {
         guint64 *key = g_new(guint64, 1);
-        *key = 12;
-        dissector_handle_t hdl = create_dissector_handle(dissect_block_bcb, proto_bpsec);
+        *key = BP_BLOCKTYPE_BCB;
+        dissector_handle_t hdl = create_dissector_handle_with_name(dissect_block_bcb, proto_bpsec, "Block Confidentiality Block");
         dissector_add_custom_table_handle("bpv7.block_type", key, hdl);
     }
 
