@@ -33,7 +33,7 @@
 #include <epan/strutil.h>
 #include "packet-dtls.h"
 #include "packet-coap.h"
-#include "packet-http.h"
+#include "packet-media-type.h"
 #include "packet-tcp.h"
 #include "packet-tls.h"
 
@@ -108,6 +108,14 @@ static dissector_handle_t oscore_handle;
 #define DEFAULT_COAP_CTYPE_VALUE				~0U
 #define DEFAULT_COAP_BLOCK_NUMBER				~0U
 
+/* Macros specific to the OCF version options */
+#define COAP_OCF_VERSION_SUB_MASK 				0x3F
+#define COAP_OCF_VERSION_MINOR_MASK 				0x7C0
+#define COAP_OCF_VERSION_MAJOR_MASK 				0xF800
+
+#define COAP_OCF_VERSION_MINOR_OFFSET 				6
+#define COAP_OCF_VERSION_MAJOR_OFFSET 				11
+
 /*
  * Transaction Type
  */
@@ -200,7 +208,7 @@ const value_string coap_vals_observe_options[] = {
 #define COAP_OPT_URI_HOST		3
 #define COAP_OPT_ETAG			4
 #define COAP_OPT_IF_NONE_MATCH		5
-#define COAP_OPT_OBSERVE		6	/* core-observe-16 */
+#define COAP_OPT_OBSERVE		6	/* RFC 7641 / RFC 8613 */
 #define COAP_OPT_URI_PORT		7
 #define COAP_OPT_LOCATION_PATH		8
 #define COAP_OPT_OBJECT_SECURITY	9	/* RFC 8613 */
@@ -210,13 +218,21 @@ const value_string coap_vals_observe_options[] = {
 #define COAP_OPT_URI_QUERY		15
 #define COAP_OPT_HOP_LIMIT		16	/* RFC 8768 */
 #define COAP_OPT_ACCEPT			17
+#define COAP_OPT_QBLOCK1		19	/* RFC 9177 */
 #define COAP_OPT_LOCATION_QUERY		20
+#define COAP_OPT_EDHOC			21	/* draft-ietf-core-oscore-edhoc*/
 #define COAP_OPT_BLOCK2			23	/* RFC 7959 / RFC 8323 */
 #define COAP_OPT_BLOCK1			27	/* RFC 7959 / RFC 8323 */
 #define COAP_OPT_SIZE2			28	/* RFC 7959 */
+#define COAP_OPT_QBLOCK2		31	/* RFC 9177 */
 #define COAP_OPT_PROXY_URI		35
 #define COAP_OPT_PROXY_SCHEME		39
 #define COAP_OPT_SIZE1			60
+#define COAP_OPT_ECHO			252	/* RFC 9175*/
+#define COAP_OPT_NO_RESPONSE		258	/* RFC 7967 / RFC 8613 */
+#define COAP_OPT_REQUEST_TAG		292	/* RFC 9175 */
+#define COAP_OPT_OCF_ACCEPT		2049	/* OCF Core specification */
+#define COAP_OPT_OCF_CONTENT		2053	/* OCF Core specification */
 
 static const value_string vals_opt_type[] = {
 	{ COAP_OPT_IF_MATCH,       "If-Match" },
@@ -232,7 +248,9 @@ static const value_string vals_opt_type[] = {
 	{ COAP_OPT_URI_QUERY,      "Uri-Query" },
 	{ COAP_OPT_HOP_LIMIT,      "Hop-Limit" },
 	{ COAP_OPT_ACCEPT,         "Accept" },
+	{ COAP_OPT_QBLOCK1,        "Q-Block1" },
 	{ COAP_OPT_LOCATION_QUERY, "Location-Query" },
+	{ COAP_OPT_EDHOC,          "EDHOC" },
 	{ COAP_OPT_PROXY_URI,      "Proxy-Uri" },
 	{ COAP_OPT_PROXY_SCHEME,   "Proxy-Scheme" },
 	{ COAP_OPT_SIZE1,          "Size1" },
@@ -240,6 +258,12 @@ static const value_string vals_opt_type[] = {
 	{ COAP_OPT_BLOCK2,         "Block2" },
 	{ COAP_OPT_BLOCK1,         "Block1" },
 	{ COAP_OPT_SIZE2,          "Size2" },
+	{ COAP_OPT_QBLOCK2,        "Q-Block2" },
+	{ COAP_OPT_ECHO,           "Echo" },
+	{ COAP_OPT_NO_RESPONSE,    "No-Response" },
+	{ COAP_OPT_REQUEST_TAG,    "Request-Tag" },
+	{ COAP_OPT_OCF_ACCEPT,     "OCF-Accept-Content-Format-Version" },
+	{ COAP_OPT_OCF_CONTENT,    "OCF-Content-Format-Version" },
 	{ 0, NULL },
 };
 
@@ -261,7 +285,9 @@ struct coap_option_range_t {
 	{ COAP_OPT_URI_QUERY,       1, 255 },
 	{ COAP_OPT_HOP_LIMIT,       1,   1 },
 	{ COAP_OPT_ACCEPT,          0,   2 },
+	{ COAP_OPT_QBLOCK1,         0,   3 },
 	{ COAP_OPT_LOCATION_QUERY,  0, 255 },
+	{ COAP_OPT_EDHOC,           0,   0 },
 	{ COAP_OPT_PROXY_URI,       1,1034 },
 	{ COAP_OPT_PROXY_SCHEME,    1, 255 },
 	{ COAP_OPT_SIZE1,           0,   4 },
@@ -269,10 +295,23 @@ struct coap_option_range_t {
 	{ COAP_OPT_BLOCK2,          0,   3 },
 	{ COAP_OPT_BLOCK1,          0,   3 },
 	{ COAP_OPT_SIZE2,           0,   4 },
+	{ COAP_OPT_QBLOCK2,         0,   3 },
+	{ COAP_OPT_ECHO,            1,  40 },
+	{ COAP_OPT_NO_RESPONSE,     0,   1 },
+	{ COAP_OPT_REQUEST_TAG,     0,   8 },
+	{ COAP_OPT_OCF_ACCEPT,      2,   2 },
+	{ COAP_OPT_OCF_CONTENT,     2,   2 },
 };
 
 static const value_string vals_ctype[] = {
 	{  0, "text/plain; charset=utf-8" },
+	{ 16, "application/cose; cose-type=\"cose-encrypt0\"" },
+	{ 17, "application/cose; cose-type=\"cose-mac0\"" },
+	{ 18, "application/cose; cose-type=\"cose-sign1\"" },
+	{ 19, "application/ace+cbor" },
+	{ 21, "image/gif" },
+	{ 22, "image/jpeg" },
+	{ 23, "image/png" },
 	{ 40, "application/link-format" },
 	{ 41, "application/xml" },
 	{ 42, "application/octet-stream" },
@@ -283,6 +322,7 @@ static const value_string vals_ctype[] = {
 	{ 60, "application/cbor" },
 	{ 61, "application/cwt" },
 	{ 62, "application/multipart-core" },
+	{ 63, "application/cbor-seq" },
 	{ 96, "application/cose; cose-type=\"cose-encrypt\"" },
 	{ 97, "application/cose; cose-type=\"cose-mac\"" },
 	{ 98, "application/cose; cose-type=\"cose-sign\"" },
@@ -294,7 +334,10 @@ static const value_string vals_ctype[] = {
 	{ 113, "application/sensml+cbor" },
 	{ 114, "application/senml-exi" },
 	{ 115, "application/sensml-exi" },
+	{ 140, "application/yang-data+cbor; id=sid" },
 	{ 256, "application/coap-group+json" },
+	{ 257, "application/concise-problem-details+cbor" },
+	{ 258, "application/swid+cbor" },
 	{ 271, "application/dots+cbor" },
 	{ 272, "application/missing-blocks+cbor-seq" },
 	{ 280, "application/pkcs7-mime; smime-type=server-generated-key" },
@@ -303,17 +346,28 @@ static const value_string vals_ctype[] = {
 	{ 285, "application/csrattrs" },
 	{ 286, "application/pkcs10" },
 	{ 287, "application/pkix-cert" },
+	{ 290, "application/aif+cbor" },
+	{ 291, "application/aif+json" },
 	{ 310, "application/senml+xml" },
 	{ 311, "application/sensml+xml" },
 	{ 320, "application/senml-etch+json" },
 	{ 322, "application/senml-etch+cbor" },
+	{ 340, "application/yang-data+cbor" },
+	{ 341, "application/yang-data+cbor; id=name" },
+	{ 322, "application/senml-etch+cbor" },
 	{ 432, "application/td+json" },
+	{ 433, "application/tm+json" },
 	{ 1542, "application/vnd.oma.lwm2m+tlv" },
 	{ 1543, "application/vnd.oma.lwm2m+json" },
 	{ 10000, "application/vnd.ocf+cbor" },
 	{ 10001, "application/oscore" },
+	{ 10002, "application/javascript" },
+	{ 11050, "application/json (Content Coding: deflate)" },
+	{ 11060, "application/cbor (Content Coding: deflate)" },
 	{ 11542, "application/vnd.oma.lwm2m+tlv" },
 	{ 11543, "application/vnd.oma.lwm2m+json" },
+	{ 20000, "text/css" },
+	{ 30000, "image/svg+xml" },
 	{ 0, NULL },
 };
 
@@ -759,10 +813,26 @@ dissect_coap_opt_block(tvbuff_t *tvb, proto_item *head_item, proto_tree *subtree
 	    tvb, offset + opt_length - 1, 1, encoded_block_size, "Block Size: %u (%u encoded)", block_esize, encoded_block_size);
 
 	/* add info to the head of the packet detail */
-	proto_item_append_text(head_item, ": NUM:%u, M:%u, SZX:%u",
+	proto_item_append_text(head_item, ": NUM:%u, M:%u, SZ:%u",
 	    coinfo->block_number, coinfo->block_mflag, block_esize);
 }
 
+static void
+dissect_coap_opt_ocf_version(tvbuff_t *tvb, proto_item *head_item, proto_tree *subtree, gint offset, gint opt_length, int hfindex)
+{
+	guint option_value = coap_get_opt_uint(tvb, offset, opt_length);
+
+	guint sub_version = option_value & COAP_OCF_VERSION_SUB_MASK;
+	guint minor_version = (option_value & COAP_OCF_VERSION_MINOR_MASK) >> COAP_OCF_VERSION_MINOR_OFFSET;
+	guint major_version = (option_value & COAP_OCF_VERSION_MAJOR_MASK) >> COAP_OCF_VERSION_MAJOR_OFFSET;
+
+	proto_tree_add_uint(subtree, hfindex, tvb, offset, opt_length, option_value);
+
+	/* add info to the head of the packet detail */
+	proto_item_append_text(head_item, ": %u.%u.%u",
+	    major_version, minor_version, sub_version);
+}
+
 static void
 dissect_coap_opt_uri_port(tvbuff_t *tvb, proto_item *head_item, proto_tree *subtree, gint offset, gint opt_length, coap_info *coinfo, int hf)
 {
@@ -999,6 +1069,18 @@ dissect_coap_options_main(tvbuff_t *tvb, packet_info *pinfo, proto_tree *coap_tr
 		dissect_coap_opt_uri_query(tvb, pinfo, item, subtree, offset,
 		    opt_length, coinfo, dissect_hf->hf.opt_uri_query);
 		break;
+	case COAP_OPT_ECHO:
+		dissect_coap_opt_hex_string(tvb, pinfo, item, subtree, offset,
+		    opt_length, dissect_hf->hf.opt_echo);
+		break;
+	case COAP_OPT_REQUEST_TAG:
+		dissect_coap_opt_hex_string(tvb, pinfo, item, subtree, offset,
+		    opt_length, dissect_hf->hf.opt_request_tag);
+		break;
+	case COAP_OPT_NO_RESPONSE:
+		dissect_coap_opt_uint(tvb, item, subtree, offset,
+		    opt_length, dissect_hf->hf.opt_no_response);
+		break;
 	case COAP_OPT_BLOCK2:
 		coinfo->block_option = 2;
 		dissect_coap_opt_block(tvb, item, subtree, offset,
@@ -1009,12 +1091,32 @@ dissect_coap_options_main(tvbuff_t *tvb, packet_info *pinfo, proto_tree *coap_tr
 		dissect_coap_opt_block(tvb, item, subtree, offset,
 		    opt_length, coinfo, dissect_hf);
 		break;
+	case COAP_OPT_QBLOCK2:
+		coinfo->block_option = 2;
+		dissect_coap_opt_block(tvb, item, subtree, offset,
+		    opt_length, coinfo, dissect_hf);
+		break;
+	case COAP_OPT_QBLOCK1:
+		coinfo->block_option = 1;
+		dissect_coap_opt_block(tvb, item, subtree, offset,
+		    opt_length, coinfo, dissect_hf);
+		break;
 	case COAP_OPT_IF_NONE_MATCH:
 		break;
+	case COAP_OPT_EDHOC:
+		break;
 	case COAP_OPT_SIZE2:
 		dissect_coap_opt_uint(tvb, item, subtree, offset,
 		    opt_length, dissect_hf->hf.opt_block_size);
 		break;
+	case COAP_OPT_OCF_CONTENT:
+		dissect_coap_opt_ocf_version(tvb, item, subtree, offset,
+		    opt_length, dissect_hf->hf.opt_ocf_version);
+		break;
+	case COAP_OPT_OCF_ACCEPT:
+		dissect_coap_opt_ocf_version(tvb, item, subtree, offset,
+		    opt_length, dissect_hf->hf.opt_ocf_accept_version);
+		break;
 	default:
 		dissect_coap_opt_hex_string(tvb, pinfo, item, subtree, offset,
 		    opt_length, dissect_hf->hf.opt_unknown);
@@ -1080,7 +1182,7 @@ dissect_coap_payload(tvbuff_t *tvb, packet_info *pinfo, proto_tree *coap_tree, p
 	tvbuff_t   *payload_tvb;
 	guint	    payload_length = offset_end - offset;
 	const char *coap_ctype_str_dis;
-	http_message_info_t message_info = {0};
+	media_content_info_t content_info = {0};
 	char	    str_payload[80];
 	int	    result = 0;
 
@@ -1127,8 +1229,8 @@ dissect_coap_payload(tvbuff_t *tvb, packet_info *pinfo, proto_tree *coap_tree, p
 	proto_item_set_generated(length_item);
 	payload_tvb = tvb_new_subset_length(tvb, offset, payload_length);
 
-	message_info.type = HTTP_OTHERS;
-	message_info.media_str = wmem_strbuf_get_str(coinfo->uri_str_strbuf);
+	content_info.type = MEDIA_CONTAINER_HTTP_OTHERS;
+	content_info.media_str = wmem_strbuf_get_str(coinfo->uri_str_strbuf);
 	/*
 	 * The Thread protocol uses application/octet-stream for its
 	 * messages, rather than having its own media type for those
@@ -1145,7 +1247,7 @@ dissect_coap_payload(tvbuff_t *tvb, packet_info *pinfo, proto_tree *coap_tree, p
 		 */
 		result = dissector_try_string(coap_tmf_media_type_dissector_table,
 		    coap_ctype_str_dis, payload_tvb, pinfo, parent_tree,
-		    &message_info);
+		    &content_info);
 	}
 	if (result == 0) {
 		/*
@@ -1154,7 +1256,7 @@ dissect_coap_payload(tvbuff_t *tvb, packet_info *pinfo, proto_tree *coap_tree, p
 		 */
 		dissector_try_string(media_type_dissector_table,
 		    coap_ctype_str_dis, payload_tvb, pinfo, parent_tree,
-		    &message_info);
+		    &content_info);
 	}
 	if (coinfo->object_security && !oscore) {
 		proto_item_set_text(payload_item, "Encrypted OSCORE Data");
@@ -1323,8 +1425,8 @@ dissect_coap_message(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree,
 	coinfo->block_option = 0;
 	coinfo->block_number = DEFAULT_COAP_BLOCK_NUMBER;
 	coinfo->block_mflag  = 0;
-	coinfo->uri_str_strbuf   = wmem_strbuf_sized_new(pinfo->pool, 0, 1024);
-	coinfo->uri_query_strbuf = wmem_strbuf_sized_new(pinfo->pool, 0, 1024);
+	coinfo->uri_str_strbuf   = wmem_strbuf_create(pinfo->pool);
+	coinfo->uri_query_strbuf = wmem_strbuf_create(pinfo->pool);
 	 /* Allocate pointers and static elements of oscore_info_t, arrays are allocated only if object security option is found during option parsing */
 	coinfo->oscore_info = wmem_new0(pinfo->pool, oscore_info_t);
 	coinfo->object_security = FALSE;
@@ -1741,7 +1843,7 @@ proto_register_coap(void)
 	 */
 	coap_tmf_media_type_dissector_table =
 	    register_dissector_table("coap_tmf_media_type",
-		"Internet media type for CoAP-TMF", proto_coap, FT_STRING, BASE_NONE);
+		"Internet media type for CoAP-TMF", proto_coap, FT_STRING, STRING_CASE_SENSITIVE);
 }
 
 void
