@@ -348,13 +348,25 @@ gboolean cose_param_key_equal(gconstpointer a, gconstpointer b) {
     return match;
 }
 
+void cose_param_key_free(gpointer ptr) {
+    cose_param_key_t *obj = (cose_param_key_t *)ptr;
+    if (obj->principal) {
+        g_variant_unref(obj->principal);
+    }
+    if (obj->label) {
+        g_variant_unref(obj->label);
+    }
+    g_free(obj);
+}
+
 /** Dissect an ID-value pair within a context.
  *
  * @param dis_table The cose_param_key_t dissector table.
  * @param[in,out] ctx The context from other pairs.
+ * @return True if the pair was dissected (even as opaque CBOR data).
  */
-static void dissect_header_pair(dissector_table_t dis_table, cose_header_context_t *ctx, tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gint *offset) {
-    wscbor_chunk_t *chunk_label = wscbor_chunk_read(wmem_packet_scope(), tvb, offset);
+static gboolean dissect_header_pair(dissector_table_t dis_table, cose_header_context_t *ctx, tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gint *offset) {
+    wscbor_chunk_t *chunk_label = wscbor_chunk_read(pinfo->pool, tvb, offset);
 
     proto_item *item_label = NULL;
     proto_tree *volatile tree_label = NULL;
@@ -366,17 +378,17 @@ static void dissect_header_pair(dissector_table_t dis_table, cose_header_context
     switch (chunk_label->type_major) {
         case CBOR_TYPE_UINT:
         case CBOR_TYPE_NEGINT: {
-            gint64 *label = wscbor_require_int64(wmem_packet_scope(), chunk_label);
+            gint64 *label = wscbor_require_int64(pinfo->pool, chunk_label);
             item_label = proto_tree_add_cbor_int64(tree, hf_hdr_label_int, pinfo, tvb, chunk_label, label);
             if (label) {
                 key.label = ctx->label =
                     g_variant_new_int64(*label);
-                label_str = wmem_strdup_printf(wmem_packet_scope(), "%" PRId64, *label);
+                label_str = wmem_strdup_printf(pinfo->pool, "%" PRId64, *label);
             }
             break;
         }
         case CBOR_TYPE_STRING: {
-            const char *label = wscbor_require_tstr(wmem_packet_scope(), chunk_label);
+            const char *label = wscbor_require_tstr(pinfo->pool, chunk_label);
             item_label = proto_tree_add_cbor_tstr(tree, hf_hdr_label_tstr, pinfo, tvb, chunk_label);
             if (label) {
                 key.label = ctx->label =
@@ -406,7 +418,9 @@ static void dissect_header_pair(dissector_table_t dis_table, cose_header_context
 
     // Peek into the value as tvb
     const gint offset_value = *offset;
-    wscbor_skip_next_item(wmem_packet_scope(), tvb, offset);
+    if (!wscbor_skip_next_item(pinfo->pool, tvb, offset)) {
+        return FALSE;
+    }
     tvb_value = tvb_new_subset_length(tvb, offset_value, *offset - offset_value);
 
     gint sublen = 0;
@@ -428,6 +442,7 @@ static void dissect_header_pair(dissector_table_t dis_table, cose_header_context
         CATCH_ALL {}
         ENDTRY;
     }
+    return TRUE;
 }
 
 /** Dissect an entire header map, either for messages, recipients, or keys.
@@ -438,23 +453,25 @@ static void dissect_header_pair(dissector_table_t dis_table, cose_header_context
  * @param[in,out] offset The data offset.
  */
 static void dissect_header_map(dissector_table_t dis_table, tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gint *offset) {
-    wscbor_chunk_t *chunk_hdr_map = wscbor_chunk_read(wmem_packet_scope(), tvb, offset);
+    wscbor_chunk_t *chunk_hdr_map = wscbor_chunk_read(pinfo->pool, tvb, offset);
     wscbor_require_map(chunk_hdr_map);
     proto_item *item_hdr_map = proto_tree_get_parent(tree);
     wscbor_chunk_mark_errors(pinfo, item_hdr_map, chunk_hdr_map);
-    if (!wscbor_skip_if_errors(wmem_packet_scope(), tvb, offset, chunk_hdr_map)) {
+    if (!wscbor_skip_if_errors(pinfo->pool, tvb, offset, chunk_hdr_map)) {
         proto_tree *tree_hdr_map = proto_item_add_subtree(item_hdr_map, ett_hdr_map);
 
-        cose_header_context_t *ctx = wmem_new0(wmem_packet_scope(), cose_header_context_t);
+        cose_header_context_t *ctx = wmem_new0(pinfo->pool, cose_header_context_t);
 
         for (guint64 ix = 0; ix < chunk_hdr_map->head_value; ++ix) {
-            dissect_header_pair(dis_table, ctx, tvb, pinfo, tree_hdr_map, offset);
+            if (!dissect_header_pair(dis_table, ctx, tvb, pinfo, tree_hdr_map, offset)) {
+                break;
+            }
         }
 
         if (ctx->principal) {
             g_variant_unref(ctx->principal);
         }
-        wmem_free(wmem_packet_scope(), ctx);
+        wmem_free(pinfo->pool, ctx);
     }
 
     proto_item_set_len(item_hdr_map, *offset - chunk_hdr_map->start);
@@ -481,8 +498,8 @@ static void dissect_msg_tag(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *t
  */
 static void dissect_headers(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gint *offset) {
     // Protected in bstr
-    wscbor_chunk_t *chunk_prot_bstr = wscbor_chunk_read(wmem_packet_scope(), tvb, offset);
-    tvbuff_t *tvb_prot = wscbor_require_bstr(wmem_packet_scope(), chunk_prot_bstr);
+    wscbor_chunk_t *chunk_prot_bstr = wscbor_chunk_read(pinfo->pool, tvb, offset);
+    tvbuff_t *tvb_prot = wscbor_require_bstr(pinfo->pool, chunk_prot_bstr);
     proto_item *item_prot_bstr = proto_tree_add_cbor_bstr(tree, hf_hdr_prot_bstr, pinfo, tvb, chunk_prot_bstr);
     if (tvb_prot) {
         proto_tree *tree_prot = proto_item_add_subtree(item_prot_bstr, ett_prot_bstr);
@@ -504,25 +521,25 @@ static void dissect_headers(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 /** Common behavior for payload.
  */
 static void dissect_payload(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gint *offset) {
-    wscbor_chunk_t *chunk = wscbor_chunk_read(wmem_packet_scope(), tvb, offset);
+    wscbor_chunk_t *chunk = wscbor_chunk_read(pinfo->pool, tvb, offset);
     if (chunk->type_major == CBOR_TYPE_FLOAT_CTRL) {
         proto_tree_add_cbor_ctrl(tree, hf_payload_null, pinfo, tvb, chunk);
     }
     else {
-        wscbor_require_bstr(wmem_packet_scope(), chunk);
+        wscbor_require_bstr(pinfo->pool, chunk);
         proto_tree_add_cbor_bstr(tree, hf_payload_bstr, pinfo, tvb, chunk);
     }
 }
 static void dissect_signature(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gint *offset) {
-    wscbor_chunk_t *chunk = wscbor_chunk_read(wmem_packet_scope(), tvb, offset);
-    wscbor_require_bstr(wmem_packet_scope(), chunk);
+    wscbor_chunk_t *chunk = wscbor_chunk_read(pinfo->pool, tvb, offset);
+    wscbor_require_bstr(pinfo->pool, chunk);
     proto_tree_add_cbor_bstr(tree, hf_signature, pinfo, tvb, chunk);
 }
 static void dissect_cose_signature(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gint *offset) {
-    wscbor_chunk_t *chunk_sig = wscbor_chunk_read(wmem_packet_scope(), tvb, offset);
+    wscbor_chunk_t *chunk_sig = wscbor_chunk_read(pinfo->pool, tvb, offset);
     wscbor_require_array_size(chunk_sig, 3, 3);
     proto_item *item_sig = proto_tree_add_cbor_container(tree, hf_cose_signature, pinfo, tvb, chunk_sig);
-    if (!wscbor_skip_if_errors(wmem_packet_scope(), tvb, offset, chunk_sig)) {
+    if (!wscbor_skip_if_errors(pinfo->pool, tvb, offset, chunk_sig)) {
         proto_tree *tree_sig = proto_item_add_subtree(item_sig, ett_sig);
 
         dissect_headers(tvb, pinfo, tree_sig, offset);
@@ -531,21 +548,21 @@ static void dissect_cose_signature(tvbuff_t *tvb, packet_info *pinfo, proto_tree
     proto_item_set_len(item_sig, *offset - chunk_sig->start);
 }
 static void dissect_ciphertext(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gint *offset) {
-    wscbor_chunk_t *chunk = wscbor_chunk_read(wmem_packet_scope(), tvb, offset);
+    wscbor_chunk_t *chunk = wscbor_chunk_read(pinfo->pool, tvb, offset);
     if (chunk->type_major == CBOR_TYPE_FLOAT_CTRL) {
         proto_tree_add_cbor_ctrl(tree, hf_ciphertext_null, pinfo, tvb, chunk);
     }
     else {
-        wscbor_require_bstr(wmem_packet_scope(), chunk);
+        wscbor_require_bstr(pinfo->pool, chunk);
         proto_tree_add_cbor_bstr(tree, hf_ciphertext_bstr, pinfo, tvb, chunk);
     }
 }
 static void dissect_cose_recipient(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gint *offset);
 static void dissect_cose_recipient_list(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gint *offset) {
-    wscbor_chunk_t *chunk_list = wscbor_chunk_read(wmem_packet_scope(), tvb, offset);
+    wscbor_chunk_t *chunk_list = wscbor_chunk_read(pinfo->pool, tvb, offset);
     wscbor_require_array(chunk_list);
     proto_item *item_list = proto_tree_add_cbor_container(tree, hf_cose_recipient_list, pinfo, tvb, chunk_list);
-    if (!wscbor_skip_if_errors(wmem_packet_scope(), tvb, offset, chunk_list)) {
+    if (!wscbor_skip_if_errors(pinfo->pool, tvb, offset, chunk_list)) {
         proto_tree *tree_recip_list = proto_item_add_subtree(item_list, ett_recip_list);
 
         for (guint64 ix = 0; ix < chunk_list->head_value; ++ix) {
@@ -555,10 +572,10 @@ static void dissect_cose_recipient_list(tvbuff_t *tvb, packet_info *pinfo, proto
     proto_item_set_len(item_list, *offset - chunk_list->start);
 }
 static void dissect_cose_recipient(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gint *offset) {
-    wscbor_chunk_t *chunk_recip = wscbor_chunk_read(wmem_packet_scope(), tvb, offset);
+    wscbor_chunk_t *chunk_recip = wscbor_chunk_read(pinfo->pool, tvb, offset);
     wscbor_require_array_size(chunk_recip, 3, 4);
     proto_item *item_recip = proto_tree_add_cbor_container(tree, hf_cose_recipient, pinfo, tvb, chunk_recip);
-    if (!wscbor_skip_if_errors(wmem_packet_scope(), tvb, offset, chunk_recip)) {
+    if (!wscbor_skip_if_errors(pinfo->pool, tvb, offset, chunk_recip)) {
         proto_tree *tree_recip = proto_item_add_subtree(item_recip, ett_recip);
 
         dissect_headers(tvb, pinfo, tree_recip, offset);
@@ -571,8 +588,8 @@ static void dissect_cose_recipient(tvbuff_t *tvb, packet_info *pinfo, proto_tree
 
 }
 static void dissect_tag(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gint *offset) {
-    wscbor_chunk_t *chunk = wscbor_chunk_read(wmem_packet_scope(), tvb, offset);
-    wscbor_require_bstr(wmem_packet_scope(), chunk);
+    wscbor_chunk_t *chunk = wscbor_chunk_read(pinfo->pool, tvb, offset);
+    wscbor_require_bstr(pinfo->pool, chunk);
     proto_tree_add_cbor_bstr(tree, hf_tag, pinfo, tvb, chunk);
 }
 
@@ -580,21 +597,21 @@ static void dissect_tag(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gin
 static int dissect_cose_sign(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk_msg = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
+    wscbor_chunk_t *chunk_msg = wscbor_chunk_read(pinfo->pool, tvb, &offset);
     wscbor_require_array_size(chunk_msg, 4, 4);
     proto_item *item_msg = proto_tree_add_cbor_container(tree, proto_cose, pinfo, tvb, chunk_msg);
     proto_item_append_text(item_msg, ": COSE_Sign");
-    if (!wscbor_skip_if_errors(wmem_packet_scope(), tvb, &offset, chunk_msg)) {
+    if (!wscbor_skip_if_errors(pinfo->pool, tvb, &offset, chunk_msg)) {
         proto_tree *tree_msg = proto_item_add_subtree(item_msg, ett_msg);
         dissect_msg_tag(tvb, pinfo, tree_msg, chunk_msg, data);
 
         dissect_headers(tvb, pinfo, tree_msg, &offset);
         dissect_payload(tvb, pinfo, tree_msg, &offset);
 
-        wscbor_chunk_t *chunk_sig_list = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
+        wscbor_chunk_t *chunk_sig_list = wscbor_chunk_read(pinfo->pool, tvb, &offset);
         wscbor_require_array(chunk_sig_list);
         proto_item *item_sig_list = proto_tree_add_cbor_container(tree_msg, hf_cose_signature_list, pinfo, tvb, chunk_sig_list);
-        if (!wscbor_skip_if_errors(wmem_packet_scope(), tvb, &offset, chunk_sig_list)) {
+        if (!wscbor_skip_if_errors(pinfo->pool, tvb, &offset, chunk_sig_list)) {
             proto_tree *tree_sig_list = proto_item_add_subtree(item_sig_list, ett_sig_list);
 
             for (guint64 ix = 0; ix < chunk_sig_list->head_value; ++ix) {
@@ -613,11 +630,11 @@ static int dissect_cose_sign_media_type(tvbuff_t *tvb, packet_info *pinfo, proto
 static int dissect_cose_sign1(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk_msg = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
+    wscbor_chunk_t *chunk_msg = wscbor_chunk_read(pinfo->pool, tvb, &offset);
     wscbor_require_array_size(chunk_msg, 4, 4);
     proto_item *item_msg = proto_tree_add_cbor_container(tree, proto_cose, pinfo, tvb, chunk_msg);
     proto_item_append_text(item_msg, ": COSE_Sign1");
-    if (!wscbor_skip_if_errors(wmem_packet_scope(), tvb, &offset, chunk_msg)) {
+    if (!wscbor_skip_if_errors(pinfo->pool, tvb, &offset, chunk_msg)) {
         proto_tree *tree_msg = proto_item_add_subtree(item_msg, ett_msg);
         dissect_msg_tag(tvb, pinfo, tree_msg, chunk_msg, data);
 
@@ -635,11 +652,11 @@ static int dissect_cose_sign1_media_type(tvbuff_t *tvb, packet_info *pinfo, prot
 static int dissect_cose_encrypt(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk_msg = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
+    wscbor_chunk_t *chunk_msg = wscbor_chunk_read(pinfo->pool, tvb, &offset);
     wscbor_require_array_size(chunk_msg, 4, 4);
     proto_item *item_msg = proto_tree_add_cbor_container(tree, proto_cose, pinfo, tvb, chunk_msg);
     proto_item_append_text(item_msg, ": COSE_Encrypt");
-    if (!wscbor_skip_if_errors(wmem_packet_scope(), tvb, &offset, chunk_msg)) {
+    if (!wscbor_skip_if_errors(pinfo->pool, tvb, &offset, chunk_msg)) {
         proto_tree *tree_msg = proto_item_add_subtree(item_msg, ett_msg);
         dissect_msg_tag(tvb, pinfo, tree_msg, chunk_msg, data);
 
@@ -657,11 +674,11 @@ static int dissect_cose_encrypt_media_type(tvbuff_t *tvb, packet_info *pinfo, pr
 static int dissect_cose_encrypt0(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk_msg = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
+    wscbor_chunk_t *chunk_msg = wscbor_chunk_read(pinfo->pool, tvb, &offset);
     wscbor_require_array_size(chunk_msg, 3, 3);
     proto_item *item_msg = proto_tree_add_cbor_container(tree, proto_cose, pinfo, tvb, chunk_msg);
     proto_item_append_text(item_msg, ": COSE_Encrypt0");
-    if (!wscbor_skip_if_errors(wmem_packet_scope(), tvb, &offset, chunk_msg)) {
+    if (!wscbor_skip_if_errors(pinfo->pool, tvb, &offset, chunk_msg)) {
         proto_tree *tree_msg = proto_item_add_subtree(item_msg, ett_msg);
         dissect_msg_tag(tvb, pinfo, tree_msg, chunk_msg, data);
 
@@ -678,11 +695,11 @@ static int dissect_cose_encrypt0_media_type(tvbuff_t *tvb, packet_info *pinfo, p
 static int dissect_cose_mac(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk_msg = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
+    wscbor_chunk_t *chunk_msg = wscbor_chunk_read(pinfo->pool, tvb, &offset);
     wscbor_require_array_size(chunk_msg, 5, 5);
     proto_item *item_msg = proto_tree_add_cbor_container(tree, proto_cose, pinfo, tvb, chunk_msg);
     proto_item_append_text(item_msg, ": COSE_Mac");
-    if (!wscbor_skip_if_errors(wmem_packet_scope(), tvb, &offset, chunk_msg)) {
+    if (!wscbor_skip_if_errors(pinfo->pool, tvb, &offset, chunk_msg)) {
         proto_tree *tree_msg = proto_item_add_subtree(item_msg, ett_msg);
         dissect_msg_tag(tvb, pinfo, tree_msg, chunk_msg, data);
 
@@ -701,11 +718,11 @@ static int dissect_cose_mac_media_type(tvbuff_t *tvb, packet_info *pinfo, proto_
 static int dissect_cose_mac0(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk_msg = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
+    wscbor_chunk_t *chunk_msg = wscbor_chunk_read(pinfo->pool, tvb, &offset);
     wscbor_require_array_size(chunk_msg, 4, 4);
     proto_item *item_msg = proto_tree_add_cbor_container(tree, proto_cose, pinfo, tvb, chunk_msg);
     proto_item_append_text(item_msg, ": COSE_Mac0");
-    if (!wscbor_skip_if_errors(wmem_packet_scope(), tvb, &offset, chunk_msg)) {
+    if (!wscbor_skip_if_errors(pinfo->pool, tvb, &offset, chunk_msg)) {
         proto_tree *tree_msg = proto_item_add_subtree(item_msg, ett_msg);
         dissect_msg_tag(tvb, pinfo, tree_msg, chunk_msg, data);
 
@@ -727,7 +744,7 @@ static int dissect_cose_msg_tagged(tvbuff_t *tvb, packet_info *pinfo, proto_tree
     gint offset = 0;
 
     // All messages have the same base structure, attempt all tags present
-    wscbor_chunk_t *chunk = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
+    wscbor_chunk_t *chunk = wscbor_chunk_read(pinfo->pool, tvb, &offset);
     for (wmem_list_frame_t *it = wmem_list_head(chunk->tags); it;
             it = wmem_list_frame_next(it)) {
         wscbor_tag_t *tag = (wscbor_tag_t *) wmem_list_frame_data(it);
@@ -744,17 +761,17 @@ static int dissect_cose_msg_tagged(tvbuff_t *tvb, packet_info *pinfo, proto_tree
     }
 
     ws_warning("main dissector did not match any known tag");
-    proto_item *item_msg = proto_tree_add_item(tree, proto_cose, tvb, 0, -1, 0);
+    proto_item *item_msg = proto_tree_add_item(tree, proto_cose, tvb, 0, -1, ENC_NA);
     expert_add_info(pinfo, item_msg, &ei_invalid_tag);
     return -1;
 }
 
 static void dissect_value_alg(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gint *offset, GVariant **value) {
-    wscbor_chunk_t *chunk = wscbor_chunk_read(wmem_packet_scope(), tvb, offset);
+    wscbor_chunk_t *chunk = wscbor_chunk_read(pinfo->pool, tvb, offset);
     switch (chunk->type_major) {
         case CBOR_TYPE_UINT:
         case CBOR_TYPE_NEGINT: {
-            gint64 *val = wscbor_require_int64(wmem_packet_scope(), chunk);
+            gint64 *val = wscbor_require_int64(pinfo->pool, chunk);
             proto_tree_add_cbor_int64(tree, hf_hdr_alg_int, pinfo, tvb, chunk, val);
             if (value && val) {
                 *value = g_variant_new_int64(*val);
@@ -762,7 +779,7 @@ static void dissect_value_alg(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tre
             break;
         }
         case CBOR_TYPE_STRING: {
-            const char *val = wscbor_require_tstr(wmem_packet_scope(), chunk);
+            const char *val = wscbor_require_tstr(pinfo->pool, chunk);
             proto_tree_add_cbor_tstr(tree, hf_hdr_alg_tstr, pinfo, tvb, chunk);
             if (value && val) {
                 *value = g_variant_new_string(val);
@@ -777,8 +794,8 @@ static void dissect_value_alg(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tre
 static int dissect_header_salt(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
-    wscbor_require_bstr(wmem_packet_scope(), chunk);
+    wscbor_chunk_t *chunk = wscbor_chunk_read(pinfo->pool, tvb, &offset);
+    wscbor_require_bstr(pinfo->pool, chunk);
     proto_tree_add_cbor_bstr(tree, hf_hdr_salt, pinfo, tvb, chunk);
 
     return offset;
@@ -816,18 +833,18 @@ static int dissect_header_alg(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tre
 static int dissect_header_crit(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk_list = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
+    wscbor_chunk_t *chunk_list = wscbor_chunk_read(pinfo->pool, tvb, &offset);
     wscbor_require_array(chunk_list);
     proto_item *item_list = proto_tree_add_cbor_container(tree, hf_hdr_crit_list, pinfo, tvb, chunk_list);
-    if (!wscbor_skip_if_errors(wmem_packet_scope(), tvb, &offset, chunk_list)) {
+    if (!wscbor_skip_if_errors(pinfo->pool, tvb, &offset, chunk_list)) {
         proto_tree *tree_list = proto_item_add_subtree(item_list, ett_hdr_crit_list);
 
         for (guint64 ix = 0; ix < chunk_list->head_value; ++ix) {
-            wscbor_chunk_t *chunk_label = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
+            wscbor_chunk_t *chunk_label = wscbor_chunk_read(pinfo->pool, tvb, &offset);
             switch (chunk_label->type_major) {
                 case CBOR_TYPE_UINT:
                 case CBOR_TYPE_NEGINT: {
-                    gint64 *label = wscbor_require_int64(wmem_packet_scope(), chunk_label);
+                    gint64 *label = wscbor_require_int64(pinfo->pool, chunk_label);
                     proto_tree_add_cbor_int64(tree_list, hf_hdr_label_int, pinfo, tvb, chunk_label, label);
                     break;
                 }
@@ -848,10 +865,10 @@ static int dissect_header_crit(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tr
 static int dissect_header_ctype(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
+    wscbor_chunk_t *chunk = wscbor_chunk_read(pinfo->pool, tvb, &offset);
     switch (chunk->type_major) {
         case CBOR_TYPE_UINT: {
-            guint64 *val = wscbor_require_uint64(wmem_packet_scope(), chunk);
+            guint64 *val = wscbor_require_uint64(pinfo->pool, chunk);
             proto_tree_add_cbor_uint64(tree, hf_hdr_ctype_uint, pinfo, tvb, chunk, val);
             break;
         }
@@ -869,8 +886,8 @@ static int dissect_header_ctype(tvbuff_t *tvb, packet_info *pinfo, proto_tree *t
 static int dissect_header_kid(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
-    wscbor_require_bstr(wmem_packet_scope(), chunk);
+    wscbor_chunk_t *chunk = wscbor_chunk_read(pinfo->pool, tvb, &offset);
+    wscbor_require_bstr(pinfo->pool, chunk);
     proto_tree_add_cbor_bstr(tree, hf_hdr_kid, pinfo, tvb, chunk);
 
     return offset;
@@ -879,8 +896,8 @@ static int dissect_header_kid(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tre
 static int dissect_header_iv(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
-    wscbor_require_bstr(wmem_packet_scope(), chunk);
+    wscbor_chunk_t *chunk = wscbor_chunk_read(pinfo->pool, tvb, &offset);
+    wscbor_require_bstr(pinfo->pool, chunk);
     proto_tree_add_cbor_bstr(tree, hf_hdr_iv, pinfo, tvb, chunk);
 
     return offset;
@@ -889,20 +906,20 @@ static int dissect_header_iv(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree
 static int dissect_header_piv(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
-    wscbor_require_bstr(wmem_packet_scope(), chunk);
+    wscbor_chunk_t *chunk = wscbor_chunk_read(pinfo->pool, tvb, &offset);
+    wscbor_require_bstr(pinfo->pool, chunk);
     proto_tree_add_cbor_bstr(tree, hf_hdr_piv, pinfo, tvb, chunk);
 
     return offset;
 }
 
 static void dissect_value_x5cert(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gint *offset) {
-    wscbor_chunk_t *chunk_item = wscbor_chunk_read(wmem_packet_scope(), tvb, offset);
-    tvbuff_t *tvb_item = wscbor_require_bstr(wmem_packet_scope(), chunk_item);
+    wscbor_chunk_t *chunk_item = wscbor_chunk_read(pinfo->pool, tvb, offset);
+    tvbuff_t *tvb_item = wscbor_require_bstr(pinfo->pool, chunk_item);
 
     if (tvb_item) {
         // disallow column text rewrite
-        gchar *info_text = wmem_strdup(wmem_packet_scope(), col_get_text(pinfo->cinfo, COL_INFO));
+        gchar *info_text = wmem_strdup(pinfo->pool, col_get_text(pinfo->cinfo, COL_INFO));
 
         TRY {
             dissector_try_string(
@@ -925,11 +942,11 @@ static void dissect_value_cosex509(tvbuff_t *tvb, packet_info *pinfo, proto_tree
     proto_item *item_ctr = proto_tree_add_item(tree, hfindex, tvb, 0, -1, ENC_NA);
     proto_tree *tree_ctr = proto_item_add_subtree(item_ctr, ett_hdr_x5cert_list);
 
-    wscbor_chunk_t *chunk_ctr = wscbor_chunk_read(wmem_packet_scope(), tvb, offset);
+    wscbor_chunk_t *chunk_ctr = wscbor_chunk_read(pinfo->pool, tvb, offset);
     switch (chunk_ctr->type_major) {
         case CBOR_TYPE_ARRAY: {
             wscbor_require_array(chunk_ctr);
-            if (!wscbor_skip_if_errors(wmem_packet_scope(), tvb, offset, chunk_ctr)) {
+            if (!wscbor_skip_if_errors(pinfo->pool, tvb, offset, chunk_ctr)) {
                 for (guint64 ix = 0; ix < chunk_ctr->head_value; ++ix) {
                     dissect_value_x5cert(tvb, pinfo, tree_ctr, offset);
                 }
@@ -961,16 +978,16 @@ static int dissect_header_x5chain(tvbuff_t *tvb, packet_info *pinfo, proto_tree
 static int dissect_header_x5t(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk_list = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
+    wscbor_chunk_t *chunk_list = wscbor_chunk_read(pinfo->pool, tvb, &offset);
     wscbor_require_array_size(chunk_list, 2, 2);
     proto_item *item_list = proto_tree_add_cbor_container(tree, hf_hdr_x5t, pinfo, tvb, chunk_list);
-    if (!wscbor_skip_if_errors(wmem_packet_scope(), tvb, &offset, chunk_list)) {
+    if (!wscbor_skip_if_errors(pinfo->pool, tvb, &offset, chunk_list)) {
         proto_tree *tree_list = proto_item_add_subtree(item_list, ett_hdr_x5t_list);
 
         dissect_value_alg(tvb, pinfo, tree_list, &offset, NULL);
 
-        wscbor_chunk_t *chunk_hash = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
-        wscbor_require_bstr(wmem_packet_scope(), chunk_hash);
+        wscbor_chunk_t *chunk_hash = wscbor_chunk_read(pinfo->pool, tvb, &offset);
+        wscbor_require_bstr(pinfo->pool, chunk_hash);
         proto_tree_add_cbor_bstr(tree_list, hf_hdr_x5t_hash, pinfo, tvb, chunk_hash);
 
     }
@@ -981,7 +998,7 @@ static int dissect_header_x5t(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tre
 static int dissect_header_x5u(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
+    wscbor_chunk_t *chunk = wscbor_chunk_read(pinfo->pool, tvb, &offset);
     wscbor_require_major_type(chunk, CBOR_TYPE_STRING);
     proto_tree_add_cbor_tstr(tree, hf_hdr_x5u, pinfo, tvb, chunk);
 
@@ -991,7 +1008,7 @@ static int dissect_header_x5u(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tre
 static int dissect_cose_key(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_) {
     gint offset = 0;
 
-    proto_item *item_msg = proto_tree_add_item(tree, proto_cose, tvb, 0, -1, 0);
+    proto_item *item_msg = proto_tree_add_item(tree, proto_cose, tvb, 0, -1, ENC_NA);
     proto_item_append_text(item_msg, ": COSE_Key");
 
     dissect_value_cose_key(tvb, pinfo, tree, &offset);
@@ -1002,11 +1019,11 @@ static int dissect_cose_key(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 static int dissect_cose_key_set(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk_set = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
+    wscbor_chunk_t *chunk_set = wscbor_chunk_read(pinfo->pool, tvb, &offset);
     wscbor_require_array(chunk_set);
     proto_item *item_set = proto_tree_add_cbor_container(tree, proto_cose, pinfo, tvb, chunk_set);
     proto_item_append_text(item_set, ": COSE_KeySet");
-    if (!wscbor_skip_if_errors(wmem_packet_scope(), tvb, &offset, chunk_set)) {
+    if (!wscbor_skip_if_errors(pinfo->pool, tvb, &offset, chunk_set)) {
         proto_tree *tree_set = proto_item_add_subtree(item_set, ett_key_set);
 
         for (guint64 ix = 0; ix < chunk_set->head_value; ++ix) {
@@ -1027,11 +1044,11 @@ static int dissect_keyparam_kty(tvbuff_t *tvb, packet_info *pinfo, proto_tree *t
     cose_header_context_t *ctx = (cose_header_context_t *)data;
     gint offset = 0;
 
-    wscbor_chunk_t *chunk = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
+    wscbor_chunk_t *chunk = wscbor_chunk_read(pinfo->pool, tvb, &offset);
     switch (chunk->type_major) {
         case CBOR_TYPE_UINT:
         case CBOR_TYPE_NEGINT: {
-            gint64 *val = wscbor_require_int64(wmem_packet_scope(), chunk);
+            gint64 *val = wscbor_require_int64(pinfo->pool, chunk);
             proto_tree_add_cbor_int64(tree, hf_keyparam_kty_int, pinfo, tvb, chunk, val);
             if (val) {
                 ctx->principal = g_variant_new_int64(*val);
@@ -1039,7 +1056,7 @@ static int dissect_keyparam_kty(tvbuff_t *tvb, packet_info *pinfo, proto_tree *t
             break;
         }
         case CBOR_TYPE_STRING: {
-            const char *val = wscbor_require_tstr(wmem_packet_scope(), chunk);
+            const char *val = wscbor_require_tstr(pinfo->pool, chunk);
             proto_tree_add_cbor_tstr(tree, hf_keyparam_kty_tstr, pinfo, tvb, chunk);
             if (val) {
                 ctx->principal = g_variant_new_string(val);
@@ -1056,18 +1073,18 @@ static int dissect_keyparam_kty(tvbuff_t *tvb, packet_info *pinfo, proto_tree *t
 static int dissect_keyparam_keyops(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk_list = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
+    wscbor_chunk_t *chunk_list = wscbor_chunk_read(pinfo->pool, tvb, &offset);
     wscbor_require_array(chunk_list);
     proto_item *item_list = proto_tree_add_cbor_container(tree, hf_keyparam_keyops_list, pinfo, tvb, chunk_list);
-    if (!wscbor_skip_if_errors(wmem_packet_scope(), tvb, &offset, chunk_list)) {
+    if (!wscbor_skip_if_errors(pinfo->pool, tvb, &offset, chunk_list)) {
         proto_tree *tree_list = proto_item_add_subtree(item_list, ett_keyops_list);
 
         for (guint64 ix = 0; ix < chunk_list->head_value; ++ix) {
-            wscbor_chunk_t *chunk = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
+            wscbor_chunk_t *chunk = wscbor_chunk_read(pinfo->pool, tvb, &offset);
             switch (chunk->type_major) {
                 case CBOR_TYPE_UINT:
                 case CBOR_TYPE_NEGINT: {
-                    gint64 *val = wscbor_require_int64(wmem_packet_scope(), chunk);
+                    gint64 *val = wscbor_require_int64(pinfo->pool, chunk);
                     proto_tree_add_cbor_int64(tree_list, hf_keyparam_keyops_int, pinfo, tvb, chunk, val);
                     break;
                 }
@@ -1088,8 +1105,8 @@ static int dissect_keyparam_keyops(tvbuff_t *tvb, packet_info *pinfo, proto_tree
 static int dissect_keyparam_baseiv(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
-    wscbor_require_bstr(wmem_packet_scope(), chunk);
+    wscbor_chunk_t *chunk = wscbor_chunk_read(pinfo->pool, tvb, &offset);
+    wscbor_require_bstr(pinfo->pool, chunk);
     proto_tree_add_cbor_bstr(tree, hf_keyparam_baseiv, pinfo, tvb, chunk);
 
     return offset;
@@ -1098,11 +1115,11 @@ static int dissect_keyparam_baseiv(tvbuff_t *tvb, packet_info *pinfo, proto_tree
 static int dissect_keyparam_crv(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
+    wscbor_chunk_t *chunk = wscbor_chunk_read(pinfo->pool, tvb, &offset);
     switch (chunk->type_major) {
         case CBOR_TYPE_UINT:
         case CBOR_TYPE_NEGINT: {
-            gint64 *val = wscbor_require_int64(wmem_packet_scope(), chunk);
+            gint64 *val = wscbor_require_int64(pinfo->pool, chunk);
             proto_tree_add_cbor_int64(tree, hf_keyparam_crv_int, pinfo, tvb, chunk, val);
             break;
         }
@@ -1120,8 +1137,8 @@ static int dissect_keyparam_crv(tvbuff_t *tvb, packet_info *pinfo, proto_tree *t
 static int dissect_keyparam_xcoord(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
-    wscbor_require_bstr(wmem_packet_scope(), chunk);
+    wscbor_chunk_t *chunk = wscbor_chunk_read(pinfo->pool, tvb, &offset);
+    wscbor_require_bstr(pinfo->pool, chunk);
     proto_tree_add_cbor_bstr(tree, hf_keyparam_xcoord, pinfo, tvb, chunk);
 
     return offset;
@@ -1130,7 +1147,7 @@ static int dissect_keyparam_xcoord(tvbuff_t *tvb, packet_info *pinfo, proto_tree
 static int dissect_keyparam_ycoord(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
+    wscbor_chunk_t *chunk = wscbor_chunk_read(pinfo->pool, tvb, &offset);
     switch (chunk->type_major) {
         case CBOR_TYPE_FLOAT_CTRL: {
             proto_tree_add_item(tree, hf_keyparam_ycoord, tvb, 0, 0, ENC_NA);
@@ -1150,8 +1167,8 @@ static int dissect_keyparam_ycoord(tvbuff_t *tvb, packet_info *pinfo, proto_tree
 static int dissect_keyparam_dcoord(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
-    wscbor_require_bstr(wmem_packet_scope(), chunk);
+    wscbor_chunk_t *chunk = wscbor_chunk_read(pinfo->pool, tvb, &offset);
+    wscbor_require_bstr(pinfo->pool, chunk);
     proto_tree_add_cbor_bstr(tree, hf_keyparam_dcoord, pinfo, tvb, chunk);
 
     return offset;
@@ -1160,8 +1177,8 @@ static int dissect_keyparam_dcoord(tvbuff_t *tvb, packet_info *pinfo, proto_tree
 static int dissect_keyparam_k(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_) {
     gint offset = 0;
 
-    wscbor_chunk_t *chunk = wscbor_chunk_read(wmem_packet_scope(), tvb, &offset);
-    wscbor_require_bstr(wmem_packet_scope(), chunk);
+    wscbor_chunk_t *chunk = wscbor_chunk_read(pinfo->pool, tvb, &offset);
+    wscbor_require_bstr(pinfo->pool, chunk);
     proto_tree_add_cbor_bstr(tree, hf_keyparam_k, pinfo, tvb, chunk);
 
     return offset;
@@ -1227,11 +1244,7 @@ static void cose_reinit(void) {
 
 /// Overall registration of the protocol
 void proto_register_cose(void) {
-    proto_cose = proto_register_protocol(
-        "CBOR Object Signing and Encryption", /* name */
-        proto_name_cose, /* short name */
-        "cose" /* abbrev */
-    );
+    proto_cose = proto_register_protocol("CBOR Object Signing and Encryption", proto_name_cose, "cose");
     register_init_routine(&cose_init);
     register_cleanup_routine(&cose_cleanup);
 
@@ -1250,7 +1263,7 @@ void proto_register_cose(void) {
 
     handle_cose_msg_hdr = register_dissector("cose.msg.headers", dissect_cose_msg_header_map, proto_cose);
 
-    table_cose_msg_tag = register_custom_dissector_table("cose.msgtag", "COSE Message Tag", proto_cose, g_int64_hash, g_int64_equal);
+    table_cose_msg_tag = register_custom_dissector_table("cose.msgtag", "COSE Message Tag", proto_cose, g_int64_hash, g_int64_equal, g_free);
     handle_cose_msg_tagged = register_dissector("cose", dissect_cose_msg_tagged, proto_cose_params);
     handle_cose_sign = register_dissector("cose_sign", dissect_cose_sign, proto_cose);
     handle_cose_sign_media_type = register_dissector("cose_sign_media_type", dissect_cose_sign_media_type, proto_cose);
@@ -1265,12 +1278,12 @@ void proto_register_cose(void) {
     handle_cose_mac0 = register_dissector("cose_mac0", dissect_cose_mac0, proto_cose);
     handle_cose_mac0_media_type = register_dissector("cose_mac0_media_type", dissect_cose_mac0_media_type, proto_cose);
 
-    table_header = register_custom_dissector_table("cose.header", "COSE Header Parameter", proto_cose, cose_param_key_hash, cose_param_key_equal);
+    table_header = register_custom_dissector_table("cose.header", "COSE Header Parameter", proto_cose, cose_param_key_hash, cose_param_key_equal, cose_param_key_free);
 
     handle_cose_key = register_dissector("cose_key", dissect_cose_key, proto_cose);
     handle_cose_key_set = register_dissector("cose_key_set", dissect_cose_key_set, proto_cose);
 
-    table_keyparam = register_custom_dissector_table("cose.keyparam", "COSE Key Parameter", proto_cose, cose_param_key_hash, cose_param_key_equal);
+    table_keyparam = register_custom_dissector_table("cose.keyparam", "COSE Key Parameter", proto_cose, cose_param_key_hash, cose_param_key_equal, cose_param_key_free);
 
     module_t *module_cose = prefs_register_protocol(proto_cose, cose_reinit);
     (void)module_cose;
@@ -1321,6 +1334,7 @@ void proto_reg_handoff_cose(void) {
         register_keyparam_dissector(dissect_keyparam_crv, kty, g_variant_new_int64(-1), "crv");
         register_keyparam_dissector(dissect_keyparam_xcoord, kty, g_variant_new_int64(-2), "x");
         register_keyparam_dissector(dissect_keyparam_dcoord, kty, g_variant_new_int64(-3), "d");
+        g_variant_unref(kty);
     }
     {
         GVariant *kty = g_variant_new_int64(2);
@@ -1328,10 +1342,12 @@ void proto_reg_handoff_cose(void) {
         register_keyparam_dissector(dissect_keyparam_xcoord, kty, g_variant_new_int64(-2), "x");
         register_keyparam_dissector(dissect_keyparam_ycoord, kty, g_variant_new_int64(-3), "y");
         register_keyparam_dissector(dissect_keyparam_dcoord, kty, g_variant_new_int64(-4), "d");
+        g_variant_unref(kty);
     }
     {
         GVariant *kty = g_variant_new_int64(4);
         register_keyparam_dissector(dissect_keyparam_k, kty, g_variant_new_int64(-1), "k");
+        g_variant_unref(kty);
     }
 
     cose_reinit();
