@@ -18,6 +18,8 @@
 void proto_register_dpnet(void);
 void proto_reg_handoff_dpnet(void);
 
+static dissector_handle_t dpnet_handle;
+
 #define DPNET_PORT 6073
 
 static int proto_dpnet = -1;
@@ -130,13 +132,13 @@ static gint ett_dpnet_desc_flags = -1;
 #define PACKET_SIGNING_FAST                  0x01
 #define PACKET_SIGNING_FULL                  0x02
 
-#define SESSION_CLIENT_SERVER                0x00000001
-#define SESSION_MIGRATE_HOST                 0x00000004
-#define SESSION_NODPNSVR                     0x00000040
-#define SESSION_REQUIREPASSWORD              0x00000080
-#define SESSION_NOENUMS                      0x00000100
-#define SESSION_FAST_SIGNED                  0x00000200
-#define SESSION_FULL_SIGNED                  0x00000400
+#define SESSION_CLIENT_SERVER                0x0001
+#define SESSION_MIGRATE_HOST                 0x0004
+#define SESSION_NODPNSVR                     0x0040
+#define SESSION_REQUIREPASSWORD              0x0080
+#define SESSION_NOENUMS                      0x0100
+#define SESSION_FAST_SIGNED                  0x0200
+#define SESSION_FULL_SIGNED                  0x0400
 
 static const value_string packetenumttypes[] = {
     { 1, "Application GUID" },
@@ -541,13 +543,13 @@ proto_register_dpnet(void)
         },
         { &hf_dpnet_session_offset,
             { "Session Offset", "dpnet.session_offset",
-            FT_UINT16, BASE_DEC,
+            FT_UINT32, BASE_DEC,
             NULL, 0,
             NULL, HFILL }
         },
         { &hf_dpnet_session_size,
             { "Session Size", "dpnet.session_size",
-            FT_UINT16, BASE_DEC,
+            FT_UINT32, BASE_DEC,
             NULL, 0,
             NULL, HFILL }
         },
@@ -583,13 +585,13 @@ proto_register_dpnet(void)
         },
         { &hf_dpnet_application_offset,
             { "Application Offset", "dpnet.application_offset",
-            FT_UINT16, BASE_DEC,
+            FT_UINT32, BASE_DEC,
             NULL, 0,
             NULL, HFILL }
         },
         { &hf_dpnet_application_size,
             { "Application Size", "dpnet.application_size",
-            FT_UINT16, BASE_DEC,
+            FT_UINT32, BASE_DEC,
             NULL, 0,
             NULL, HFILL }
         },
@@ -847,14 +849,13 @@ proto_register_dpnet(void)
 
     proto_register_field_array(proto_dpnet, hf, array_length(hf));
     proto_register_subtree_array(ett, array_length(ett));
+
+    dpnet_handle = register_dissector("dpnet", dissect_dpnet, proto_dpnet);
 }
 
 void
 proto_reg_handoff_dpnet(void)
 {
-    static dissector_handle_t dpnet_handle;
-
-    dpnet_handle = create_dissector_handle(dissect_dpnet, proto_dpnet);
     dissector_add_uint("udp.port", DPNET_PORT, dpnet_handle);
 }
 
