@@ -63,19 +63,14 @@
 #include <epan/prefs.h>
 #include <epan/strutil.h>
 #include <epan/proto_data.h>
-#include <epan/dissectors/packet-http2.h>
-
 #include "packet-http.h"
+#include "packet-http2.h"
+#include "packet-media-type.h"
+
 #include "wsutil/pint.h"
 
 #define GRPC_MESSAGE_HEAD_LEN 5
 
-/* http2 standard headers */
-#define HTTP2_HEADER_PATH ":path"
-#define HTTP2_HEADER_CONTENT_TYPE "content-type"
-/* http2 for grpc */
-#define HTTP2_HEADER_GRPC_ENCODING "grpc-encoding"
-
 /* calculate the size of a bytes after decoding as base64 */
 #define BASE64_ENCODE_SIZE(len)  ((len) / 3 * 4 + ((len) % 3 == 0 ? 0 : 4))
 
@@ -404,11 +399,10 @@ static int
 dissect_grpc(tvbuff_t* tvb, packet_info* pinfo, proto_tree* tree, void* data)
 {
     int ret;
-    http_conv_t* http_conv;
+    http_req_res_t* curr_req_res;
     tvbuff_t* real_data_tvb;
     grpc_context_info_t grpc_ctx = { 0 };
-    conversation_t* conv = find_or_create_conversation(pinfo);
-    http_message_info_t* http_msg_info = (http_message_info_t*)data;
+    media_content_info_t* content_info = (media_content_info_t*)data;
     gboolean is_grpc_web_text = g_str_has_prefix(pinfo->match_string, "application/grpc-web-text");
 
     if (is_grpc_web_text) {
@@ -429,13 +423,13 @@ dissect_grpc(tvbuff_t* tvb, packet_info* pinfo, proto_tree* tree, void* data)
         grpc_ctx.encoding = http2_get_header_value(pinfo, HTTP2_HEADER_GRPC_ENCODING, FALSE);
     }
     else if (proto_is_frame_protocol(pinfo->layers, "http")) {
-        http_conv = (http_conv_t*)conversation_get_proto_data(conv, proto_http);
-        DISSECTOR_ASSERT_HINT(http_conv && http_msg_info, "Unexpected error: HTTP conversation or HTTP message info not available.");
-        grpc_ctx.is_request = (http_msg_info->type == HTTP_REQUEST);
-        grpc_ctx.path = http_conv->request_uri;
+        curr_req_res = (http_req_res_t*)p_get_proto_data(wmem_file_scope(), pinfo, proto_http, 0);
+        DISSECTOR_ASSERT_HINT(curr_req_res && content_info, "Unexpected error: HTTP request/reply or HTTP message info not available.");
+        grpc_ctx.is_request = (content_info->type == MEDIA_CONTAINER_HTTP_REQUEST);
+        grpc_ctx.path = curr_req_res->request_uri;
         grpc_ctx.content_type = pinfo->match_string; /* only for grpc-web(-text) over http1.1 */
-        if (http_msg_info->data) {
-            grpc_ctx.encoding = (const gchar*)wmem_map_lookup((wmem_map_t *)http_msg_info->data, HTTP2_HEADER_GRPC_ENCODING);
+        if (content_info->data) {
+            grpc_ctx.encoding = (const gchar*)wmem_map_lookup((wmem_map_t *)content_info->data, HTTP2_HEADER_GRPC_ENCODING);
         }
     }
     else {
@@ -539,7 +533,7 @@ proto_register_grpc(void)
     */
     grpc_message_type_subdissector_table =
         register_dissector_table("grpc_message_type",
-            "GRPC message type", proto_grpc, FT_STRING, BASE_NONE);
+            "GRPC message type", proto_grpc, FT_STRING, STRING_CASE_SENSITIVE);
 }
 
 void
