@@ -1760,11 +1760,11 @@ proto_register_iso14443(void)
         },
         { &hf_iso14443_cid_following,
             { "CID following", "iso14443.cid_following", FT_BOOLEAN, 8,
-                TFS(&tfs_true_false), 0x08, NULL, HFILL }
+                NULL, 0x08, NULL, HFILL }
         },
         { &hf_iso14443_nad_following,
             { "NAD following", "iso14443.nad_following", FT_BOOLEAN, 8,
-                TFS(&tfs_true_false), 0x04, NULL, HFILL }
+                NULL, 0x04, NULL, HFILL }
         },
         { &hf_iso14443_nak,
             { "NAK/ACK", "iso14443.nak", FT_BOOLEAN, 8,
@@ -1880,8 +1880,7 @@ proto_register_iso14443(void)
 
     expert_module_t* expert_iso14443;
 
-    proto_iso14443 = proto_register_protocol(
-            "ISO/IEC 14443", "ISO 14443", "iso14443");
+    proto_iso14443 = proto_register_protocol("ISO/IEC 14443", "ISO 14443", "iso14443");
     proto_register_field_array(proto_iso14443, hf, array_length(hf));
     proto_register_subtree_array(ett, array_length(ett));
     expert_iso14443 = expert_register_protocol(proto_iso14443);
