@@ -24,11 +24,13 @@
 #include <wsutil/str_util.h>
 
 #include "packet-gsm_a_common.h"
-#include "packet-http.h"
+#include "packet-media-type.h"
 
 void proto_register_lwm2mtlv(void);
 void proto_reg_handoff_lwm2mtlv(void);
 
+static dissector_handle_t lwm2mtlv_handle;
+
 static int proto_lwm2mtlv = -1;
 
 static int hf_lwm2mtlv_object_name               = -1;
@@ -316,7 +318,7 @@ static guint num_lwm2m_uat_object_names;
 static lwm2m_resource_t *lwm2m_uat_resources;
 static guint num_lwm2m_uat_resources;
 
-static gboolean lwm2m_object_name_update_cb(void *record, char **error)
+static bool lwm2m_object_name_update_cb(void *record, char **error)
 {
 	lwm2m_object_name_t *rec = (lwm2m_object_name_t *)record;
 
@@ -356,7 +358,7 @@ static void lwm2m_object_name_free_cb(void *record)
 UAT_DEC_CB_DEF(object_name, object_id, lwm2m_object_name_t)
 UAT_CSTRING_CB_DEF(object_name, name, lwm2m_object_name_t)
 
-static gboolean lwm2m_resource_update_cb(void *record, char **error)
+static bool lwm2m_resource_update_cb(void *record, char **error)
 {
 	lwm2m_resource_t *rec = (lwm2m_resource_t *)record;
 	char c;
@@ -939,12 +941,12 @@ dissect_lwm2mtlv(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data
 {
 	proto_tree* lwm2mtlv_tree;
 	proto_item* lwm2mtlv_item;
-	http_message_info_t *message_info = (http_message_info_t *) data;
+	media_content_info_t *content_info = (media_content_info_t *) data;
 	gint object_id = -1;
 	gint resource_id = -1;
 
-	if (message_info && message_info->media_str && message_info->media_str[0]) {
-		gchar **ids = wmem_strsplit(pinfo->pool, message_info->media_str, "/", 5);
+	if (content_info && content_info->media_str && content_info->media_str[0]) {
+		gchar **ids = wmem_strsplit(pinfo->pool, content_info->media_str, "/", 5);
 
 		/* URI path is defined as:
 		 *  ids[1] = Object ID
@@ -1177,7 +1179,7 @@ void proto_register_lwm2mtlv(void)
 	proto_register_field_array(proto_lwm2mtlv, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
 
-	register_dissector("lwm2mtlv", dissect_lwm2mtlv, proto_lwm2mtlv);
+	lwm2mtlv_handle = register_dissector("lwm2mtlv", dissect_lwm2mtlv, proto_lwm2mtlv);
 
 	/* Register the dissector shutdown function */
 	register_shutdown_routine(lwm2m_shutdown_routine);
@@ -1200,9 +1202,6 @@ void proto_register_lwm2mtlv(void)
 void
 proto_reg_handoff_lwm2mtlv(void)
 {
-	static dissector_handle_t lwm2mtlv_handle;
-
-	lwm2mtlv_handle = create_dissector_handle(dissect_lwm2mtlv, proto_lwm2mtlv);
 	dissector_add_string("media_type", "application/vnd.oma.lwm2m+tlv", lwm2mtlv_handle);
 }
 
