@@ -24,6 +24,9 @@ void proto_register_pppoes(void);
 void proto_register_pppoe(void);
 void proto_reg_handoff_pppoes(void);
 
+static dissector_handle_t pppoed_handle;
+static dissector_handle_t pppoes_handle;
+
 static int proto_pppoed = -1;
 
 /* Common to session and discovery protocols */
@@ -194,9 +197,9 @@ static gboolean global_pppoe_show_tags_and_lengths = FALSE;
 #define PPPOE_TAG_VSPEC_DSLF_ACCESS_LOOP_ENCAP_ENCAPS_2_ETH_OVER_AAL5_NULL_WITH_FCS    0x07
 #define PPPOE_TAG_VSPEC_DSLF_ACCESS_LOOP_ENCAP_ENCAPS_2_ETH_OVER_AAL5_NULL_WITHOUT_FCS 0x08
 
-#define PPPOE_CDR_MASK        0x06
-#define PPPOE_MDR_MASK        0x18
-#define PPPOE_RCV_ONLY_MASK   0x01
+#define PPPOE_CDR_MASK        0x0006
+#define PPPOE_MDR_MASK        0x0018
+#define PPPOE_RCV_ONLY_MASK   0x0001
 
 #define PPPOE_SCALE_KBPS      0x00
 #define PPPOE_SCALE_MBPS      0x01
@@ -1010,6 +1013,9 @@ void proto_register_pppoed(void)
 	proto_register_subtree_array(ett, array_length(ett));
 	proto_register_field_array(proto_pppoed, hf, array_length(hf));
 
+	/* Register dissector handle */
+	pppoed_handle = register_dissector("pppoed", dissect_pppoed, proto_pppoed);
+
 	/* Preference setting */
 	pppoed_module = prefs_register_protocol(proto_pppoed, NULL);
 	prefs_register_bool_preference(pppoed_module, "show_tags_and_lengths",
@@ -1020,9 +1026,6 @@ void proto_register_pppoed(void)
 
 void proto_reg_handoff_pppoed(void)
 {
-	dissector_handle_t pppoed_handle;
-
-	pppoed_handle = create_dissector_handle(dissect_pppoed, proto_pppoed);
 	dissector_add_uint("ethertype", ETHERTYPE_PPPOED, pppoed_handle);
 }
 
@@ -1148,9 +1151,9 @@ static int dissect_pppoes(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, v
 		cp_code = tvb_get_ntohs(tvb, 6);
 		/*
 		 * The session payload length expressly does not include pad bytes
-		 *  when LCP or IPCP are present, so avoid the spurious error message
+		 * when LCP or IPCP or IPv6CP are present, so avoid the spurious error message
 		 */
-		if ((cp_code != PPP_LCP) && (cp_code != PPP_IPCP) &&
+		if ((cp_code != PPP_LCP) && (cp_code != PPP_IPCP) && (cp_code != PPP_IPV6CP) &&
 			(reported_payload_length != actual_payload_length) &&
 			((reported_payload_length + 4) != actual_payload_length)) {
 			proto_item_append_text(ti, " [incorrect, should be %u]",
@@ -1225,6 +1228,8 @@ void proto_register_pppoes(void)
 
 	proto_register_subtree_array(ett, array_length(ett));
 	proto_register_field_array(proto_pppoes, hf, array_length(hf));
+
+	pppoes_handle = register_dissector("pppoes", dissect_pppoes, proto_pppoes);
 }
 
 void proto_register_pppoe(void)
@@ -1282,8 +1287,6 @@ void proto_register_pppoe(void)
 
 void proto_reg_handoff_pppoes(void)
 {
-	dissector_handle_t pppoes_handle  =
-	    create_dissector_handle(dissect_pppoes, proto_pppoes);
 	dissector_add_uint("ethertype", ETHERTYPE_PPPOES, pppoes_handle);
 	dissector_add_uint("wtap_encap", WTAP_ENCAP_PPP_ETHER, pppoes_handle);
 
