@@ -40,6 +40,8 @@ void proto_reg_handoff_udp(void);
 
 static dissector_handle_t udp_handle;
 static dissector_handle_t udplite_handle;
+static capture_dissector_handle_t udp_cap_handle;
+static capture_dissector_handle_t udplite_cap_handle;
 
 static int udp_tap = -1;
 static int udp_follow_tap = -1;
@@ -79,6 +81,7 @@ static expert_field ei_udp_possible_traceroute = EI_INIT;
 static expert_field ei_udp_length_bad = EI_INIT;
 static expert_field ei_udplite_checksum_coverage_bad = EI_INIT;
 static expert_field ei_udp_checksum_zero = EI_INIT;
+static expert_field ei_udp_checksum_partial = EI_INIT;
 static expert_field ei_udp_checksum_bad = EI_INIT;
 static expert_field ei_udp_length_bad_zero = EI_INIT;
 
@@ -347,13 +350,13 @@ udpip_endpoint_packet(void *pit, packet_info *pinfo, epan_dissect_t *edt _U_, co
 }
 
 static gboolean
-udp_filter_valid(packet_info *pinfo)
+udp_filter_valid(packet_info *pinfo, void *user_data _U_)
 {
     return proto_is_frame_protocol(pinfo->layers, "udp");
 }
 
 static gchar*
-udp_build_filter(packet_info *pinfo)
+udp_build_filter(packet_info *pinfo, void *user_data _U_)
 {
     if( pinfo->net_src.type == AT_IPv4 && pinfo->net_dst.type == AT_IPv4 ) {
         /* UDP over IPv4 */
@@ -1126,13 +1129,6 @@ dissect(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, guint32 ip_proto)
                 DISSECTOR_ASSERT_NOT_REACHED();
                 break;
             }
-            SET_CKSUM_VEC_TVB(cksum_vec[3], tvb, offset, udph->uh_sum_cov);
-            computed_cksum = in_cksum(&cksum_vec[0], 4);
-
-            item = proto_tree_add_checksum(udp_tree, tvb, offset + 6, hf_udp_checksum, hf_udp_checksum_status, &ei_udp_checksum_bad,
-                                                                            pinfo, computed_cksum, ENC_BIG_ENDIAN, PROTO_CHECKSUM_VERIFY|PROTO_CHECKSUM_IN_CKSUM);
-            checksum_tree = proto_item_add_subtree(item, ett_udp_checksum);
-
             /*
              * in_cksum() should never return 0xFFFF here, because, to quote
              * RFC 1624 section 3 "Discussion":
@@ -1160,15 +1156,49 @@ dissect(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, guint32 ip_proto)
              * all zero, so the sum won't be 0 (+0), and thus the negation
              * won't be -0, i.e. won't be 0xFFFF.
              */
+            /*
+             * Linux and Windows, at least, when performing Local Checksum
+             * Offload (during Generic Segmentation Offload or at other
+             * times), place the one's complement sum of the pseudo header
+             * in the checksum fields initially, which provides the necessary
+             * correction when a device (or driver) computes the one's
+             * complement checksum of each buffer in the skbuff. Since it is
+             * not inverted, the partial_cksum will never be 0x0000 by the
+             * same argument as above.
+             */
+            uint16_t partial_cksum;
+            SET_CKSUM_VEC_TVB(cksum_vec[3], tvb, offset, udph->uh_sum_cov);
+            computed_cksum = in_cksum_ret_partial(&cksum_vec[0], 4, &partial_cksum);
+            uint16_t shouldbe_cksum = in_cksum_shouldbe(udph->uh_sum, computed_cksum);
+            if (computed_cksum != 0 && udph->uh_sum == g_htons(partial_cksum)) {
+                /* Don't use PROTO_CHECKSUM_IN_CKSUM because we expect the value
+                 * to match what we pass in. */
+                item = proto_tree_add_checksum(udp_tree, tvb, offset + 6, hf_udp_checksum, hf_udp_checksum_status, &ei_udp_checksum_bad,
+                                                                                    pinfo, g_htons(partial_cksum), ENC_BIG_ENDIAN, PROTO_CHECKSUM_VERIFY);
+                proto_item_append_text(item, " (matches partial checksum, not 0x%04x, likely caused by \"UDP checksum offload\")", shouldbe_cksum);
+                expert_add_info(pinfo, item, &ei_udp_checksum_partial);
+                computed_cksum = 0;
+                /* XXX: Find some way hint to QUIC (or other dissectors) that
+                 * GSO is a possibility so that extra effort can be made to
+                 * recover coalesced PDUs? E.g., by searching heuristically
+                 * through the payload for the DCID bytes if they're non-zero?
+                 * Add a new status, e.g. PROTO_CHECKSUM_E_PARTIAL?
+                 */
+            } else {
+                item = proto_tree_add_checksum(udp_tree, tvb, offset + 6, hf_udp_checksum, hf_udp_checksum_status, &ei_udp_checksum_bad,
+                                                                                    pinfo, computed_cksum, ENC_BIG_ENDIAN, PROTO_CHECKSUM_VERIFY|PROTO_CHECKSUM_IN_CKSUM);
+            }
+            checksum_tree = proto_item_add_subtree(item, ett_udp_checksum);
+
             if (computed_cksum != 0) {
-                 proto_item_append_text(item, " (maybe caused by \"UDP checksum offload\"?)");
-                 col_append_str(pinfo->cinfo, COL_INFO, " [UDP CHECKSUM INCORRECT]");
-                 calc_item = proto_tree_add_uint(checksum_tree, hf_udp_checksum_calculated,
-                         tvb, offset + 6, 2, in_cksum_shouldbe(udph->uh_sum, computed_cksum));
+                proto_item_append_text(item, " (maybe caused by \"UDP checksum offload\"?)");
+                col_append_str(pinfo->cinfo, COL_INFO, " [UDP CHECKSUM INCORRECT]");
+                calc_item = proto_tree_add_uint(checksum_tree, hf_udp_checksum_calculated,
+                        tvb, offset + 6, 2, in_cksum_shouldbe(udph->uh_sum, computed_cksum));
             }
             else {
-                 calc_item = proto_tree_add_uint(checksum_tree, hf_udp_checksum_calculated,
-                         tvb, offset + 6, 2, udph->uh_sum);
+                calc_item = proto_tree_add_uint(checksum_tree, hf_udp_checksum_calculated,
+                        tvb, offset + 6, 2, shouldbe_cksum);
             }
             proto_item_set_generated(calc_item);
 
@@ -1397,6 +1427,7 @@ proto_register_udp(void)
         { &ei_udp_length_bad, { "udp.length.bad", PI_MALFORMED, PI_ERROR, "Bad length value", EXPFILL }},
         { &ei_udplite_checksum_coverage_bad, { "udplite.checksum_coverage.bad", PI_MALFORMED, PI_ERROR, "Bad checksum coverage length value", EXPFILL }},
         { &ei_udp_checksum_zero, { "udp.checksum.zero", PI_CHECKSUM, PI_ERROR, "Illegal checksum value (0)", EXPFILL }},
+        { &ei_udp_checksum_partial, { "udp.checksum.partial", PI_CHECKSUM, PI_NOTE, "Partial (pseudo header) checksum (likely caused by \"UDP checksum offload\")", EXPFILL }},
         { &ei_udp_checksum_bad, { "udp.checksum.bad", PI_CHECKSUM, PI_ERROR, "Bad checksum", EXPFILL }},
         { &ei_udp_length_bad_zero, { "udp.length.bad_zero", PI_PROTOCOL, PI_WARN, "Length is zero but payload < 65536", EXPFILL }},
     };
@@ -1415,11 +1446,13 @@ proto_register_udp(void)
     proto_udp = proto_register_protocol("User Datagram Protocol", "UDP", "udp");
     proto_register_field_array(proto_udp, hf_udp, array_length(hf_udp));
     udp_handle = register_dissector("udp", dissect_udp, proto_udp);
+    udp_cap_handle = register_capture_dissector("udp", capture_udp, proto_udp);
     expert_udp = expert_register_protocol(proto_udp);
 
     proto_udplite = proto_register_protocol("Lightweight User Datagram Protocol", "UDP-Lite", "udplite");
     proto_register_field_array(proto_udplite, hf_udplite, array_length(hf_udplite));
-    udplite_handle = create_dissector_handle(dissect_udplite, proto_udplite);
+    udplite_handle = register_dissector("udplite", dissect_udplite, proto_udplite);
+    udplite_cap_handle = register_capture_dissector("udplite", capture_udp, proto_udplite);
 
     proto_register_subtree_array(ett, array_length(ett));
     expert_register_field_array(expert_udp, ei, array_length(ei));
@@ -1474,9 +1507,9 @@ proto_register_udp(void)
 
     register_decode_as(&udp_da);
     register_conversation_table(proto_udp, FALSE, udpip_conversation_packet, udpip_endpoint_packet);
-    register_conversation_filter("udp", "UDP", udp_filter_valid, udp_build_filter);
+    register_conversation_filter("udp", "UDP", udp_filter_valid, udp_build_filter, NULL);
     register_follow_stream(proto_udp, "udp_follow", udp_follow_conv_filter, udp_follow_index_filter, udp_follow_address_filter,
-                        udp_port_to_display, follow_tvb_tap_listener);
+                        udp_port_to_display, follow_tvb_tap_listener, get_udp_stream_count, NULL);
 
     register_init_routine(udp_init);
 }
@@ -1484,15 +1517,12 @@ proto_register_udp(void)
 void
 proto_reg_handoff_udp(void)
 {
-    capture_dissector_handle_t udp_cap_handle;
 
     dissector_add_uint("ip.proto", IP_PROTO_UDP, udp_handle);
     dissector_add_uint("ip.proto", IP_PROTO_UDPLITE, udplite_handle);
 
-    udp_cap_handle = create_capture_dissector_handle(capture_udp, proto_udp);
     capture_dissector_add_uint("ip.proto", IP_PROTO_UDP, udp_cap_handle);
-    udp_cap_handle = create_capture_dissector_handle(capture_udp, proto_udplite);
-    capture_dissector_add_uint("ip.proto", IP_PROTO_UDPLITE, udp_cap_handle);
+    capture_dissector_add_uint("ip.proto", IP_PROTO_UDPLITE, udplite_cap_handle);
 
     udp_tap = register_tap("udp");
     udp_follow_tap = register_tap("udp_follow");
