@@ -34,6 +34,8 @@
 void proto_register_usbip(void);
 void proto_reg_handoff_usbip(void);
 
+static dissector_handle_t usbip_handle;
+
 /* Initialize the protocol and registered fields
  */
 static int proto_usbip = -1;
@@ -773,16 +775,24 @@ get_usbip_message_len(packet_info *pinfo _U_, tvbuff_t *tvb, int offset,
 
         case OP_CMD_SUBMIT: {
             int expected_size = USBIP_HEADER_LEN;
+            int actual_length = tvb_get_ntohl(tvb, offset + 0x18);
+            int dir = tvb_get_ntohl(tvb, offset + 0xc);
+            int n_iso = tvb_get_ntohl(tvb, offset + 0x20);
 
-            if (tvb_get_ntohl(tvb, offset + 0xc) == USBIP_DIR_OUT)
-                expected_size += tvb_get_ntohl(tvb, offset + 0x18);
+            if (dir == USBIP_DIR_OUT)
+                expected_size += actual_length;
+
+            if (n_iso > 0)
+                expected_size += n_iso * 4 * 4;
 
-            expected_size += tvb_get_ntohl(tvb, offset + 0x20) * 4 * 4;
             return expected_size;
         }
 
         case OP_RET_SUBMIT: {
             int expected_size = USBIP_HEADER_LEN;
+            int actual_length = tvb_get_ntohl(tvb, offset + 0x18);
+            int n_iso = tvb_get_ntohl(tvb, offset + 0x20);
+
             usbip_transaction_t *usbip_trans = NULL;
             usbip_conv_info_t *usbip_info = usbip_get_usbip_conv(pinfo);
             guint32 status = tvb_get_ntohl(tvb, offset + 0x14);
@@ -792,11 +802,13 @@ get_usbip_message_len(packet_info *pinfo _U_, tvbuff_t *tvb, int offset,
                     usbip_info->pdus, tvb_get_ntohl(tvb, offset + 4));
 
                 if (usbip_trans && usbip_trans->dir == USBIP_DIR_IN && status == 0)
-                    expected_size += tvb_get_ntohl(tvb, offset + 0x18);
+                    expected_size += actual_length;
             }
 
-            if (status == 0)
-                expected_size += tvb_get_ntohl(tvb, offset + 0x20) * 4 * 4;
+            if (status == 0) {
+                if (n_iso >= 0)
+                    expected_size += n_iso * 4 * 4;
+            }
             else
                 expected_size = tvb_captured_length(tvb);
 
@@ -993,12 +1005,12 @@ proto_register_usbip(void)
 
         {&hf_usbip_direction,
          {"Direction",                     "usbip.endpoint_number.direction",
-          FT_UINT8, BASE_HEX, VALS(usb_endpoint_direction_vals), 0x1,
+          FT_UINT32, BASE_HEX, VALS(usb_endpoint_direction_vals), 0x00000001,
           "USB endpoint direction", HFILL}},
 
         {&hf_usbip_ep,
          {"Endpoint",                      "usbip.endpoint_number",
-          FT_UINT8, BASE_HEX, NULL, 0xf,
+          FT_UINT32, BASE_HEX, NULL, 0x0000000f,
           "USB endpoint number", HFILL}},
 
         {&hf_usbip_transfer_flags,
@@ -1051,6 +1063,8 @@ proto_register_usbip(void)
     proto_register_field_array(proto_usbip, hf, array_length(hf));
     proto_register_subtree_array(ett, array_length(ett));
 
+    usbip_handle = register_dissector("usbip", dissect_usbip, proto_usbip);
+
     expert_usbip = expert_register_protocol(proto_usbip);
     expert_register_field_array(expert_usbip, ei, array_length(ei));
 
@@ -1059,9 +1073,6 @@ proto_register_usbip(void)
 void
 proto_reg_handoff_usbip(void)
 {
-    dissector_handle_t usbip_handle;
-
-    usbip_handle = create_dissector_handle(dissect_usbip, proto_usbip);
     dissector_add_for_decode_as_with_preference("tcp.port", usbip_handle);
 }
 
