@@ -23,6 +23,8 @@
 void proto_register_vsip(void);
 void proto_reg_handoff_vsip(void);
 
+static dissector_handle_t vsip_handle;
+
 static const value_string EVsipMessageType_vals[] =
 {
    { 1, "VSIP Ping Request"},
@@ -929,7 +931,7 @@ static guint32 vsip_PingResp(proto_tree *tree, packet_info *pinfo _U_, tvbuff_t
    proto_tree_add_item(tree, hf_vsip_PingResp_ProductType, tvb, offset, 2, ENC_BIG_ENDIAN);
    offset += 2;
 
-   proto_tree_add_item(tree, hf_vsip_PingResp_Status, tvb, offset, 2, ENC_BIG_ENDIAN);
+   proto_tree_add_item(tree, hf_vsip_PingResp_Status, tvb, offset, 1, ENC_BIG_ENDIAN);
    offset += 1;
 
    len = tvb_get_ntohs(tvb, offset);
@@ -1611,12 +1613,11 @@ static guint32 vsip_ErrorVAResponse(proto_tree *tree, packet_info *pinfo, tvbuff
 static guint32 vsip_dissect_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree  *tree)
 {
     int soffset = offset;
-    guint16 version;
+    guint32 version;
     guint8 type;
     proto_item *ti;
 
-    version = tvb_get_ntohs(tvb, offset);
-    proto_tree_add_item(tree, hf_vsip_Version, tvb, offset, 2, version);
+    proto_tree_add_item_ret_uint(tree, hf_vsip_Version, tvb, offset, 2, ENC_BIG_ENDIAN, &version);
     offset += 2;
 
     type = tvb_get_guint8(tvb, offset);
@@ -1632,7 +1633,7 @@ static guint32 vsip_dissect_pdu(tvbuff_t *tvb, int offset, packet_info *pinfo, p
         proto_tree_add_item(tree, hf_vsip_PacketSize, tvb, offset, 4, ENC_BIG_ENDIAN);
         offset += 4;
     }
-    else if(version == 256)
+    else if (version == 256)
     {
         proto_tree_add_item(tree, hf_vsip_PacketSize, tvb, offset, 2, ENC_BIG_ENDIAN);
         offset += 2;
@@ -2061,13 +2062,11 @@ void proto_register_vsip(void)
 
     proto_register_subtree_array(ett, array_length(ett));
     proto_register_field_array(proto_vsip, hf, array_length(hf));
+    vsip_handle = register_dissector("vsip", dissect_vsip, proto_vsip);
 }
 
 void proto_reg_handoff_vsip(void)
 {
-    dissector_handle_t vsip_handle;
-
-    vsip_handle = create_dissector_handle(dissect_vsip, proto_vsip);
     dissector_add_for_decode_as_with_preference("udp.port", vsip_handle);
     dissector_add_for_decode_as_with_preference("tcp.port", vsip_handle);
 }
