@@ -16,6 +16,7 @@
 #include "strutil.h"
 
 #include <wsutil/str_util.h>
+#include <wsutil/unicode-utils.h>
 #include <epan/proto.h>
 
 #ifdef _WIN32
@@ -24,8 +25,6 @@
 #include <wchar.h>
 #endif
 
-static const char hex[16] = { '0', '1', '2', '3', '4', '5', '6', '7',
-                              '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
 
 /*
  * Given a pointer into a data buffer, and to the end of the buffer,
@@ -124,473 +123,6 @@ get_token_len(const guchar *linep, const guchar *lineend,
     return token_len;
 }
 
-
-#define    INITIAL_FMTBUF_SIZE    128
-
-/*
- * Declare, and initialize, the variables used for an output buffer.
- */
-#define FMTBUF_VARS \
-    gchar *fmtbuf = (gchar*)wmem_alloc(allocator, INITIAL_FMTBUF_SIZE); \
-    guint fmtbuf_len = INITIAL_FMTBUF_SIZE; \
-    guint column = 0
-
-/*
- * Expand the buffer to be large enough to add nbytes bytes, plus a
- * terminating '\0'.
- */
-#define FMTBUF_EXPAND(nbytes) \
-    /* \
-     * Is there enough room for those bytes and also enough room for \
-     * a terminating '\0'? \
-     */ \
-    if (column+(nbytes+1) >= fmtbuf_len) { \
-        /* \
-         * Double the buffer's size if it's not big enough. \
-         * The size of the buffer starts at 128, so doubling its size \
-         * adds at least another 128 bytes, which is more than enough \
-         * for one more character plus a terminating '\0'. \
-         */ \
-        fmtbuf_len *= 2; \
-        fmtbuf = (gchar *)wmem_realloc(allocator, fmtbuf, fmtbuf_len); \
-    }
-
-/*
- * Put a byte into the buffer; space must have been ensured for it.
- */
-#define FMTBUF_PUTCHAR(b) \
-    fmtbuf[column] = (b); \
-    column++
-
-/*
- * Add the one-byte argument, as an octal escape sequence, to the end
- * of the buffer.
- */
-#define FMTBUF_PUTBYTE_OCTAL(b) \
-    FMTBUF_PUTCHAR((((b)>>6)&03) + '0'); \
-    FMTBUF_PUTCHAR((((b)>>3)&07) + '0'); \
-    FMTBUF_PUTCHAR((((b)>>0)&07) + '0')
-
-/*
- * Add the one-byte argument, as a hex escape sequence, to the end
- * of the buffer.
- */
-#define FMTBUF_PUTBYTE_HEX(b) \
-    FMTBUF_PUTCHAR('\\'); \
-    FMTBUF_PUTCHAR('x'); \
-    FMTBUF_PUTCHAR(hex[((b) >> 4) & 0xF]); \
-    FMTBUF_PUTCHAR(hex[((b) >> 0) & 0xF])
-
-/*
- * Put the trailing '\0' at the end of the buffer.
- */
-#define FMTBUF_ENDSTR \
-    fmtbuf[column] = '\0'
-
-/* REPLACEMENT CHARACTER */
-#define UNREPL 0xFFFD
-
-#define UNPOOP 0x1F4A9
-
-static gchar *
-format_text_internal(wmem_allocator_t *allocator,
-                        const guchar *string, size_t len,
-                        gboolean replace_space)
-{
-    FMTBUF_VARS;
-    const guchar *stringend = string + len;
-    guchar c;
-
-    while (string < stringend) {
-        /*
-         * Get the first byte of this character.
-         */
-        c = *string++;
-        if (g_ascii_isprint(c)) {
-            /*
-             * Printable ASCII, so not part of a multi-byte UTF-8 sequence.
-             * Make sure there's enough room for one more byte, and add
-             * the character.
-             */
-            FMTBUF_EXPAND(1);
-            FMTBUF_PUTCHAR(c);
-        } else if (replace_space && g_ascii_isspace(c)) {
-            /*
-             * ASCII, so not part of a multi-byte UTF-8 sequence, but
-             * not printable, but is a space character; show it as a
-             * blank.
-             *
-             * Make sure there's enough room for one more byte, and add
-             * the blank.
-             */
-            FMTBUF_EXPAND(1);
-            FMTBUF_PUTCHAR(' ');
-        } else if (c < 128) {
-            /*
-             * ASCII, so not part of a multi-byte UTF-8 sequence, but not
-             * printable.
-             *
-             * That requires a minimum of 2 bytes, one for the backslash
-             * and one for a letter, so make sure we have enough room
-             * for that, plus a trailing '\0'.
-             */
-            FMTBUF_EXPAND(2);
-            FMTBUF_PUTCHAR('\\');
-            switch (c) {
-
-                case '\a':
-                    FMTBUF_PUTCHAR('a');
-                    break;
-
-                case '\b':
-                    FMTBUF_PUTCHAR('b'); /* BS */
-                    break;
-
-                case '\f':
-                    FMTBUF_PUTCHAR('f'); /* FF */
-                    break;
-
-                case '\n':
-                    FMTBUF_PUTCHAR('n'); /* NL */
-                    break;
-
-                case '\r':
-                    FMTBUF_PUTCHAR('r'); /* CR */
-                    break;
-
-                case '\t':
-                    FMTBUF_PUTCHAR('t'); /* tab */
-                    break;
-
-                case '\v':
-                    FMTBUF_PUTCHAR('v');
-                    break;
-
-                default:
-                    /*
-                     * We've already put the backslash, but this
-                     * will put 3 more characters for the octal
-                     * number; make sure we have enough room for
-                     * that, plus the trailing '\0'.
-                     */
-                    FMTBUF_EXPAND(3);
-                    FMTBUF_PUTBYTE_OCTAL(c);
-                    break;
-            }
-        } else {
-            /*
-             * We've fetched the first byte of a multi-byte UTF-8
-             * sequence into c.
-             */
-            int utf8_len;
-            guchar mask;
-            gunichar uc;
-            guchar first;
-
-            if ((c & 0xe0) == 0xc0) {
-                /* Starts a 2-byte UTF-8 sequence; 1 byte left */
-                utf8_len = 1;
-                mask = 0x1f;
-            } else if ((c & 0xf0) == 0xe0) {
-                /* Starts a 3-byte UTF-8 sequence; 2 bytes left */
-                utf8_len = 2;
-                mask = 0x0f;
-            } else if ((c & 0xf8) == 0xf0) {
-                /* Starts a 4-byte UTF-8 sequence; 3 bytes left */
-                utf8_len = 3;
-                mask = 0x07;
-            } else if ((c & 0xfc) == 0xf8) {
-                /* Starts an old-style 5-byte UTF-8 sequence; 4 bytes left */
-                utf8_len = 4;
-                mask = 0x03;
-            } else if ((c & 0xfe) == 0xfc) {
-                /* Starts an old-style 6-byte UTF-8 sequence; 5 bytes left */
-                utf8_len = 5;
-                mask = 0x01;
-            } else {
-                /* 0xfe or 0xff or a continuation byte - not valid */
-                utf8_len = -1;
-            }
-            if (utf8_len > 0) {
-                /* Try to construct the Unicode character */
-                uc = c & mask;
-                for (int i = 0; i < utf8_len; i++) {
-                    if (string >= stringend) {
-                        /*
-                         * Ran out of octets, so the character is
-                         * incomplete.  Put in a REPLACEMENT CHARACTER
-                         * instead, and then continue the loop, which
-                         * will terminate.
-                         */
-                        uc = UNREPL;
-                        break;
-                    }
-                    c = *string;
-                    if ((c & 0xc0) != 0x80) {
-                        /*
-                         * Not valid UTF-8 continuation character; put in
-                         * a replacement character, and then re-process
-                         * this octet as the beginning of a new character.
-                         */
-                        uc = UNREPL;
-                        break;
-                    }
-                    string++;
-                    uc = (uc << 6) | (c & 0x3f);
-                }
-
-                /*
-                 * If this isn't a valid Unicode character, put in
-                 * a REPLACEMENT CHARACTER.
-                 */
-                if (!g_unichar_validate(uc))
-                    uc = UNREPL;
-            } else {
-                /* 0xfe or 0xff; put it a REPLACEMENT CHARACTER */
-                uc = UNREPL;
-            }
-
-            /*
-             * OK, is it a printable Unicode character?
-             */
-            if (g_unichar_isprint(uc)) {
-                /*
-                 * Yes - put it into the string as UTF-8.
-                 * This means that if it was an overlong
-                 * encoding, this will put out the right
-                 * sized encoding.
-                 */
-                if (uc < 0x80) {
-                    first = 0;
-                    utf8_len = 1;
-                } else if (uc < 0x800) {
-                    first = 0xc0;
-                    utf8_len = 2;
-                } else if (uc < 0x10000) {
-                    first = 0xe0;
-                    utf8_len = 3;
-                } else if (uc < 0x200000) {
-                    first = 0xf0;
-                    utf8_len = 4;
-                } else if (uc < 0x4000000) {
-                    /*
-                     * This should never happen, as Unicode doesn't
-                     * go that high.
-                     */
-                    first = 0xf8;
-                    utf8_len = 5;
-                } else {
-                    /*
-                     * This should never happen, as Unicode doesn't
-                     * go that high.
-                     */
-                    first = 0xfc;
-                    utf8_len = 6;
-                }
-                FMTBUF_EXPAND(utf8_len);
-                for (int i = utf8_len - 1; i > 0; i--) {
-                    fmtbuf[column + i] = (uc & 0x3f) | 0x80;
-                    uc >>= 6;
-                }
-                fmtbuf[column] = uc | first;
-                column += utf8_len;
-            } else if (replace_space && g_unichar_isspace(uc)) {
-                /*
-                 * Not printable, but is a space character; show it
-                 * as a blank.
-                 *
-                 * Make sure there's enough room for one more byte,
-                 * and add the blank.
-                 */
-                FMTBUF_EXPAND(1);
-                FMTBUF_PUTCHAR(' ');
-            } else if (c < 128) {
-                /*
-                 * ASCII, but not printable.
-                 * Yes, this could happen with an overlong encoding.
-                 *
-                 * That requires a minimum of 2 bytes, one for the
-                 * backslash and one for a letter, so make sure we
-                 * have enough room for that, plus a trailing '\0'.
-                 */
-                FMTBUF_EXPAND(2);
-                FMTBUF_PUTCHAR('\\');
-                switch (c) {
-
-                    case '\a':
-                        FMTBUF_PUTCHAR('a');
-                        break;
-
-                    case '\b':
-                        FMTBUF_PUTCHAR('b'); /* BS */
-                        break;
-
-                    case '\f':
-                        FMTBUF_PUTCHAR('f'); /* FF */
-                        break;
-
-                    case '\n':
-                        FMTBUF_PUTCHAR('n'); /* NL */
-                        break;
-
-                    case '\r':
-                        FMTBUF_PUTCHAR('r'); /* CR */
-                        break;
-
-                    case '\t':
-                        FMTBUF_PUTCHAR('t'); /* tab */
-                        break;
-
-                    case '\v':
-                        FMTBUF_PUTCHAR('v');
-                        break;
-
-                    default:
-                        /*
-                         * We've already put the backslash, but this
-                         * will put 3 more characters for the octal
-                         * number; make sure we have enough room for
-                         * that, plus the trailing '\0'.
-                         */
-                        FMTBUF_EXPAND(3);
-                        FMTBUF_PUTBYTE_OCTAL(c);
-                        break;
-                }
-            } else {
-                /*
-                 * Unicode, but not printable, and not ASCII;
-                 * put it out as \uxxxx or \Uxxxxxxxx.
-                 */
-                if (uc <= 0xFFFF) {
-                    FMTBUF_EXPAND(6);
-                    FMTBUF_PUTCHAR('\\');
-                    FMTBUF_PUTCHAR('u');
-                    FMTBUF_PUTCHAR(hex[(uc >> 12) & 0xF]);
-                    FMTBUF_PUTCHAR(hex[(uc >> 8) & 0xF]);
-                    FMTBUF_PUTCHAR(hex[(uc >> 4) & 0xF]);
-                    FMTBUF_PUTCHAR(hex[(uc >> 0) & 0xF]);
-                } else {
-                    FMTBUF_EXPAND(10);
-                    FMTBUF_PUTCHAR('\\');
-                    FMTBUF_PUTCHAR('U');
-                    FMTBUF_PUTCHAR(hex[(uc >> 28) & 0xF]);
-                    FMTBUF_PUTCHAR(hex[(uc >> 24) & 0xF]);
-                    FMTBUF_PUTCHAR(hex[(uc >> 20) & 0xF]);
-                    FMTBUF_PUTCHAR(hex[(uc >> 16) & 0xF]);
-                    FMTBUF_PUTCHAR(hex[(uc >> 12) & 0xF]);
-                    FMTBUF_PUTCHAR(hex[(uc >> 8) & 0xF]);
-                    FMTBUF_PUTCHAR(hex[(uc >> 4) & 0xF]);
-                    FMTBUF_PUTCHAR(hex[(uc >> 0) & 0xF]);
-                }
-            }
-        }
-    }
-
-    FMTBUF_ENDSTR;
-    return fmtbuf;
-}
-
-/*
- * Given a wmem scope, a not-necessarily-null-terminated string,
- * expected to be in UTF-8 but possibly containing invalid sequences
- * (as it may have come from packet data), and the length of the string,
- * generate a valid UTF-8 string from it, allocated in the specified
- * wmem scope, that:
- *
- *   shows printable Unicode characters as themselves;
- *
- *   shows non-printable ASCII characters as C-style escapes (octal
- *   if not one of the standard ones such as LF -> '\n');
- *
- *   shows non-printable Unicode-but-not-ASCII characters as
- *   their universal character names;
- *
- *   shows illegal UTF-8 sequences as a sequence of bytes represented
- *   as C-style hex escapes (XXX: Does not actually do this. Some illegal
- *   sequences, such as overlong encodings, the sequences reserved for
- *   UTF-16 surrogate halves (paired or unpaired), and values outside
- *   Unicode (i.e., the old sequences for code points above U+10FFFF)
- *   will be decoded in a permissive way. Other illegal sequences,
- *   such 0xFE and 0xFF and the presence of a continuation byte where
- *   not expected (or vice versa its absence), are replaced with
- *   REPLACEMENT CHARACTER.)
- *
- * and return a pointer to it.
- */
-gchar *
-format_text(wmem_allocator_t *allocator,
-                        const guchar *string, size_t len)
-{
-    return format_text_internal(allocator, string, len, FALSE);
-}
-
-/** Given a wmem scope and a null-terminated string, expected to be in
- *  UTF-8 but possibly containing invalid sequences (as it may have come
- *  from packet data), and the length of the string, generate a valid
- *  UTF-8 string from it, allocated in the specified wmem scope, that:
- *
- *   shows printable Unicode characters as themselves;
- *
- *   shows non-printable ASCII characters as C-style escapes (octal
- *   if not one of the standard ones such as LF -> '\n');
- *
- *   shows non-printable Unicode-but-not-ASCII characters as
- *   their universal character names;
- *
- *   shows illegal UTF-8 sequences as a sequence of bytes represented
- *   as C-style hex escapes;
- *
- *  and return a pointer to it.
- */
-gchar *
-format_text_string(wmem_allocator_t* allocator, const guchar *string)
-{
-    return format_text_internal(allocator, string, strlen(string), FALSE);
-}
-
-/*
- * Given a string, generate a string from it that shows non-printable
- * characters as C-style escapes except a whitespace character
- * (space, tab, carriage return, new line, vertical tab, or formfeed)
- * which will be replaced by a space, and return a pointer to it.
- */
-gchar *
-format_text_wsp(wmem_allocator_t* allocator, const guchar *string, size_t len)
-{
-    return format_text_internal(allocator, string, len, TRUE);
-}
-
-/*
- * Given a string, generate a string from it that shows non-printable
- * characters as the chr parameter passed, except a whitespace character
- * (space, tab, carriage return, new line, vertical tab, or formfeed)
- * which will be replaced by a space, and return a pointer to it.
- *
- * This does *not* treat the input string as UTF-8.
- *
- * This is useful for displaying binary data that frequently but not always
- * contains text; otherwise the number of C escape codes makes it unreadable.
- */
-char *
-format_text_chr(wmem_allocator_t *allocator, const char *string, size_t len, char chr)
-{
-    wmem_strbuf_t *buf;
-
-    buf = wmem_strbuf_sized_new(allocator, len + 1, 0);
-    for (const char *p = string; p < string + len; p++) {
-        if (g_ascii_isprint(*p)) {
-            wmem_strbuf_append_c(buf, *p);
-        }
-        else if (g_ascii_isspace(*p)) {
-            wmem_strbuf_append_c(buf, ' ');
-        }
-        else {
-            wmem_strbuf_append_c(buf, chr);
-        }
-    }
-    return wmem_strbuf_finalize(buf);
-}
-
 static gboolean
 is_byte_sep(guint8 c)
 {
@@ -807,16 +339,15 @@ hex_str_to_bytes_encoding(const gchar *hex_str, GByteArray *bytes, const gchar *
                 if (fail_if_partial) retval = FALSE;
                 break;
             }
-            ++end;
 
-            d = str_to_nibble[(guchar)*end];
+            d = str_to_nibble[(guchar)*(end+1)];
             if (d < 0) {
                 if (fail_if_partial) retval = FALSE;
                 break;
             }
             val = ((guint8)c * 16) + d;
             g_byte_array_append(bytes, &val, 1);
-            ++end;
+            end += 2;
 
             /* check for separator and peek at next char to make sure we should keep going */
             if (sep > 0 && *end == sep && str_to_nibble[(guchar)*(end+1)] > -1) {
@@ -846,15 +377,19 @@ hex_str_to_bytes_encoding(const gchar *hex_str, GByteArray *bytes, const gchar *
 }
 
 /*
- * Turn an RFC 3986 percent-encoded string into a byte array.
+ * Turn an RFC 3986 percent-encoded array of characters, not
+ * necessarily null-terminated, into a byte array.
  * XXX - We don't check for reserved characters.
+ * XXX - g_uri_unescape_bytes is superior, but limited to
+ * glib >= 2.66
  */
 #define HEX_DIGIT_BUF_LEN 3
 gboolean
-uri_str_to_bytes(const char *uri_str, GByteArray *bytes)
+uri_to_bytes(const char *uri_str, GByteArray *bytes, size_t len)
 {
     guint8        val;
-    const gchar    *p;
+    const gchar  *p;
+    const gchar  *uri_end = uri_str + len;
     gchar         hex_digit[HEX_DIGIT_BUF_LEN];
 
     g_byte_array_set_size(bytes, 0);
@@ -864,7 +399,7 @@ uri_str_to_bytes(const char *uri_str, GByteArray *bytes)
 
     p = uri_str;
 
-    while (*p) {
+    while (p < uri_end) {
         if (!g_ascii_isprint(*p))
             return FALSE;
         if (*p == '%') {
@@ -888,6 +423,17 @@ uri_str_to_bytes(const char *uri_str, GByteArray *bytes)
     return TRUE;
 }
 
+/*
+ * Turn an RFC 3986 percent-encoded string into a byte array.
+ * XXX - We don't check for reserved characters.
+ * XXX - Just use g_uri_unescape_string instead?
+ */
+gboolean
+uri_str_to_bytes(const char *uri_str, GByteArray *bytes)
+{
+    return uri_to_bytes(uri_str, bytes, strlen(uri_str));
+}
+
 /**
  * Create a copy of a GByteArray
  *
@@ -1027,9 +573,23 @@ xml_escape(const gchar *unescaped)
             case '"':
                 g_string_append(buffer, "&quot;");
                 break;
-            default:
+            case '\t':
+            case '\n':
+            case '\r':
                 g_string_append_c(buffer, c);
                 break;
+            default:
+                /* XML 1.0 doesn't allow ASCII control characters, except
+                 * for the three whitespace ones above (which do *not*
+                 * include '\v' and '\f', so not the same group as isspace),
+                 * even as character references.
+                 * There's no official way to escape them, so we'll do this. */
+                if (g_ascii_iscntrl(c)) {
+                    g_string_append_printf(buffer, "\\x%x", c);
+                } else {
+                    g_string_append_c(buffer, c);
+                }
+                break;
         }
     }
     /* Return the string value contained within the GString
@@ -1235,6 +795,156 @@ module_check_valid_name(const char *name, gboolean lower_only)
     return c;
 }
 
+static const char _hex[16] = { '0', '1', '2', '3', '4', '5', '6', '7',
+                              '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
+
+/*
+ * Copy byte by byte without UTF-8 truncation (assume valid UTF-8 input).
+ * Return byte size written, or that would have been
+ * written with enough space.
+ */
+size_t
+ws_label_strcpy(char *label_str, size_t buf_size, size_t pos,
+                const uint8_t *str, int flags)
+{
+    if (pos >= buf_size)
+        return pos;
+
+    uint8_t r = 0;
+    ssize_t chlen;
+    ssize_t idx, src_len;
+    ssize_t free_len;
+
+    label_str[pos] = '\0';
+
+    ws_return_val_if(str == NULL, pos);
+    idx = 0;
+    src_len = strlen(str);
+    free_len = buf_size - pos - 1;
+
+    while (idx < src_len) {
+        chlen = ws_utf8_char_len(str[idx]);
+        if (chlen <= 0) {
+            /* We were passed invalid UTF-8. This is an error. Complain and do... something. */
+            ws_log_utf8(str, -1, NULL);
+            /*
+             * XXX If we are going to return here instead of trying to recover maybe the log level should
+             * be higher than DEBUG.
+             */
+            return pos;
+        }
+
+        /* ASCII */
+        if (chlen == 1) {
+            if (flags & FORMAT_LABEL_REPLACE_SPACE && g_ascii_isspace(str[idx])) {
+                if (free_len >= 1) {
+                    label_str[pos] = ' ';
+                    label_str[pos+1] = '\0';
+                }
+                pos++;
+                idx++;
+                free_len--;
+                continue;
+            }
+
+            r = 0;
+            switch (str[idx]) {
+                case '\a': r = 'a'; break;
+                case '\b': r = 'b'; break;
+                case '\f': r = 'f'; break;
+                case '\n': r = 'n'; break;
+                case '\r': r = 'r'; break;
+                case '\t': r = 't'; break;
+                case '\v': r = 'v'; break;
+            }
+            if (r != 0) {
+                if (free_len >= 2) {
+                    label_str[pos] = '\\';
+                    label_str[pos+1] = r;
+                    label_str[pos+2] = '\0';
+                }
+                pos += 2;
+                idx += 1;
+                free_len -= 2;
+                continue;
+            }
+
+            if (g_ascii_isprint(str[idx])) {
+                if (free_len >= 1) {
+                    label_str[pos] = str[idx];
+                    label_str[pos+1] = '\0';
+                }
+                pos++;
+                idx++;
+                free_len--;
+                continue;
+            }
+
+            if (free_len >= 4) {
+                label_str[pos+0] = '\\';
+                label_str[pos+1] = 'x';
+
+                uint8_t ch = str[idx];
+                label_str[pos+2] = _hex[ch >> 4];
+                label_str[pos+3] = _hex[ch & 0x0F];
+                label_str[pos+4] = '\0';
+            }
+            pos += 4;
+            idx += chlen;
+            free_len -= 4;
+            continue;
+        }
+
+        /* UTF-8 multibyte */
+        if (chlen == 2 && str[idx] == 0xC2 &&
+                                str[idx+1] >= 0x80 && str[idx+1] <= 0x9F) {
+            /*
+             * Escape the C1 control codes. C0 (covered above) and C1 are
+             * inband signalling and transparent to Unicode.
+             * Anything else probably has text semantics should not be removed.
+             */
+            /*
+             * Special case: The second UTF-8 byte is the same as the Unicode
+             * code point for range U+0080 - U+009F.
+             */
+            if (free_len >= 6) {
+                label_str[pos+0] = '\\';
+                label_str[pos+1] = 'u';
+                label_str[pos+2] = '0';
+                label_str[pos+3] = '0';
+
+                uint8_t ch = str[idx+1];
+                label_str[pos+4] = _hex[ch >> 4];
+                label_str[pos+5] = _hex[ch & 0x0F];
+                label_str[pos+6] = '\0';
+            }
+            pos += 6;
+            idx += chlen;
+            free_len -= 6;
+            continue;
+        }
+
+        /* Just copy */
+        if (free_len >= chlen) {
+            for (ssize_t j = 0; j < chlen; j++) {
+                label_str[pos+j] = str[idx+j];
+            }
+            label_str[pos+chlen] = '\0';
+        }
+        pos += chlen;
+        idx += chlen;
+        free_len -= chlen;
+    }
+
+    return pos;
+}
+
+size_t
+ws_label_strcat(char *label_str, size_t bufsize, const uint8_t *str, int flags)
+{
+    return ws_label_strcpy(label_str, bufsize, strlen(label_str), str, flags);
+}
+
 /*
  * Editor modelines  -  https://www.wireshark.org/tools/modelines.html
  *
