@@ -233,7 +233,8 @@ WSLUA_METHOD Proto_register_heuristic(lua_State* L) {
     Proto proto = checkProto(L,1);
     const gchar *listname = luaL_checkstring(L, WSLUA_ARG_Proto_register_heuristic_LISTNAME);
     const gchar *proto_name = proto->name;
-    const int top = lua_gettop(L);
+    const int top _U_ = lua_gettop(L);
+
     gchar *short_name;
 
     if (!proto_name || proto->hfid == -1) {
@@ -377,6 +378,9 @@ static int Proto_get_prefs(lua_State* L) {
 
 /* WSLUA_ATTRIBUTE Proto_prefs_changed WO The preferences changed routine of this dissector,
    a Lua function you define.
+
+   The function is called when the protocol's preferences are changed.
+   It is passed no arguments.
  */
 static int Proto_set_prefs_changed(lua_State* L) {
     Proto proto = checkProto(L,1);
@@ -396,7 +400,8 @@ static int Proto_set_prefs_changed(lua_State* L) {
 
 /* WSLUA_ATTRIBUTE Proto_init WO The init routine of this dissector, a function you define.
 
-   The called init function is passed no arguments.
+   The init function is called when the a new capture file is opened or when
+   the open capture file is closed.  It is passed no arguments.
 */
 static int Proto_set_init(lua_State* L) {
     Proto proto = checkProto(L,1);
