@@ -519,6 +519,8 @@ static extcap_arg *extcap_parse_arg_sentence(GList *args, extcap_token_sentence
             target_arg->arg_type = EXTCAP_ARG_BOOLFLAG;
         } else if (g_ascii_strcasecmp(param_value, "selector") == 0) {
             target_arg->arg_type = EXTCAP_ARG_SELECTOR;
+        } else if (g_ascii_strcasecmp(param_value, "editselector") == 0) {
+            target_arg->arg_type = EXTCAP_ARG_EDIT_SELECTOR;
         } else if (g_ascii_strcasecmp(param_value, "radio") == 0) {
             target_arg->arg_type = EXTCAP_ARG_RADIO;
         } else if (g_ascii_strcasecmp(param_value, "string") == 0) {
