@@ -28,6 +28,8 @@
 void proto_register_mac_mgmt_msg_ucd(void);
 void proto_reg_handoff_mac_mgmt_msg_ucd(void);
 
+static dissector_handle_t ucd_handle;
+
 extern gboolean include_cor2_changes;
 
 guint cqich_id_size;		/* Set for CQICH_Alloc_IE */
@@ -1219,13 +1221,11 @@ void proto_register_mac_mgmt_msg_ucd(void)
 
 	proto_register_field_array(proto_mac_mgmt_msg_ucd_decoder, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
+	ucd_handle = register_dissector("mac_mgmt_msg_ucd_handler", dissect_mac_mgmt_msg_ucd_decoder, proto_mac_mgmt_msg_ucd_decoder);
 }
 
 void proto_reg_handoff_mac_mgmt_msg_ucd(void)
 {
-	dissector_handle_t ucd_handle;
-
-	ucd_handle = create_dissector_handle(dissect_mac_mgmt_msg_ucd_decoder, proto_mac_mgmt_msg_ucd_decoder);
 	dissector_add_uint("wmx.mgmtmsg", MAC_MGMT_MSG_UCD, ucd_handle);
 }
 
