@@ -15,12 +15,13 @@
 /*
  * Rawshark does the following:
  * - Opens a specified file or named pipe
- * - Applies a specfied DLT or "decode as" encapsulation
+ * - Applies a specified DLT or "decode as" encapsulation
  * - Reads frames prepended with a libpcap packet header.
  * - Prints a status line, followed by fields from a specified list.
  */
 
 #include <config.h>
+#define WS_LOG_DOMAIN  LOG_DOMAIN_MAIN
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -35,13 +36,13 @@
 
 #include <errno.h>
 
+#include <ws_exit_codes.h>
 #include <wsutil/ws_getopt.h>
 
 #include <glib.h>
 #include <epan/epan.h>
 
-#include <ui/cmdarg_err.h>
-#include <ui/exit_codes.h>
+#include <wsutil/cmdarg_err.h>
 #include <wsutil/filesystem.h>
 #include <wsutil/file_util.h>
 #include <wsutil/socket.h>
@@ -50,7 +51,7 @@
 #include <wsutil/report_message.h>
 #include <wsutil/please_report_bug.h>
 #include <wsutil/wslog.h>
-#include <ui/clopts_common.h>
+#include <wsutil/clopts_common.h>
 
 #ifdef _WIN32
 #include <wsutil/unicode-utils.h>
@@ -84,7 +85,7 @@
 #include <wiretap/pcap-encap.h>
 
 #include <cli_main.h>
-#include <ui/version_info.h>
+#include <wsutil/version_info.h>
 
 #include "capture/capture-pcap-util.h"
 
@@ -175,22 +176,38 @@ print_usage(FILE *output)
     fprintf(output, "  -d <encap:linktype>|<proto:protoname>\n");
     fprintf(output, "                           packet encapsulation or protocol\n");
     fprintf(output, "  -F <field>               field to display\n");
-#ifndef _WIN32
+#if !defined(_WIN32) && defined(RLIMIT_AS)
     fprintf(output, "  -m                       virtual memory limit, in bytes\n");
 #endif
-    fprintf(output, "  -n                       disable all name resolution (def: all enabled)\n");
+    fprintf(output, "  -n                       disable all name resolutions (def: \"mNd\" enabled, or\n");
+    fprintf(output, "                           as set in preferences)\n");
     fprintf(output, "  -N <name resolve flags>  enable specific name resolution(s): \"mnNtdv\"\n");
     fprintf(output, "  -p                       use the system's packet header format\n");
     fprintf(output, "                           (which may have 64-bit timestamps)\n");
     fprintf(output, "  -R <read filter>         packet filter in Wireshark display filter syntax\n");
     fprintf(output, "  -s                       skip PCAP header on input\n");
+    fprintf(output, "  --enable-protocol <proto_name>\n");
+    fprintf(output, "                           enable dissection of proto_name\n");
+    fprintf(output, "  --disable-protocol <proto_name>\n");
+    fprintf(output, "                           disable dissection of proto_name\n");
+    fprintf(output, "  --only-protocols <protocols>\n");
+    fprintf(output, "                           Only enable dissection of these protocols, comma\n");
+    fprintf(output, "                           separated. Disable everything else\n");
+    fprintf(output, "  --disable-all-protocols\n");
+    fprintf(output, "                           Disable dissection of all protocols\n");
+    fprintf(output, "  --enable-heuristic <short_name>\n");
+    fprintf(output, "                           enable dissection of heuristic protocol\n");
+    fprintf(output, "  --disable-heuristic <short_name>\n");
+    fprintf(output, "                           disable dissection of heuristic protocol\n");
 
     fprintf(output, "\n");
     fprintf(output, "Output:\n");
     fprintf(output, "  -l                       flush output after each packet\n");
     fprintf(output, "  -S                       format string for fields\n");
     fprintf(output, "                           (%%D - name, %%S - stringval, %%N numval)\n");
-    fprintf(output, "  -t ad|a|r|d|dd|e         output format of time stamps (def: r: rel. to first)\n");
+    fprintf(output, "  -t (a|ad|adoy|d|dd|e|r|u|ud|udoy)[.[N]]|.[N]\n");
+    fprintf(output, "                           output format of time stamps (def: r: rel. to first)\n");
+    fprintf(output, "  -u s|hms                 output format of seconds (def: s: seconds)\n");
     fprintf(output, "\n");
 
     ws_log_print_usage(output);
@@ -198,9 +215,10 @@ print_usage(FILE *output)
 
     fprintf(output, "\n");
     fprintf(output, "Miscellaneous:\n");
-    fprintf(output, "  -h                       display this help and exit\n");
+    fprintf(output, "  -h, --help               display this help and exit\n");
+    fprintf(output, "  -v, --version            display version info and exit\n");
     fprintf(output, "  -o <name>:<value> ...    override preference setting\n");
-    fprintf(output, "  -v                       display version info and exit\n");
+    fprintf(output, "  -K <keytab>              keytab file to use for kerberos decryption\n");
 }
 
 /**
@@ -397,15 +415,15 @@ main(int argc, char *argv[])
 {
     char                *err_msg;
     int                  opt, i;
+    df_error_t          *df_err;
 
-#ifndef _WIN32
+#if !defined(_WIN32) && defined(RLIMIT_AS)
     struct rlimit limit;
 #endif  /* !_WIN32 */
 
     gchar               *pipe_name = NULL;
     gchar               *rfilters[64];
     e_prefs             *prefs_p;
-    char                 badopt;
     GPtrArray           *disp_fields = g_ptr_array_new();
     guint                fc;
     gboolean             skip_pcap_header = FALSE;
@@ -413,10 +431,11 @@ main(int argc, char *argv[])
     static const struct ws_option long_options[] = {
       {"help", ws_no_argument, NULL, 'h'},
       {"version", ws_no_argument, NULL, 'v'},
+      LONGOPT_DISSECT_COMMON
       {0, 0, 0, 0 }
     };
 
-#define OPTSTRING_INIT "d:F:hlm:nN:o:pr:R:sS:t:v"
+#define OPTSTRING_INIT OPTSTRING_DISSECT_COMMON "F:hlm:o:pr:R:sS:v"
 
     static const char    optstring[] = OPTSTRING_INIT;
     static const struct report_message_routines rawshark_report_routines = {
@@ -448,7 +467,9 @@ main(int argc, char *argv[])
     ws_log_init("rawshark", vcmdarg_err);
 
     /* Early logging command-line initialization. */
-    ws_log_parse_args(&argc, argv, vcmdarg_err, INVALID_OPTION);
+    ws_log_parse_args(&argc, argv, vcmdarg_err, WS_EXIT_INVALID_OPTION);
+
+    ws_noisy("Finished log init and parsing command line log arguments");
 
     /* Initialize the version information. */
     ws_init_version_info("Rawshark",
@@ -510,7 +531,7 @@ main(int argc, char *argv[])
        dissectors, and we must do it before we read the preferences, in
        case any dissectors register preferences. */
     if (!epan_init(NULL, NULL, TRUE)) {
-        ret = INIT_FAILED;
+        ret = WS_EXIT_INIT_FAILED;
         goto clean_exit;
     }
 
@@ -536,9 +557,12 @@ main(int argc, char *argv[])
     while ((opt = ws_getopt_long(argc, argv, optstring, long_options, NULL)) != -1) {
         switch (opt) {
             case 'd':        /* Payload type */
+                /* XXX: This option should probably be changed so it doesn't
+                 * conflict with the common dissection option for Decode As.
+                 */
                 if (!set_link_type(ws_optarg)) {
                     cmdarg_err("Invalid link type or protocol \"%s\"", ws_optarg);
-                    ret = INVALID_OPTION;
+                    ret = WS_EXIT_INVALID_OPTION;
                     goto clean_exit;
                 }
                 break;
@@ -565,30 +589,19 @@ main(int argc, char *argv[])
                    and the output buffer is only flushed when it fills up). */
                 line_buffered = TRUE;
                 break;
-#ifndef _WIN32
+#if !defined(_WIN32) && defined(RLIMIT_AS)
             case 'm':
                 limit.rlim_cur = get_positive_int(ws_optarg, "memory limit");
                 limit.rlim_max = get_positive_int(ws_optarg, "memory limit");
 
                 if(setrlimit(RLIMIT_AS, &limit) != 0) {
-                    cmdarg_err("setrlimit() returned error");
-                    ret = INVALID_OPTION;
+                    cmdarg_err("setrlimit(RLIMIT_AS) failed: %s",
+                               g_strerror(errno));
+                    ret = WS_EXIT_INVALID_OPTION;
                     goto clean_exit;
                 }
                 break;
 #endif
-            case 'n':        /* No name resolution */
-                disable_name_resolution();
-                break;
-            case 'N':        /* Select what types of addresses/port #s to resolve */
-                badopt = string_to_name_resolve(ws_optarg, &gbl_resolv_flags);
-                if (badopt != '\0') {
-                    cmdarg_err("-N specifies unknown resolving option '%c'; valid options are 'd', m', 'n', 'N', and 't'",
-                               badopt);
-                    ret = INVALID_OPTION;
-                    goto clean_exit;
-                }
-                break;
             case 'o':        /* Override preference from command line */
             {
                 char *errmsg = NULL;
@@ -602,19 +615,19 @@ main(int argc, char *argv[])
                         cmdarg_err("Invalid -o flag \"%s\"%s%s", ws_optarg,
                                 errmsg ? ": " : "", errmsg ? errmsg : "");
                         g_free(errmsg);
-                        ret = INVALID_OPTION;
+                        ret = WS_EXIT_INVALID_OPTION;
                         goto clean_exit;
                         break;
 
                     case PREFS_SET_NO_SUCH_PREF:
                         cmdarg_err("-o flag \"%s\" specifies unknown preference", ws_optarg);
-                        ret = INVALID_OPTION;
+                        ret = WS_EXIT_INVALID_OPTION;
                         goto clean_exit;
                         break;
 
                     case PREFS_SET_OBSOLETE:
                         cmdarg_err("-o flag \"%s\" specifies obsolete preference", ws_optarg);
-                        ret = INVALID_OPTION;
+                        ret = WS_EXIT_INVALID_OPTION;
                         goto clean_exit;
                         break;
                 }
@@ -632,7 +645,7 @@ main(int argc, char *argv[])
                 }
                 else {
                     cmdarg_err("Too many display filters");
-                    ret = INVALID_OPTION;
+                    ret = WS_EXIT_INVALID_OPTION;
                     goto clean_exit;
                 }
                 break;
@@ -642,45 +655,7 @@ main(int argc, char *argv[])
             case 'S':        /* Print string representations */
                 if (!parse_field_string_format(ws_optarg)) {
                     cmdarg_err("Invalid field string format");
-                    ret = INVALID_OPTION;
-                    goto clean_exit;
-                }
-                break;
-            case 't':        /* Time stamp type */
-                if (strcmp(ws_optarg, "r") == 0)
-                    timestamp_set_type(TS_RELATIVE);
-                else if (strcmp(ws_optarg, "a") == 0)
-                    timestamp_set_type(TS_ABSOLUTE);
-                else if (strcmp(ws_optarg, "ad") == 0)
-                    timestamp_set_type(TS_ABSOLUTE_WITH_YMD);
-                else if (strcmp(ws_optarg, "adoy") == 0)
-                    timestamp_set_type(TS_ABSOLUTE_WITH_YDOY);
-                else if (strcmp(ws_optarg, "d") == 0)
-                    timestamp_set_type(TS_DELTA);
-                else if (strcmp(ws_optarg, "dd") == 0)
-                    timestamp_set_type(TS_DELTA_DIS);
-                else if (strcmp(ws_optarg, "e") == 0)
-                    timestamp_set_type(TS_EPOCH);
-                else if (strcmp(ws_optarg, "u") == 0)
-                    timestamp_set_type(TS_UTC);
-                else if (strcmp(ws_optarg, "ud") == 0)
-                    timestamp_set_type(TS_UTC_WITH_YMD);
-                else if (strcmp(ws_optarg, "udoy") == 0)
-                    timestamp_set_type(TS_UTC_WITH_YDOY);
-                else {
-                    cmdarg_err("Invalid time stamp type \"%s\"",
-                               ws_optarg);
-                    cmdarg_err_cont(
-"It must be \"a\" for absolute, \"ad\" for absolute with YYYY-MM-DD date,");
-                    cmdarg_err_cont(
-"\"adoy\" for absolute with YYYY/DOY date, \"d\" for delta,");
-                    cmdarg_err_cont(
-"\"dd\" for delta displayed, \"e\" for epoch, \"r\" for relative,");
-                    cmdarg_err_cont(
-"\"u\" for absolute UTC, \"ud\" for absolute UTC with YYYY-MM-DD date,");
-                    cmdarg_err_cont(
-"or \"udoy\" for absolute UTC with YYYY/DOY date.");
-                    ret = INVALID_OPTION;
+                    ret = WS_EXIT_INVALID_OPTION;
                     goto clean_exit;
                 }
                 break;
@@ -689,10 +664,30 @@ main(int argc, char *argv[])
                 show_version();
                 goto clean_exit;
             }
+            /* Common dissection options - 'd' for Decode As also makes
+             * sense, but rawshark uses it for the payload link layer/
+             * dissector selection.
+             */
+            case 'K':        /* Kerberos keytab file */
+            case 'n':        /* No name resolution */
+            case 'N':        /* Select what types of addresses/port #s to resolve */
+            case 't':        /* Time stamp type */
+            case 'u':        /* Seconds type */
+            case LONGOPT_DISABLE_PROTOCOL: /* disable dissection of protocol */
+            case LONGOPT_ENABLE_HEURISTIC: /* enable heuristic dissection of protocol */
+            case LONGOPT_DISABLE_HEURISTIC: /* disable heuristic dissection of protocol */
+            case LONGOPT_ENABLE_PROTOCOL: /* enable dissection of protocol (that is disabled by default) */
+            case LONGOPT_ONLY_PROTOCOLS: /* enable dissection of only this comma separated list of protocols */
+            case LONGOPT_DISABLE_ALL_PROTOCOLS: /* enable dissection of protocol (that is disabled by default) */
+                if (!dissect_opts_handle_opt(opt, ws_optarg)) {
+                    ret = WS_EXIT_INVALID_OPTION;
+                    goto clean_exit;
+                }
+                break;
             default:
             case '?':        /* Bad flag - print usage message */
                 print_usage(stderr);
-                ret = INVALID_OPTION;
+                ret = WS_EXIT_INVALID_OPTION;
                 goto clean_exit;
         }
     }
@@ -721,7 +716,7 @@ main(int argc, char *argv[])
             if (n_rfilters != 0) {
                 cmdarg_err("Read filters were specified both with \"-R\" "
                            "and with additional command-line arguments");
-                ret = INVALID_OPTION;
+                ret = WS_EXIT_INVALID_OPTION;
                 goto clean_exit;
             }
             rfilters[n_rfilters] = get_args_as_string(argc, argv, ws_optind);
@@ -731,7 +726,7 @@ main(int argc, char *argv[])
     /* Make sure we got a dissector handle for our payload. */
     if (encap == WTAP_ENCAP_UNKNOWN) {
         cmdarg_err("No valid payload dissector specified.");
-        ret = INVALID_OPTION;
+        ret = WS_EXIT_INVALID_OPTION;
         goto clean_exit;
     }
 
@@ -741,24 +736,32 @@ main(int argc, char *argv[])
         cmdarg_err("%s", err_msg);
         g_free(err_msg);
         cmdarg_err_cont("%s", please_report_bug());
-        ret = INIT_FAILED;
+        ret = WS_EXIT_INIT_FAILED;
         goto clean_exit;
     }
 
+    if (global_dissect_options.time_format != TS_NOT_SET)
+        timestamp_set_type(global_dissect_options.time_format);
+    if (global_dissect_options.time_precision != TS_PREC_NOT_SET)
+        timestamp_set_precision(global_dissect_options.time_precision);
+
     /*
      * Enabled and disabled protocols and heuristic dissectors as per
      * command-line options.
      */
-    setup_enabled_and_disabled_protocols();
+    if (!setup_enabled_and_disabled_protocols()) {
+        ret = WS_EXIT_INVALID_OPTION;
+        goto clean_exit;
+    }
 
     /* Build the column format array */
     build_column_format_array(&cfile.cinfo, prefs_p->num_cols, TRUE);
 
     if (n_rfilters != 0) {
         for (i = 0; i < n_rfilters; i++) {
-            if (!dfilter_compile(rfilters[i], &rfcodes[n_rfcodes], &err_msg)) {
-                cmdarg_err("%s", err_msg);
-                g_free(err_msg);
+            if (!dfilter_compile(rfilters[i], &rfcodes[n_rfcodes], &df_err)) {
+                cmdarg_err("%s", df_err->msg);
+                df_error_free(&df_err);
                 ret = INVALID_DFILTER;
                 goto clean_exit;
             }
@@ -779,7 +782,7 @@ main(int argc, char *argv[])
         relinquish_special_privs_perm();
 
         if (raw_cf_open(&cfile, pipe_name) != CF_OK) {
-            ret = OPEN_ERROR;
+            ret = WS_EXIT_OPEN_ERROR;
             goto clean_exit;
         }
 
@@ -800,13 +803,13 @@ main(int argc, char *argv[])
 
         /* Process the packets in the file */
         if (!load_cap_file(&cfile)) {
-            ret = OPEN_ERROR;
+            ret = WS_EXIT_OPEN_ERROR;
             goto clean_exit;
         }
     } else {
         /* If you want to capture live packets, use TShark. */
         cmdarg_err("Input file or pipe name not specified.");
-        ret = OPEN_ERROR;
+        ret = WS_EXIT_OPEN_ERROR;
         goto clean_exit;
     }
 
@@ -1129,7 +1132,6 @@ static gboolean print_field_value(field_info *finfo, int cmd_line_index)
     gint32             svalue;
     guint64            uvalue64;
     gint64             svalue64;
-    const true_false_string *tfstring = &tfs_true_false;
 
     hfinfo = finfo->hfinfo;
 
@@ -1137,7 +1139,7 @@ static gboolean print_field_value(field_info *finfo, int cmd_line_index)
         label_s = g_string_new("");
     }
 
-    fs_buf = fvalue_to_string_repr(NULL, &finfo->value,
+    fs_buf = fvalue_to_string_repr(NULL, finfo->value,
                           FTREPR_DFILTER, finfo->hfinfo->display);
     if (fs_buf != NULL) {
         /*
@@ -1148,7 +1150,7 @@ static gboolean print_field_value(field_info *finfo, int cmd_line_index)
         fs_ptr = fs_buf;
 
         /* String types are quoted. Remove them. */
-        if (IS_FT_STRING(fvalue_type_ftenum(&finfo->value)) && fs_len > 2) {
+        if (FT_IS_STRING(fvalue_type_ftenum(finfo->value)) && fs_len > 2) {
             fs_buf[fs_len - 1] = '\0';
             fs_ptr++;
         }
@@ -1171,16 +1173,15 @@ static gboolean print_field_value(field_info *finfo, int cmd_line_index)
                     case SF_STRVAL:
                         switch(hfinfo->type) {
                             case FT_BOOLEAN:
-                                uvalue64 = fvalue_get_uinteger64(&finfo->value);
-                                tfstring = (const true_false_string*) hfinfo->strings;
-                                g_string_append(label_s, tfs_get_string(!!uvalue64, tfstring));
+                                uvalue64 = fvalue_get_uinteger64(finfo->value);
+                                g_string_append(label_s, tfs_get_string(!!uvalue64, hfinfo->strings));
                                 break;
                             case FT_INT8:
                             case FT_INT16:
                             case FT_INT24:
                             case FT_INT32:
                                 DISSECTOR_ASSERT(!hfinfo->bitmask);
-                                svalue = fvalue_get_sinteger(&finfo->value);
+                                svalue = fvalue_get_sinteger(finfo->value);
                                 if (hfinfo->display & BASE_RANGE_STRING) {
                                     g_string_append(label_s, rval_to_str_const(svalue, (const range_string *) hfinfo->strings, "Unknown"));
                                 } else if (hfinfo->display & BASE_EXT_STRING) {
@@ -1194,7 +1195,7 @@ static gboolean print_field_value(field_info *finfo, int cmd_line_index)
                             case FT_INT56:
                             case FT_INT64:
                                 DISSECTOR_ASSERT(!hfinfo->bitmask);
-                                svalue64 = fvalue_get_sinteger64(&finfo->value);
+                                svalue64 = fvalue_get_sinteger64(finfo->value);
                                 if (hfinfo->display & BASE_VAL64_STRING) {
                                     g_string_append(label_s, val64_to_str_const(svalue64, (const val64_string *)(hfinfo->strings), "Unknown"));
                                 }
@@ -1204,7 +1205,7 @@ static gboolean print_field_value(field_info *finfo, int cmd_line_index)
                             case FT_UINT24:
                             case FT_UINT32:
                                 DISSECTOR_ASSERT(!hfinfo->bitmask);
-                                uvalue = fvalue_get_uinteger(&finfo->value);
+                                uvalue = fvalue_get_uinteger(finfo->value);
                                 if (!hfinfo->bitmask && hfinfo->display & BASE_RANGE_STRING) {
                                     g_string_append(label_s, rval_to_str_const(uvalue, (const range_string *) hfinfo->strings, "Unknown"));
                                 } else if (hfinfo->display & BASE_EXT_STRING) {
@@ -1218,7 +1219,7 @@ static gboolean print_field_value(field_info *finfo, int cmd_line_index)
                             case FT_UINT56:
                             case FT_UINT64:
                                 DISSECTOR_ASSERT(!hfinfo->bitmask);
-                                uvalue64 = fvalue_get_uinteger64(&finfo->value);
+                                uvalue64 = fvalue_get_uinteger64(finfo->value);
                                 if (hfinfo->display & BASE_VAL64_STRING) {
                                     g_string_append(label_s, val64_to_str_const(uvalue64, (const val64_string *)(hfinfo->strings), "Unknown"));
                                 }
@@ -1420,16 +1421,16 @@ show_print_file_io_error(int err)
 static const nstime_t *
 raw_get_frame_ts(struct packet_provider_data *prov, guint32 frame_num)
 {
-    if (prov->ref && prov->ref->num == frame_num)
-        return &prov->ref->abs_ts;
+    const frame_data *ts_fd = NULL;
+    if (prov->ref && prov->ref->num == frame_num) {
+        ts_fd = prov->ref;
+    } else if (prov->prev_dis && prov->prev_dis->num == frame_num) {
+        ts_fd = prov->prev_dis;
+    } else if (prov->prev_cap && prov->prev_cap->num == frame_num) {
+        ts_fd = prov->prev_cap;
+    }
 
-    if (prov->prev_dis && prov->prev_dis->num == frame_num)
-        return &prov->prev_dis->abs_ts;
-
-    if (prov->prev_cap && prov->prev_cap->num == frame_num)
-        return &prov->prev_cap->abs_ts;
-
-    return NULL;
+    return (ts_fd && ts_fd->has_ts) ? &ts_fd->abs_ts : NULL;
 }
 
 static epan_t *
