@@ -8,17 +8,19 @@
 #
 '''Capture tests'''
 
-import fixtures
 import glob
 import hashlib
 import os
 import socket
 import subprocess
 import subprocesstest
+from subprocesstest import cat_dhcp_command, cat_cap_file_command, count_output, grep_output, check_packet_count
 import sys
 import threading
 import time
 import uuid
+import sysconfig
+import pytest
 
 capture_duration = 5
 
@@ -43,7 +45,7 @@ class UdpTrafficGenerator(threading.Thread):
             self.join()
 
 
-@fixtures.fixture
+@pytest.fixture
 def traffic_generator():
     '''
     Traffic generator factory. Invoking it returns a tuple (start_func, cfilter)
@@ -65,7 +67,7 @@ def traffic_generator():
             thread.stop()
 
 
-@fixtures.fixture(scope='session')
+@pytest.fixture(scope='session')
 def wireshark_k(wireshark_command):
     return tuple(list(wireshark_command) + ['-k'])
 
@@ -75,22 +77,20 @@ def capture_command(*args, shell=False):
     if type(cmd_args[0]) != str:
         # Assume something like ['wireshark', '-k']
         cmd_args = list(cmd_args[0]) + list(cmd_args)[1:]
-    if sys.platform == "win32":
-        cmd_args[0] = '"{}"'.format(cmd_args[0])
     if shell:
         cmd_args = ' '.join(cmd_args)
     return cmd_args
 
 
-@fixtures.fixture
-def check_capture_10_packets(capture_interface, cmd_dumpcap, traffic_generator):
+@pytest.fixture
+def check_capture_10_packets(capture_interface, cmd_capinfos, traffic_generator, result_file):
     start_traffic, cfilter = traffic_generator
-    def check_capture_10_packets_real(self, cmd=None, to_stdout=False):
-        self.assertIsNotNone(cmd)
-        testout_file = self.filename_from_id(testout_pcap)
+    def check_capture_10_packets_real(self, cmd=None, to_stdout=False, env=None):
+        assert cmd is not None
+        testout_file = result_file(testout_pcap)
         stop_traffic = start_traffic()
         if to_stdout:
-            capture_proc = self.runProcess(capture_command(cmd,
+            subprocesstest.check_run(capture_command(cmd,
                 '-i', '"{}"'.format(capture_interface),
                 '-p',
                 '-w', '-',
@@ -100,66 +100,60 @@ def check_capture_10_packets(capture_interface, cmd_dumpcap, traffic_generator):
                 '>', testout_file,
                 shell=True
             ),
-            shell=True
-            )
+            shell=True, env=env)
         else:
-            capture_proc = self.runProcess(capture_command(cmd,
+            subprocesstest.check_run(capture_command(cmd,
                 '-i', capture_interface,
                 '-p',
                 '-w', testout_file,
                 '-c', '10',
                 '-a', 'duration:{}'.format(capture_duration),
                 '-f', cfilter,
-            ))
+            ), env=env)
         stop_traffic()
-        capture_returncode = capture_proc.returncode
-        if capture_returncode != 0:
-            self.log_fd.write('{} -D output:\n'.format(cmd))
-            self.runProcess((cmd, '-D'))
-        self.assertEqual(capture_returncode, 0)
-        self.checkPacketCount(10)
+        check_packet_count(cmd_capinfos, 10, testout_file)
     return check_capture_10_packets_real
 
 
-@fixtures.fixture
-def check_capture_fifo(cmd_dumpcap):
+@pytest.fixture
+def check_capture_fifo(cmd_capinfos, result_file):
     if sys.platform == 'win32':
-        fixtures.skip('Test requires OS fifo support.')
+        pytest.skip('Test requires OS fifo support.')
 
-    def check_capture_fifo_real(self, cmd=None):
-        self.assertIsNotNone(cmd)
-        testout_file = self.filename_from_id(testout_pcap)
-        fifo_file = self.filename_from_id('testout.fifo')
+    def check_capture_fifo_real(self, cmd=None, env=None):
+        assert cmd is not None
+        testout_file = result_file(testout_pcap)
+        fifo_file = result_file('testout.fifo')
         try:
             # If a previous test left its fifo laying around, e.g. from a failure, remove it.
             os.unlink(fifo_file)
         except Exception:
             pass
         os.mkfifo(fifo_file)
-        slow_dhcp_cmd = subprocesstest.cat_dhcp_command('slow')
-        fifo_proc = self.startProcess(
+        slow_dhcp_cmd = cat_dhcp_command('slow')
+        fifo_proc = subprocess.Popen(
             ('{0} > {1}'.format(slow_dhcp_cmd, fifo_file)),
             shell=True)
-        capture_proc = self.assertRun(capture_command(cmd,
+        subprocesstest.check_run(capture_command(cmd,
             '-i', fifo_file,
             '-p',
             '-w', testout_file,
             '-a', 'duration:{}'.format(capture_duration),
-        ))
+        ), env=env)
         fifo_proc.kill()
-        self.assertTrue(os.path.isfile(testout_file))
-        self.checkPacketCount(8)
+        assert os.path.isfile(testout_file)
+        check_packet_count(cmd_capinfos, 8, testout_file)
     return check_capture_fifo_real
 
 
-@fixtures.fixture
-def check_capture_stdin(cmd_dumpcap):
+@pytest.fixture
+def check_capture_stdin(cmd_capinfos, result_file):
     # Capturing always requires dumpcap, hence the dependency on it.
-    def check_capture_stdin_real(self, cmd=None):
+    def check_capture_stdin_real(self, cmd=None, env=None):
         # Similar to suite_io.check_io_4_packets.
-        self.assertIsNotNone(cmd)
-        testout_file = self.filename_from_id(testout_pcap)
-        slow_dhcp_cmd = subprocesstest.cat_dhcp_command('slow')
+        assert cmd is not None
+        testout_file = result_file(testout_pcap)
+        slow_dhcp_cmd = cat_dhcp_command('slow')
         capture_cmd = capture_command(cmd,
             '-i', '-',
             '-w', testout_file,
@@ -169,24 +163,28 @@ def check_capture_stdin(cmd_dumpcap):
         is_gui = type(cmd) != str and '-k' in cmd[0]
         if is_gui:
             capture_cmd += ' --log-level=info'
-        pipe_proc = self.assertRun(slow_dhcp_cmd + ' | ' + capture_cmd, shell=True)
+        if sysconfig.get_platform().startswith('mingw'):
+            pytest.skip('FIXME Pipes are broken with the MSYS2 shell')
+        pipe_proc = subprocesstest.check_run(slow_dhcp_cmd + ' | ' + capture_cmd, shell=True, capture_output=True, env=env)
         if is_gui:
-            self.assertTrue(self.grepOutput('Wireshark is up and ready to go'), 'No startup message.')
-            self.assertTrue(self.grepOutput('Capture started'), 'No capture start message.')
-            self.assertTrue(self.grepOutput('Capture stopped'), 'No capture stop message.')
-        self.assertTrue(os.path.isfile(testout_file))
-        self.checkPacketCount(8)
+            # Wireshark uses stdout and not stderr for diagnostic messages
+            # XXX: Confirm this
+            assert grep_output(pipe_proc.stdout, 'Wireshark is up and ready to go'), 'No startup message.'
+            assert grep_output(pipe_proc.stdout, 'Capture started'), 'No capture start message.'
+            assert grep_output(pipe_proc.stdout, 'Capture stopped'), 'No capture stop message.'
+        assert os.path.isfile(testout_file)
+        check_packet_count(cmd_capinfos, 8, testout_file)
     return check_capture_stdin_real
 
 
-@fixtures.fixture
-def check_capture_read_filter(capture_interface, traffic_generator):
+@pytest.fixture
+def check_capture_read_filter(capture_interface, traffic_generator, cmd_capinfos, result_file):
     start_traffic, cfilter = traffic_generator
-    def check_capture_read_filter_real(self, cmd=None):
-        self.assertIsNotNone(cmd)
-        testout_file = self.filename_from_id(testout_pcap)
+    def check_capture_read_filter_real(self, cmd=None, env=None):
+        assert cmd is not None
+        testout_file = result_file(testout_pcap)
         stop_traffic = start_traffic()
-        capture_proc = self.assertRun(capture_command(cmd,
+        subprocesstest.check_run(capture_command(cmd,
             '-i', capture_interface,
             '-p',
             '-w', testout_file,
@@ -195,56 +193,55 @@ def check_capture_read_filter(capture_interface, traffic_generator):
             '-c', '10',
             '-a', 'duration:{}'.format(capture_duration),
             '-f', cfilter,
-        ))
+        ), env=env)
         stop_traffic()
-        self.checkPacketCount(0)
+        check_packet_count(cmd_capinfos, 0, testout_file)
     return check_capture_read_filter_real
 
-@fixtures.fixture
-def check_capture_snapshot_len(capture_interface, cmd_tshark, traffic_generator):
+@pytest.fixture
+def check_capture_snapshot_len(capture_interface, cmd_tshark, traffic_generator, cmd_capinfos, result_file):
     start_traffic, cfilter = traffic_generator
-    def check_capture_snapshot_len_real(self, cmd=None):
-        self.assertIsNotNone(cmd)
+    def check_capture_snapshot_len_real(self, cmd=None, env=None):
+        assert cmd is not None
         stop_traffic = start_traffic()
-        testout_file = self.filename_from_id(testout_pcap)
-        capture_proc = self.assertRun(capture_command(cmd,
+        testout_file = result_file(testout_pcap)
+        subprocesstest.check_run(capture_command(cmd,
             '-i', capture_interface,
             '-p',
             '-w', testout_file,
             '-s', str(snapshot_len),
             '-a', 'duration:{}'.format(capture_duration),
             '-f', cfilter,
-        ))
+        ), env=env)
         stop_traffic()
-        self.assertTrue(os.path.isfile(testout_file))
+        assert os.path.isfile(testout_file)
 
         # Use tshark to filter out all packets larger than 68 bytes.
-        testout2_file = self.filename_from_id('testout2.pcap')
+        testout2_file = result_file('testout2.pcap')
 
-        filter_proc = self.assertRun((cmd_tshark,
+        subprocesstest.check_run((cmd_tshark,
             '-r', testout_file,
             '-w', testout2_file,
             '-Y', 'frame.cap_len>{}'.format(snapshot_len),
-        ))
-        self.checkPacketCount(0, cap_file=testout2_file)
+        ), env=env)
+        check_packet_count(cmd_capinfos, 0, testout2_file)
     return check_capture_snapshot_len_real
 
 
-@fixtures.fixture
-def check_dumpcap_autostop_stdin(cmd_dumpcap):
-    def check_dumpcap_autostop_stdin_real(self, packets=None, filesize=None):
+@pytest.fixture
+def check_dumpcap_autostop_stdin(cmd_dumpcap, cmd_capinfos, result_file):
+    def check_dumpcap_autostop_stdin_real(self, packets=None, filesize=None, env=None):
         # Similar to check_capture_stdin.
-        testout_file = self.filename_from_id(testout_pcap)
-        cat100_dhcp_cmd = subprocesstest.cat_dhcp_command('cat100')
+        testout_file = result_file(testout_pcap)
+        cat100_dhcp_cmd = cat_dhcp_command('cat100')
         condition='oops:invalid'
 
-        self.assertTrue(packets is not None or filesize is not None, 'Need one of packets or filesize')
-        self.assertFalse(packets is not None and filesize is not None, 'Need one of packets or filesize')
-
         if packets is not None:
             condition = 'packets:{}'.format(packets)
         elif filesize is not None:
             condition = 'filesize:{}'.format(filesize)
+        else:
+            raise AssertionError('Need one of packets or filesize')
 
         cmd_ = '"{}"'.format(cmd_dumpcap)
         capture_cmd = ' '.join((cmd_,
@@ -252,34 +249,35 @@ def check_dumpcap_autostop_stdin(cmd_dumpcap):
             '-w', testout_file,
             '-a', condition,
         ))
-        pipe_proc = self.assertRun(cat100_dhcp_cmd + ' | ' + capture_cmd, shell=True)
-        self.assertTrue(os.path.isfile(testout_file))
+        if sysconfig.get_platform().startswith('mingw'):
+            pytest.skip('FIXME Pipes are broken with the MSYS2 shell')
+        subprocesstest.check_run(cat100_dhcp_cmd + ' | ' + capture_cmd, shell=True, env=env)
+        assert os.path.isfile(testout_file)
 
         if packets is not None:
-            self.checkPacketCount(packets)
+            check_packet_count(cmd_capinfos, packets, testout_file)
         elif filesize is not None:
             capturekb = os.path.getsize(testout_file) / 1000
-            self.assertGreaterEqual(capturekb, filesize)
+            assert capturekb >= filesize
     return check_dumpcap_autostop_stdin_real
 
 
-@fixtures.fixture
-def check_dumpcap_ringbuffer_stdin(cmd_dumpcap):
-    def check_dumpcap_ringbuffer_stdin_real(self, packets=None, filesize=None):
+@pytest.fixture
+def check_dumpcap_ringbuffer_stdin(cmd_dumpcap, cmd_capinfos, result_file):
+    def check_dumpcap_ringbuffer_stdin_real(self, packets=None, filesize=None, env=None):
         # Similar to check_capture_stdin.
         rb_unique = 'dhcp_rb_' + uuid.uuid4().hex[:6] # Random ID
-        testout_file = '{}.{}.pcapng'.format(self.id(), rb_unique)
-        testout_glob = '{}.{}_*.pcapng'.format(self.id(), rb_unique)
-        cat100_dhcp_cmd = subprocesstest.cat_dhcp_command('cat100')
+        testout_file = result_file('testout.{}.pcapng'.format(rb_unique))
+        testout_glob = result_file('testout.{}_*.pcapng'.format(rb_unique))
+        cat100_dhcp_cmd = cat_dhcp_command('cat100')
         condition='oops:invalid'
 
-        self.assertTrue(packets is not None or filesize is not None, 'Need one of packets or filesize')
-        self.assertFalse(packets is not None and filesize is not None, 'Need one of packets or filesize')
-
         if packets is not None:
             condition = 'packets:{}'.format(packets)
         elif filesize is not None:
             condition = 'filesize:{}'.format(filesize)
+        else:
+            raise AssertionError('Need one of packets or filesize')
 
         cmd_ = '"{}"'.format(cmd_dumpcap)
         capture_cmd = ' '.join((cmd_,
@@ -288,29 +286,28 @@ def check_dumpcap_ringbuffer_stdin(cmd_dumpcap):
             '-a', 'files:2',
             '-b', condition,
         ))
-        pipe_proc = self.assertRun(cat100_dhcp_cmd + ' | ' + capture_cmd, shell=True)
+        if sysconfig.get_platform().startswith('mingw'):
+            pytest.skip('FIXME Pipes are broken with the MSYS2 shell')
+        subprocesstest.check_run(cat100_dhcp_cmd + ' | ' + capture_cmd, shell=True, env=env)
 
         rb_files = glob.glob(testout_glob)
-        for rbf in rb_files:
-            self.cleanup_files.append(rbf)
-
-        self.assertEqual(len(rb_files), 2)
+        assert len(rb_files) == 2
 
         for rbf in rb_files:
-            self.assertTrue(os.path.isfile(rbf))
+            assert os.path.isfile(rbf)
             if packets is not None:
-                self.checkPacketCount(packets, cap_file=rbf)
+                check_packet_count(cmd_capinfos, packets, rbf)
             elif filesize is not None:
                 capturekb = os.path.getsize(rbf) / 1000
-                self.assertGreaterEqual(capturekb, filesize)
+                assert capturekb >= filesize
     return check_dumpcap_ringbuffer_stdin_real
 
 
-@fixtures.fixture
-def check_dumpcap_pcapng_sections(cmd_dumpcap, cmd_tshark, capture_file):
+@pytest.fixture
+def check_dumpcap_pcapng_sections(cmd_dumpcap, cmd_tshark, cmd_capinfos, capture_file, result_file):
     if sys.platform == 'win32':
-        fixtures.skip('Test requires OS fifo support.')
-    def check_dumpcap_pcapng_sections_real(self, multi_input=False, multi_output=False):
+        pytest.skip('Test requires OS fifo support.')
+    def check_dumpcap_pcapng_sections_real(self, multi_input=False, multi_output=False, env=None):
         # Make sure we always test multiple SHBs in an input.
         in_files_l = [ [
             capture_file('many_interfaces.pcapng.1'),
@@ -333,24 +330,24 @@ def check_dumpcap_pcapng_sections(cmd_dumpcap, cmd_tshark, capture_file):
         check_vals = [ check_val_d ]
 
         for in_files in in_files_l:
-            fifo_file = self.filename_from_id('dumpcap_pcapng_sections_{}.fifo'.format(len(fifo_files) + 1))
+            fifo_file = result_file('dumpcap_pcapng_sections_{}.fifo'.format(len(fifo_files) + 1))
             fifo_files.append(fifo_file)
             # If a previous test left its fifo laying around, e.g. from a failure, remove it.
             try:
                 os.unlink(fifo_file)
             except Exception: pass
             os.mkfifo(fifo_file)
-            cat_cmd = subprocesstest.cat_cap_file_command(in_files)
-            fifo_procs.append(self.startProcess(('{0} > {1}'.format(cat_cmd, fifo_file)), shell=True))
+            cat_cmd = cat_cap_file_command(in_files)
+            fifo_procs.append(subprocess.Popen(('{0} > {1}'.format(cat_cmd, fifo_file)), shell=True))
 
         if multi_output:
             rb_unique = 'sections_rb_' + uuid.uuid4().hex[:6] # Random ID
-            testout_glob = '{}.{}_*.pcapng'.format(self.id(), rb_unique)
-            testout_file = '{}.{}.pcapng'.format(self.id(), rb_unique)
+            testout_file = result_file('testout.{}.pcapng'.format(rb_unique))
+            testout_glob = result_file('testout.{}_*.pcapng'.format(rb_unique))
             check_vals.append(check_val_d.copy())
             # check_vals[]['filename'] will be filled in below
         else:
-            testout_file = self.filename_from_id(testout_pcapng)
+            testout_file = result_file(testout_pcapng)
             check_vals[0]['filename'] = testout_file
 
         # Capture commands
@@ -407,19 +404,18 @@ def check_dumpcap_pcapng_sections(cmd_dumpcap, cmd_tshark, capture_file):
 
         capture_cmd = capture_command(cmd_dumpcap, *capture_cmd_args)
 
-        capture_proc = self.assertRun(capture_cmd)
+        subprocesstest.check_run(capture_cmd, env=env)
         for fifo_proc in fifo_procs: fifo_proc.kill()
 
         rb_files = []
         if multi_output:
             rb_files = sorted(glob.glob(testout_glob))
-            self.assertEqual(len(rb_files), 2)
+            assert len(rb_files) == 2
             check_vals[0]['filename'] = rb_files[0]
             check_vals[1]['filename'] = rb_files[1]
 
         for rbf in rb_files:
-            self.cleanup_files.append(rbf)
-            self.assertTrue(os.path.isfile(rbf))
+            assert os.path.isfile(rbf)
 
         # Output tests
 
@@ -433,7 +429,7 @@ def check_dumpcap_pcapng_sections(cmd_dumpcap, cmd_tshark, capture_file):
                     in_hash.update(f.read())
             with open(testout_file, 'rb') as f:
                 out_hash.update(f.read())
-            self.assertEqual(in_hash.hexdigest(), out_hash.hexdigest())
+            assert in_hash.hexdigest() == out_hash.hexdigest()
 
         # many_interfaces.pcapng.1 : 64 packets written by "Passthrough test #1"
         # many_interfaces.pcapng.2 : 15 packets written by "Passthrough test #2"
@@ -446,158 +442,158 @@ def check_dumpcap_pcapng_sections(cmd_dumpcap, cmd_tshark, capture_file):
             # file in is nondeterministic.
             idb_compare_eq = False
         for check_val in check_vals:
-            self.checkPacketCount(check_val['packet_count'], cap_file=check_val['filename'])
+            check_packet_count(cmd_capinfos, check_val['packet_count'], check_val['filename'])
 
-            tshark_proc = self.assertRun(capture_command(cmd_tshark,
+            tshark_proc = subprocesstest.check_run(capture_command(cmd_tshark,
                 '-r', check_val['filename'],
                 '-V',
                 '-X', 'read_format:MIME Files Format'
-            ))
+            ), capture_output=True, env=env)
             # XXX Are there any other sanity checks we should run?
             if idb_compare_eq:
-                self.assertEqual(self.countOutput(r'Block: Interface Description Block',
-                    proc=tshark_proc), check_val['idb_count'])
+                assert count_output(tshark_proc.stdout, r'Block \d+: Interface Description Block \d+') \
+                        == check_val['idb_count']
             else:
-                self.assertGreaterEqual(self.countOutput(r'Block: Interface Description Block',
-                    proc=tshark_proc), check_val['idb_count'])
+                assert count_output(tshark_proc.stdout, r'Block \d+: Interface Description Block \d+') \
+                        >= check_val['idb_count']
                 idb_compare_eq = True
-            self.assertEqual(self.countOutput(r'Option: User Application = Passthrough test #1',
-                proc=tshark_proc), check_val['ua_pt1_count'])
-            self.assertEqual(self.countOutput(r'Option: User Application = Passthrough test #2',
-                proc=tshark_proc), check_val['ua_pt2_count'])
-            self.assertEqual(self.countOutput(r'Option: User Application = Passthrough test #3',
-                proc=tshark_proc), check_val['ua_pt3_count'])
-            self.assertEqual(self.countOutput(r'Option: User Application = Dumpcap \(Wireshark\)',
-                proc=tshark_proc), check_val['ua_dc_count'])
+            assert count_output(tshark_proc.stdout, r'Option: User Application = Passthrough test #1') \
+                        == check_val['ua_pt1_count']
+            assert count_output(tshark_proc.stdout, r'Option: User Application = Passthrough test #2') \
+                        == check_val['ua_pt2_count']
+            assert count_output(tshark_proc.stdout, r'Option: User Application = Passthrough test #3') \
+                        == check_val['ua_pt3_count']
+            assert count_output(tshark_proc.stdout, r'Option: User Application = Dumpcap \(Wireshark\)') \
+                        == check_val['ua_dc_count']
     return check_dumpcap_pcapng_sections_real
 
 
-@fixtures.mark_usefixtures('test_env')
-@fixtures.uses_fixtures
-class case_wireshark_capture(subprocesstest.SubprocessTestCase):
-    def test_wireshark_capture_10_packets_to_file(self, wireshark_k, check_capture_10_packets, make_screenshot_on_error):
+class TestWiresharkCapture:
+    def test_wireshark_capture_10_packets_to_file(self, request, wireshark_k, check_capture_10_packets, make_screenshot_on_error, test_env):
         '''Capture 10 packets from the network to a file using Wireshark'''
+        disabled = request.config.getoption('--disable-gui', default=False)
+        if disabled:
+            pytest.skip('GUI tests are disabled via --disable-gui')
         with make_screenshot_on_error():
-            check_capture_10_packets(self, cmd=wireshark_k)
+            check_capture_10_packets(self, cmd=wireshark_k, env=test_env)
 
     # Wireshark doesn't currently support writing to stdout while capturing.
     # def test_wireshark_capture_10_packets_to_stdout(self, wireshark_k, check_capture_10_packets):
     #     '''Capture 10 packets from the network to stdout using Wireshark'''
     #     check_capture_10_packets(self, cmd=wireshark_k, to_stdout=True)
 
-    def test_wireshark_capture_from_fifo(self, wireshark_k, check_capture_fifo, make_screenshot_on_error):
+    def test_wireshark_capture_from_fifo(self, request, wireshark_k, check_capture_fifo, make_screenshot_on_error, test_env):
         '''Capture from a fifo using Wireshark'''
+        disabled = request.config.getoption('--disable-gui', default=False)
+        if disabled:
+            pytest.skip('GUI tests are disabled via --disable-gui')
         with make_screenshot_on_error():
-            check_capture_fifo(self, cmd=wireshark_k)
+            check_capture_fifo(self, cmd=wireshark_k, env=test_env)
 
-    def test_wireshark_capture_from_stdin(self, wireshark_k, check_capture_stdin, make_screenshot_on_error):
+    def test_wireshark_capture_from_stdin(self, request, wireshark_k, check_capture_stdin, make_screenshot_on_error, test_env):
         '''Capture from stdin using Wireshark'''
+        disabled = request.config.getoption('--disable-gui', default=False)
+        if disabled:
+            pytest.skip('GUI tests are disabled via --disable-gui')
         with make_screenshot_on_error():
-            check_capture_stdin(self, cmd=wireshark_k)
+            check_capture_stdin(self, cmd=wireshark_k, env=test_env)
 
-    def test_wireshark_capture_snapshot_len(self, wireshark_k, check_capture_snapshot_len, make_screenshot_on_error):
+    def test_wireshark_capture_snapshot_len(self, request, wireshark_k, check_capture_snapshot_len, make_screenshot_on_error, test_env):
         '''Capture truncated packets using Wireshark'''
+        disabled = request.config.getoption('--disable-gui', default=False)
+        if disabled:
+            pytest.skip('GUI tests are disabled via --disable-gui')
         with make_screenshot_on_error():
-            check_capture_snapshot_len(self, cmd=wireshark_k)
+            check_capture_snapshot_len(self, cmd=wireshark_k, env=test_env)
 
 
-@fixtures.mark_usefixtures('test_env')
-@fixtures.uses_fixtures
-class case_tshark_capture(subprocesstest.SubprocessTestCase):
-    def test_tshark_capture_10_packets_to_file(self, cmd_tshark, check_capture_10_packets):
+class TestTsharkCapture:
+    def test_tshark_capture_10_packets_to_file(self, cmd_tshark, check_capture_10_packets, test_env):
         '''Capture 10 packets from the network to a file using TShark'''
-        check_capture_10_packets(self, cmd=cmd_tshark)
+        check_capture_10_packets(self, cmd=cmd_tshark, env=test_env)
 
-    def test_tshark_capture_10_packets_to_stdout(self, cmd_tshark, check_capture_10_packets):
+    def test_tshark_capture_10_packets_to_stdout(self, cmd_tshark, check_capture_10_packets, test_env):
         '''Capture 10 packets from the network to stdout using TShark'''
-        check_capture_10_packets(self, cmd=cmd_tshark, to_stdout=True)
+        check_capture_10_packets(self, cmd=cmd_tshark, to_stdout=True, env=test_env)
 
-    def test_tshark_capture_from_fifo(self, cmd_tshark, check_capture_fifo):
+    def test_tshark_capture_from_fifo(self, cmd_tshark, check_capture_fifo, test_env):
         '''Capture from a fifo using TShark'''
-        check_capture_fifo(self, cmd=cmd_tshark)
+        check_capture_fifo(self, cmd=cmd_tshark, env=test_env)
 
-    def test_tshark_capture_from_stdin(self, cmd_tshark, check_capture_stdin):
+    def test_tshark_capture_from_stdin(self, cmd_tshark, check_capture_stdin, test_env):
         '''Capture from stdin using TShark'''
-        check_capture_stdin(self, cmd=cmd_tshark)
+        check_capture_stdin(self, cmd=cmd_tshark, env=test_env)
 
-    def test_tshark_capture_snapshot_len(self, cmd_tshark, check_capture_snapshot_len):
+    def test_tshark_capture_snapshot_len(self, cmd_tshark, check_capture_snapshot_len, test_env):
         '''Capture truncated packets using TShark'''
-        check_capture_snapshot_len(self, cmd=cmd_tshark)
+        check_capture_snapshot_len(self, cmd=cmd_tshark, env=test_env)
 
 
-@fixtures.mark_usefixtures('base_env')
-@fixtures.uses_fixtures
-class case_dumpcap_capture(subprocesstest.SubprocessTestCase):
-    def test_dumpcap_capture_10_packets_to_file(self, cmd_dumpcap, check_capture_10_packets):
+class TestDumpcapCapture:
+    def test_dumpcap_capture_10_packets_to_file(self, cmd_dumpcap, check_capture_10_packets, base_env):
         '''Capture 10 packets from the network to a file using Dumpcap'''
-        check_capture_10_packets(self, cmd=cmd_dumpcap)
+        check_capture_10_packets(self, cmd=cmd_dumpcap, env=base_env)
 
-    def test_dumpcap_capture_10_packets_to_stdout(self, cmd_dumpcap, check_capture_10_packets):
+    def test_dumpcap_capture_10_packets_to_stdout(self, cmd_dumpcap, check_capture_10_packets, base_env):
         '''Capture 10 packets from the network to stdout using Dumpcap'''
-        check_capture_10_packets(self, cmd=cmd_dumpcap, to_stdout=True)
+        check_capture_10_packets(self, cmd=cmd_dumpcap, to_stdout=True, env=base_env)
 
-    def test_dumpcap_capture_from_fifo(self, cmd_dumpcap, check_capture_fifo):
+    def test_dumpcap_capture_from_fifo(self, cmd_dumpcap, check_capture_fifo, base_env):
         '''Capture from a fifo using Dumpcap'''
-        check_capture_fifo(self, cmd=cmd_dumpcap)
+        check_capture_fifo(self, cmd=cmd_dumpcap, env=base_env)
 
-    def test_dumpcap_capture_from_stdin(self, cmd_dumpcap, check_capture_stdin):
+    def test_dumpcap_capture_from_stdin(self, cmd_dumpcap, check_capture_stdin, base_env):
         '''Capture from stdin using Dumpcap'''
-        check_capture_stdin(self, cmd=cmd_dumpcap)
+        check_capture_stdin(self, cmd=cmd_dumpcap, env=base_env)
 
-    def test_dumpcap_capture_snapshot_len(self, check_capture_snapshot_len, cmd_dumpcap):
+    def test_dumpcap_capture_snapshot_len(self, check_capture_snapshot_len, cmd_dumpcap, base_env):
         '''Capture truncated packets using Dumpcap'''
-        check_capture_snapshot_len(self, cmd=cmd_dumpcap)
+        check_capture_snapshot_len(self, cmd=cmd_dumpcap, env=base_env)
 
 
-@fixtures.mark_usefixtures('base_env')
-@fixtures.uses_fixtures
-class case_dumpcap_autostop(subprocesstest.SubprocessTestCase):
+class TestDumpcapAutostop:
     # duration, filesize, packets, files
-    def test_dumpcap_autostop_filesize(self, check_dumpcap_autostop_stdin):
+    def test_dumpcap_autostop_filesize(self, check_dumpcap_autostop_stdin, base_env):
         '''Capture from stdin using Dumpcap until we reach a file size limit'''
-        check_dumpcap_autostop_stdin(self, filesize=15)
+        check_dumpcap_autostop_stdin(self, filesize=15, env=base_env)
 
-    def test_dumpcap_autostop_packets(self, check_dumpcap_autostop_stdin):
+    def test_dumpcap_autostop_packets(self, check_dumpcap_autostop_stdin, base_env):
         '''Capture from stdin using Dumpcap until we reach a packet limit'''
-        check_dumpcap_autostop_stdin(self, packets=97) # Last prime before 100. Arbitrary.
+        check_dumpcap_autostop_stdin(self, packets=97, env=base_env) # Last prime before 100. Arbitrary.
 
 
-@fixtures.mark_usefixtures('base_env')
-@fixtures.uses_fixtures
-class case_dumpcap_ringbuffer(subprocesstest.SubprocessTestCase):
+class TestDumpcapRingbuffer:
     # duration, interval, filesize, packets, files
-    def test_dumpcap_ringbuffer_filesize(self, check_dumpcap_ringbuffer_stdin):
+    def test_dumpcap_ringbuffer_filesize(self, check_dumpcap_ringbuffer_stdin, base_env):
         '''Capture from stdin using Dumpcap and write multiple files until we reach a file size limit'''
-        check_dumpcap_ringbuffer_stdin(self, filesize=15)
+        check_dumpcap_ringbuffer_stdin(self, filesize=15, env=base_env)
 
-    def test_dumpcap_ringbuffer_packets(self, check_dumpcap_ringbuffer_stdin):
+    def test_dumpcap_ringbuffer_packets(self, check_dumpcap_ringbuffer_stdin, base_env):
         '''Capture from stdin using Dumpcap and write multiple files until we reach a packet limit'''
-        check_dumpcap_ringbuffer_stdin(self, packets=47) # Last prime before 50. Arbitrary.
+        check_dumpcap_ringbuffer_stdin(self, packets=47, env=base_env) # Last prime before 50. Arbitrary.
 
 
-@fixtures.mark_usefixtures('base_env')
-@fixtures.uses_fixtures
-class case_dumpcap_pcapng_sections(subprocesstest.SubprocessTestCase):
-    def test_dumpcap_pcapng_single_in_single_out(self, check_dumpcap_pcapng_sections):
+class TestDumpcapPcapngSections:
+    def test_dumpcap_pcapng_single_in_single_out(self, check_dumpcap_pcapng_sections, base_env):
         '''Capture from a single pcapng source using Dumpcap and write a single file'''
         if sys.byteorder == 'big':
-            fixtures.skip('this test is supported on little endian only')
-        check_dumpcap_pcapng_sections(self)
+            pytest.skip('this test is supported on little endian only')
+        check_dumpcap_pcapng_sections(self, env=base_env)
 
-    def test_dumpcap_pcapng_single_in_multi_out(self, check_dumpcap_pcapng_sections):
+    def test_dumpcap_pcapng_single_in_multi_out(self, check_dumpcap_pcapng_sections, base_env):
         '''Capture from a single pcapng source using Dumpcap and write two files'''
         if sys.byteorder == 'big':
-            fixtures.skip('this test is supported on little endian only')
-        check_dumpcap_pcapng_sections(self, multi_output=True)
+            pytest.skip('this test is supported on little endian only')
+        check_dumpcap_pcapng_sections(self, multi_output=True, env=base_env)
 
-    def test_dumpcap_pcapng_multi_in_single_out(self, check_dumpcap_pcapng_sections):
+    def test_dumpcap_pcapng_multi_in_single_out(self, check_dumpcap_pcapng_sections, base_env):
         '''Capture from two pcapng sources using Dumpcap and write a single file'''
         if sys.byteorder == 'big':
-            fixtures.skip('this test is supported on little endian only')
-        check_dumpcap_pcapng_sections(self, multi_input=True)
+            pytest.skip('this test is supported on little endian only')
+        check_dumpcap_pcapng_sections(self, multi_input=True, env=base_env)
 
-    def test_dumpcap_pcapng_multi_in_multi_out(self, check_dumpcap_pcapng_sections):
+    def test_dumpcap_pcapng_multi_in_multi_out(self, check_dumpcap_pcapng_sections, base_env):
         '''Capture from two pcapng sources using Dumpcap and write two files'''
         if sys.byteorder == 'big':
-            fixtures.skip('this test is supported on little endian only')
-        check_dumpcap_pcapng_sections(self, multi_input=True, multi_output=True)
+            pytest.skip('this test is supported on little endian only')
+        check_dumpcap_pcapng_sections(self, multi_input=True, multi_output=True, env=base_env)
