@@ -49,6 +49,9 @@
 #include <ui/qt/utils/qt_ui_utils.h>
 #include <main_application.h>
 
+static const double WIDTH_SCALE_FACTOR = 1.4;
+static const double HEIGHT_SCALE_FACTOR = 1.4;
+
 CaptureFileDialog::CaptureFileDialog(QWidget *parent, capture_file *cf) :
     WiresharkFileDialog(parent),
     cap_file_(cf),
@@ -63,13 +66,14 @@ CaptureFileDialog::CaptureFileDialog(QWidget *parent, capture_file *cf) :
 {
     switch (prefs.gui_fileopen_style) {
     case FO_STYLE_LAST_OPENED:
-        /* The user has specified that we should start out in the last directory
-         * we looked in.  If we've already opened a file, use its containing
-         * directory, if we could determine it, as the directory, otherwise
-         * use the "last opened" directory saved in the preferences file if
-         * there was one.
+        /* The user has specified that we should start out in the last
+         * directory in which we opened a file.
+         *
+         * The open dialog initial directory will be that directory
+         * unless we've never opened a file, in which case it will
+         * be the user's personal data file directory.
          */
-        setDirectory(mainApp->lastOpenDir());
+        setDirectory(mainApp->openDialogInitialDir());
         break;
 
     case FO_STYLE_SPECIFIED:
@@ -700,7 +704,9 @@ QDialogButtonBox *CaptureFileDialog::addHelpButton(topic_action_e help_topic)
 
 int CaptureFileDialog::open(QString &file_name, unsigned int &type, QString &display_filter) {
     setWindowTitle(mainApp->windowTitleString(tr("Open Capture File")));
-    setNameFilters(buildFileOpenTypeList());
+    QStringList open_type_filters(buildFileOpenTypeList());
+    setNameFilters(open_type_filters);
+    selectNameFilter(open_type_filters.at(1));
     setFileMode(QFileDialog::ExistingFile);
 
     addFormatTypeSelector(left_v_box_);
@@ -709,7 +715,7 @@ int CaptureFileDialog::open(QString &file_name, unsigned int &type, QString &dis
     addHelpButton(HELP_OPEN_DIALOG);
 
     // Grow the dialog to account for the extra widgets.
-    resize(width(), height() + left_v_box_.minimumSize().height() + display_filter_edit_->minimumSize().height());
+    resize(width() * WIDTH_SCALE_FACTOR, height() * HEIGHT_SCALE_FACTOR + left_v_box_.minimumSize().height() + display_filter_edit_->minimumSize().height());
 
     display_filter.clear();
 
@@ -741,7 +747,7 @@ check_savability_t CaptureFileDialog::saveAs(QString &file_name, bool must_suppo
     addHelpButton(HELP_SAVE_DIALOG);
 
     // Grow the dialog to account for the extra widgets.
-    resize(width(), height() + left_v_box_.minimumSize().height());
+    resize(width() * WIDTH_SCALE_FACTOR, height() * HEIGHT_SCALE_FACTOR + left_v_box_.minimumSize().height());
 
     if (!file_name.isEmpty()) {
         selectFile(file_name);
@@ -791,7 +797,7 @@ check_savability_t CaptureFileDialog::exportSelectedPackets(QString &file_name,
     }
 
     // Grow the dialog to account for the extra widgets.
-    resize(width(), height() + (packet_range_group_box_.height() * 2 / 3));
+    resize(width() * WIDTH_SCALE_FACTOR, height() * HEIGHT_SCALE_FACTOR + (packet_range_group_box_.height() * 2 / 3));
 
     if (!file_name.isEmpty()) {
         selectFile(file_name);
@@ -833,7 +839,7 @@ int CaptureFileDialog::merge(QString &file_name, QString &display_filter) {
     display_filter.clear();
 
     // Grow the dialog to account for the extra widgets.
-    resize(width(), height() + right_v_box_.minimumSize().height() + display_filter_edit_->minimumSize().height());
+    resize(width() * WIDTH_SCALE_FACTOR, height() * HEIGHT_SCALE_FACTOR + right_v_box_.minimumSize().height() + display_filter_edit_->minimumSize().height());
 
     if (WiresharkFileDialog::exec() && selectedFiles().length() > 0) {
         file_name.append(selectedFiles()[0]);
