@@ -9,7 +9,7 @@
 
 #include "config.h"
 
-#include <glib.h>
+#include <wireshark.h>
 
 #include "capture_options_dialog.h"
 #include <ui/qt/widgets/capture_filter_combo.h>
@@ -31,12 +31,13 @@
 #include "ui/capture_ui_utils.h"
 #include "ui/capture_globals.h"
 #include "ui/iface_lists.h"
-#include "ui/last_open_dir.h"
+#include "ui/file_dialog.h"
 
 #include "ui/ws_ui_util.h"
 #include "ui/util.h"
 #include <wsutil/utf8_entities.h>
 #include "ui/preference_utils.h"
+#include "ui/recent.h"
 
 #include <cstdio>
 #include <epan/prefs.h>
@@ -57,6 +58,11 @@
 // - Fix InterfaceTreeDelegate method names.
 // - You can edit filters via the main CaptureFilterCombo and via each
 //   individual interface row. We should probably do one or the other.
+// - There might be a point in having the separate combo boxes in the
+//   individual interface row, if their CaptureFilterCombos actually
+//   called recent_get_cfilter_list with the interface name to get the
+//   separate list of recent capture filters for that interface, but
+//   they don't.
 
 const int stat_update_interval_ = 1000; // ms
 
@@ -111,6 +117,9 @@ public:
     {
         if (!device) return;
 
+        // Prevent infinite recursive signal loop
+        // itemChanged->interfaceItemChanged->updateInterfaceColumns
+        treeWidget()->blockSignals(true);
         QString default_str = QObject::tr("default");
 
         // XXX - this is duplicated in InterfaceTreeModel::data;
@@ -158,6 +167,7 @@ public:
             setApplicable(col_monitor_, false);
         }
 #endif
+        treeWidget()->blockSignals(false);
     }
 
     void setApplicable(int column, bool applicable = false) {
@@ -365,12 +375,12 @@ void CaptureOptionsDialog::on_capturePromModeCheckBox_toggled(bool checked)
 
 void CaptureOptionsDialog::browseButtonClicked()
 {
-    char *open_dir = NULL;
+    const char *open_dir = NULL;
 
     switch (prefs.gui_fileopen_style) {
 
     case FO_STYLE_LAST_OPENED:
-        open_dir = get_last_open_dir();
+        open_dir = get_open_dialog_initial_dir();
         break;
 
     case FO_STYLE_SPECIFIED:
@@ -504,7 +514,7 @@ void CaptureOptionsDialog::itemClicked(QTreeWidgetItem *item, int column)
         if (device->if_info.type == IF_EXTCAP) {
             /* this checks if configuration is required and not yet provided or saved via prefs */
             QString device_name = ti->data(col_extcap_, Qt::UserRole).value<QString>();
-            if (extcap_has_configuration((const char *)(device_name.toStdString().c_str()), FALSE))
+            if (extcap_has_configuration((const char *)(device_name.toStdString().c_str())))
             {
                 emit showExtcapOptions(device_name, false);
                 return;
@@ -538,7 +548,7 @@ void CaptureOptionsDialog::itemDoubleClicked(QTreeWidgetItem *item, int column)
         if (device->if_info.type == IF_EXTCAP) {
             /* this checks if configuration is required and not yet provided or saved via prefs */
             QString device_name = ti->data(col_extcap_, Qt::UserRole).value<QString>();
-            if (extcap_has_configuration((const char *)(device_name.toStdString().c_str()), TRUE))
+            if (extcap_requires_configuration((const char *)(device_name.toStdString().c_str())))
             {
                 emit showExtcapOptions(device_name, true);
                 return;
@@ -582,7 +592,7 @@ void CaptureOptionsDialog::on_cbUpdatePacketsRT_toggled(bool checked)
 
 void CaptureOptionsDialog::on_cbAutoScroll_toggled(bool checked)
 {
-    auto_scroll_live = checked;
+    recent.capture_auto_scroll = checked;
 }
 
 void CaptureOptionsDialog::on_cbExtraCaptureInfo_toggled(bool checked)
@@ -619,7 +629,7 @@ void CaptureOptionsDialog::on_buttonBox_accepted()
             if (device && device->if_info.type == IF_EXTCAP) {
                 /* this checks if configuration is required and not yet provided or saved via prefs */
                 QString device_name = ti->data(col_extcap_, Qt::UserRole).value<QString>();
-                if (extcap_has_configuration((const char *)(device_name.toStdString().c_str()), TRUE))
+                if (extcap_requires_configuration((const char *)(device_name.toStdString().c_str())))
                 {
                     emit showExtcapOptions(device_name, true);
                     return;
@@ -759,7 +769,7 @@ void CaptureOptionsDialog::updateInterfaces()
     }
 
     ui->cbUpdatePacketsRT->setChecked(global_capture_opts.real_time_mode);
-    ui->cbAutoScroll->setChecked(true);
+    ui->cbAutoScroll->setChecked(recent.capture_auto_scroll);
     ui->cbExtraCaptureInfo->setChecked(global_capture_opts.show_info);
 
     ui->cbResolveMacAddresses->setChecked(gbl_resolv_flags.mac_name);
@@ -1402,7 +1412,7 @@ QWidget* InterfaceTreeDelegate::createEditor(QWidget *parent, const QStyleOption
             QComboBox *cb = new QComboBox(parent);
             cb->addItems(valid_link_types);
 
-            connect(cb, SIGNAL(currentIndexChanged(QString)), this, SLOT(linkTypeChanged(QString)));
+            connect(cb, &QComboBox::currentTextChanged, this, &InterfaceTreeDelegate::linkTypeChanged);
             w = (QWidget*) cb;
             break;
         }
@@ -1430,6 +1440,8 @@ QWidget* InterfaceTreeDelegate::createEditor(QWidget *parent, const QStyleOption
 #endif
         case col_filter_:
         {
+            // XXX: Should this take the interface name, so that the history
+            // list is taken from the interface-specific recent cfilter list?
             CaptureFilterCombo *cf = new CaptureFilterCombo(parent, true);
             connect(cf->lineEdit(), SIGNAL(textEdited(QString)), this, SIGNAL(filterChanged(QString)));
             w = (QWidget*) cf;
@@ -1457,7 +1469,7 @@ bool InterfaceTreeDelegate::eventFilter(QObject *object, QEvent *event)
     return false;
 }
 
-void InterfaceTreeDelegate::linkTypeChanged(QString selected_link_type)
+void InterfaceTreeDelegate::linkTypeChanged(const QString selected_link_type)
 {
     GList *list;
     link_row *temp;
