@@ -32,7 +32,7 @@
 
 static QString formatString(qlonglong value)
 {
-    return QLocale::system().formattedDataSize(value, 0, QLocale::DataSizeSIFormat);
+    return QLocale().formattedDataSize(value, 0, QLocale::DataSizeSIFormat);
 }
 
 ATapDataModel::ATapDataModel(dataModelType type, int protoId, QString filter, QObject *parent):
@@ -61,7 +61,14 @@ ATapDataModel::ATapDataModel(dataModelType type, int protoId, QString filter, QO
 
 ATapDataModel::~ATapDataModel()
 {
-    remove_tap_listener(hash());
+    /* Only remove the tap if we come from a enabled model */
+    if (!_disableTap)
+        remove_tap_listener(hash());
+
+    if (_type == ATapDataModel::DATAMODEL_ENDPOINT)
+        reset_endpoint_table_data(&hash_);
+    else if (_type == ATapDataModel::DATAMODEL_CONVERSATION)
+        reset_conversation_table_data(&hash_);
 }
 
 int ATapDataModel::protoId() const
@@ -359,6 +366,10 @@ QVariant EndpointDataModel::headerData(int section, Qt::Orientation orientation,
                 return tr("Country"); break;
             case ENDP_COLUMN_GEO_CITY:
                 return tr("City"); break;
+            case ENDP_COLUMN_GEO_LATITUDE:
+                return tr("Latitude"); break;
+            case ENDP_COLUMN_GEO_LONGITUDE:
+                return tr("Longitude"); break;
             case ENDP_COLUMN_GEO_AS_NUM:
                 return tr("AS Number"); break;
             case ENDP_COLUMN_GEO_AS_ORG:
@@ -443,9 +454,14 @@ QVariant EndpointDataModel::data(const QModelIndex &idx, int role) const
                 qlonglong totalPackets = (qlonglong)(item->tx_frames_total + item->rx_frames_total);
                 qlonglong packets = (qlonglong)(item->tx_frames + item->rx_frames);
                 percent = totalPackets == 0 ? 0 : (double) packets * 100 / (double) totalPackets;
-                return QString::number(percent, 'f', 2) + "%";
             }
-            return role == Qt::DisplayRole ? QString::number(percent, 'f', 2) + "%" : (QVariant)percent;
+            QString rounded = QString::number(percent, 'f', 2);
+            /* Qt guarantees that this roundtrip conversion compares equally,
+             * so filtering with equality will work as expected.
+             * Perhaps the UNFORMATTED_DISPLAYDATA role shoud be split
+             * into one used for raw data export, and one used for comparisons.
+             */
+            return role == Qt::DisplayRole ? rounded + "%" : QVariant(rounded.toDouble());
         }
         case ENDP_COLUMN_PKT_AB:
             return role == Qt::DisplayRole ? QString("%L1").arg((qlonglong)item->tx_frames) : QVariant((qlonglong) item->tx_frames);
@@ -465,6 +481,16 @@ QVariant EndpointDataModel::data(const QModelIndex &idx, int role) const
                 return QVariant(mmdb_lookup->city);
             }
             return QVariant();
+        case ENDP_COLUMN_GEO_LATITUDE:
+            if (mmdb_lookup && mmdb_lookup->found && mmdb_lookup->latitude >= -90.0 && mmdb_lookup->latitude <= 90.0) {
+                return role == Qt::DisplayRole ? QString("%L1" UTF8_DEGREE_SIGN).arg(mmdb_lookup->latitude) : QVariant(mmdb_lookup->latitude);
+            }
+            return QVariant();
+        case ENDP_COLUMN_GEO_LONGITUDE:
+            if (mmdb_lookup && mmdb_lookup->found && mmdb_lookup->longitude >= -180.0 && mmdb_lookup->longitude <= 180.0) {
+                return role == Qt::DisplayRole ? QString("%L1" UTF8_DEGREE_SIGN).arg(mmdb_lookup->longitude) : QVariant(mmdb_lookup->longitude);
+            }
+            return QVariant();
         case ENDP_COLUMN_GEO_AS_NUM:
             if (mmdb_lookup && mmdb_lookup->found && mmdb_lookup->as_number) {
                 return QVariant(mmdb_lookup->as_number);
@@ -503,7 +529,9 @@ QVariant EndpointDataModel::data(const QModelIndex &idx, int role) const
         return ipAddress;
     }
 #endif
-    else if (role == ATapDataModel::DATA_ADDRESS_TYPE) {
+    else if (role == ATapDataModel::PROTO_ID) {
+        return protoId();
+    } else if (role == ATapDataModel::DATA_ADDRESS_TYPE) {
         if (idx.column() == EndpointDataModel::ENDP_COLUMN_ADDR)
             return (int)item->myaddress.type;
         return (int) AT_NONE;
@@ -690,7 +718,14 @@ QVariant ConversationDataModel::data(const QModelIndex &idx, int role) const
                 qlonglong packets = (qlonglong)(conv_item->tx_frames + conv_item->rx_frames);
                 percent = totalPackets == 0 ? 0 : (double) packets * 100 / (double) totalPackets;
             }
-            return role == Qt::DisplayRole ? QString::number(percent, 'f', 2) + "%" : (QVariant)percent;
+            QString rounded = QString::number(percent, 'f', 2);
+            /* Qt guarantees that this roundtrip conversion compares equally,
+             * so filtering with equality will work as expected.
+             * XXX: Perhaps the UNFORMATTED_DISPLAYDATA role shoud be split
+             * into one used for raw data export and comparisions with each
+             * other, and another for comparing with filters?
+             */
+            return role == Qt::DisplayRole ? rounded + "%" : QVariant(rounded.toDouble());
         }
         case CONV_COLUMN_PKT_AB:
         {
@@ -712,8 +747,21 @@ QVariant ConversationDataModel::data(const QModelIndex &idx, int role) const
 
             if (_absoluteTime) {
                 nstime_t *abs_time = &conv_item->start_abs_time;
+                /* XXX: QDateTime only supports millisecond resolution,
+                 * and we have microseconds or nanoseconds.
+                 * Should we use something else, particularly for exporting
+                 * raw data? GDateTime handles microseconds.
+                 */
                 QDateTime abs_dt = QDateTime::fromMSecsSinceEpoch(nstime_to_msec(abs_time));
-                return role == Qt::DisplayRole ? abs_dt.toString("hh:mm:ss.zzzz") : (QVariant)abs_dt;
+                if (role == Qt::DisplayRole) {
+                    if (_maxRelStopTime >= 24*60*60) {
+                        return abs_dt.toString(Qt::ISODateWithMs);
+                    } else {
+                        return abs_dt.time().toString(Qt::ISODateWithMs);
+                    }
+                } else {
+                    return QVariant(abs_dt);
+                }
             } else {
                 return role == Qt::DisplayRole ?
                     QString::number(nstime_to_sec(&conv_item->start_time), 'f', width) :
@@ -751,6 +799,8 @@ QVariant ConversationDataModel::data(const QModelIndex &idx, int role) const
         }
     } else if (role == ATapDataModel::ENDPOINT_DATATYPE) {
         return (int)(conv_item->ctype);
+    } else if (role == ATapDataModel::PROTO_ID) {
+        return protoId();
     } else if (role == ATapDataModel::CONVERSATION_ID) {
         return (int)(conv_item->conv_id);
     } else if (role == ATapDataModel::ROW_IS_FILTERED) {
@@ -790,7 +840,7 @@ conv_item_t * ConversationDataModel::itemForRow(int row)
 
 bool ConversationDataModel::showConversationId(int row) const
 {
-    if (!storage_ || row >= (int) storage_->len)
+    if (!storage_ || row < 0 || row >= (int) storage_->len)
         return false;
 
     conv_item_t *conv_item = (conv_item_t *)&g_array_index(storage_, conv_item_t, row);
