@@ -10,6 +10,7 @@
 #ifndef PROTO_TREE_MODEL_H
 #define PROTO_TREE_MODEL_H
 
+#include <ui/qt/utils/field_information.h>
 #include <ui/qt/utils/proto_node.h>
 
 #include <QAbstractItemModel>
@@ -21,6 +22,7 @@ class ProtoTreeModel : public QAbstractItemModel
 
 public:
     explicit ProtoTreeModel(QObject * parent = 0);
+    ~ProtoTreeModel();
 
     virtual Qt::ItemFlags flags(const QModelIndex &index) const;
     QModelIndex index(int row, int, const QModelIndex &parent = QModelIndex()) const;
@@ -31,16 +33,16 @@ public:
 
     // root_node can be NULL.
     void setRootNode(proto_node *root_node);
-    ProtoNode protoNodeFromIndex(const QModelIndex &index) const;
-    QModelIndex indexFromProtoNode(ProtoNode &index_node) const;
+    ProtoNode* protoNodeFromIndex(const QModelIndex &index) const;
+    QModelIndex indexFromProtoNode(ProtoNode *index_node) const;
 
     QModelIndex findFirstHfid(int hf_id);
     QModelIndex findFieldInformation(FieldInformation *finfo);
 
 private:
-    proto_node* root_node_;
-    static void foreachFindHfid(proto_node *node, gpointer find_hfid_ptr);
-    static void foreachFindField(proto_node *node, gpointer find_finfo_ptr);
+    ProtoNode *root_node_;
+    static bool foreachFindHfid(ProtoNode *node, gpointer find_hfid_ptr);
+    static bool foreachFindField(ProtoNode *node, gpointer find_finfo_ptr);
 };
 
 #endif // PROTO_TREE_MODEL_H
