@@ -32,6 +32,14 @@ PacketRangeGroupBox::~PacketRangeGroupBox()
 void PacketRangeGroupBox::initRange(packet_range_t *range, QString selRange) {
     if (!range) return;
 
+    range_ = nullptr;
+    // Set this before setting range_ so that on_dependedCheckBox_toggled
+    // doesn't trigger an extra updateCounts().
+    // (We could instead manually connect and disconnect signals and slots
+    // after getting a range, instead of checking for range_ in all the
+    // slots.)
+    pr_ui_->dependedCheckBox->setChecked(range->include_dependents);
+
     range_ = range;
 
     if (range_->process_filtered) {
@@ -64,6 +72,7 @@ void PacketRangeGroupBox::updateCounts() {
     bool can_select;
     bool selected_packets;
     int ignored_cnt = 0, displayed_ignored_cnt = 0;
+    int depended_cnt = 0, displayed_depended_cnt = 0;
     int label_count;
 
     if (!range_ || !range_->cf) return;
@@ -103,8 +112,13 @@ void PacketRangeGroupBox::updateCounts() {
         pr_ui_->selectedCapturedLabel->setEnabled(!displayed_checked);
         pr_ui_->selectedDisplayedLabel->setEnabled(displayed_checked);
 
-        pr_ui_->selectedCapturedLabel->setText(QString::number(range_->selection_range_cnt));
-        pr_ui_->selectedDisplayedLabel->setText(QString::number(range_->displayed_selection_range_cnt));
+        if (range_->include_dependents) {
+            pr_ui_->selectedCapturedLabel->setText(QString::number(range_->selected_plus_depends_cnt));
+            pr_ui_->selectedDisplayedLabel->setText(QString::number(range_->displayed_selected_plus_depends_cnt));
+        } else {
+            pr_ui_->selectedCapturedLabel->setText(QString::number(range_->selection_range_cnt));
+            pr_ui_->selectedDisplayedLabel->setText(QString::number(range_->displayed_selection_range_cnt));
+        }
     } else {
         if (range_->process == range_process_selected) {
             pr_ui_->allButton->setChecked(true);
@@ -135,12 +149,20 @@ void PacketRangeGroupBox::updateCounts() {
         pr_ui_->markedCapturedLabel->setEnabled(false);
         pr_ui_->markedDisplayedLabel->setEnabled(false);
     }
-    label_count = range_->cf->marked_count;
+    if (range_->include_dependents) {
+        label_count = range_->marked_plus_depends_cnt;
+    } else {
+        label_count = range_->cf->marked_count;
+    }
     if (range_->remove_ignored) {
         label_count -= range_->ignored_marked_cnt;
     }
     pr_ui_->markedCapturedLabel->setText(QString("%1").arg(label_count));
-    label_count = range_->displayed_marked_cnt;
+    if (range_->include_dependents) {
+        label_count = range_->displayed_marked_plus_depends_cnt;
+    } else {
+        label_count = range_->displayed_marked_cnt;
+    }
     if (range_->remove_ignored) {
         label_count -= range_->displayed_ignored_marked_cnt;
     }
@@ -164,12 +186,20 @@ void PacketRangeGroupBox::updateCounts() {
         pr_ui_->ftlCapturedLabel->setEnabled(false);
         pr_ui_->ftlDisplayedLabel->setEnabled(false);
     }
-    label_count = range_->mark_range_cnt;
+    if (range_->include_dependents) {
+        label_count = range_->mark_range_plus_depends_cnt;
+    } else {
+        label_count = range_->mark_range_cnt;
+    }
     if (range_->remove_ignored) {
         label_count -= range_->ignored_mark_range_cnt;
     }
     pr_ui_->ftlCapturedLabel->setText(QString("%1").arg(label_count));
-    label_count = range_->displayed_mark_range_cnt;
+    if (range_->include_dependents) {
+        label_count = range_->displayed_mark_range_plus_depends_cnt;
+    } else {
+        label_count = range_->displayed_mark_range_cnt;
+    }
     if (range_->remove_ignored) {
         label_count -= range_->displayed_ignored_mark_range_cnt;
     }
@@ -186,12 +216,20 @@ void PacketRangeGroupBox::updateCounts() {
     switch (packet_range_check(range_)) {
 
     case CVT_NO_ERROR:
-        label_count = range_->user_range_cnt;
+        if (range_->include_dependents) {
+            label_count = range_->user_range_plus_depends_cnt;
+        } else {
+            label_count = range_->user_range_cnt;
+        }
         if (range_->remove_ignored) {
             label_count -= range_->ignored_user_range_cnt;
         }
         pr_ui_->rangeCapturedLabel->setText(QString("%1").arg(label_count));
-        label_count = range_->displayed_user_range_cnt;
+        if (range_->include_dependents) {
+            label_count = range_->displayed_user_range_plus_depends_cnt;
+        } else {
+            label_count = range_->displayed_user_range_cnt;
+        }
         if (range_->remove_ignored) {
             label_count -= range_->displayed_ignored_user_range_cnt;
         }
@@ -259,6 +297,52 @@ void PacketRangeGroupBox::updateCounts() {
     pr_ui_->ignoredCapturedLabel->setText(QString("%1").arg(ignored_cnt));
     pr_ui_->ignoredDisplayedLabel->setText(QString("%1").arg(displayed_ignored_cnt));
 
+    // Depended upon / Displayed + Captured
+    switch(range_->process) {
+    case(range_process_all):
+        depended_cnt = 0;
+        displayed_depended_cnt = range_->displayed_plus_dependents_cnt - range_->displayed_cnt;
+        break;
+    case(range_process_selected):
+        depended_cnt = range_->selected_plus_depends_cnt - range_->selection_range_cnt;
+        displayed_depended_cnt = range_->displayed_selected_plus_depends_cnt - range_->displayed_selection_range_cnt;
+        break;
+    case(range_process_marked):
+        depended_cnt = range_->marked_plus_depends_cnt - range_->cf->marked_count;
+        displayed_depended_cnt = range_->displayed_marked_plus_depends_cnt - range_->displayed_marked_cnt;
+        break;
+    case(range_process_marked_range):
+        depended_cnt = range_->mark_range_plus_depends_cnt - range_->mark_range_cnt;
+        displayed_depended_cnt = range_->displayed_mark_range_plus_depends_cnt - range_->displayed_mark_range_cnt;
+        break;
+    case(range_process_user_range):
+        depended_cnt = range_->user_range_plus_depends_cnt - range_->user_range_cnt;
+        displayed_depended_cnt = range_->displayed_user_range_plus_depends_cnt - range_->displayed_user_range_cnt;
+        break;
+    default:
+        depended_cnt = 0;
+        displayed_depended_cnt = 0;
+        break;
+    }
+
+    if (displayed_checked) {
+        selected_packets = (displayed_depended_cnt != 0);
+    } else {
+        selected_packets = (depended_cnt != 0);
+    }
+
+    if (selected_packets) {
+        pr_ui_->dependedCheckBox->setEnabled(true);
+        pr_ui_->dependedCapturedLabel->setEnabled(!displayed_checked);
+        pr_ui_->dependedDisplayedLabel->setEnabled(displayed_checked);
+    } else {
+        pr_ui_->dependedCheckBox->setEnabled(false);
+        pr_ui_->dependedCapturedLabel->setEnabled(false);
+        pr_ui_->dependedDisplayedLabel->setEnabled(false);
+    }
+    pr_ui_->dependedCapturedLabel->setText(QString("%1").arg(depended_cnt));
+    pr_ui_->dependedDisplayedLabel->setText(QString("%1").arg(displayed_depended_cnt));
+
     if (orig_ss != syntax_state_) {
         pr_ui_->rangeLineEdit->setSyntaxState(syntax_state_);
         emit validityChanged(isValid());
@@ -330,3 +414,11 @@ void PacketRangeGroupBox::on_ignoredCheckBox_toggled(bool checked)
     if (range_) range_->remove_ignored = checked ? TRUE : FALSE;
     updateCounts();
 }
+
+void PacketRangeGroupBox::on_dependedCheckBox_toggled(bool checked)
+{
+    if (range_) {
+        range_->include_dependents = checked ? TRUE : FALSE;
+        updateCounts();
+    }
+}
