@@ -359,7 +359,7 @@ QToolButton *RtpPlayerDialog::addPlayerButton(QDialogButtonBox *button_box, QDia
     player_button->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
     player_button->setPopupMode(QToolButton::MenuButtonPopup);
 
-    ca = new QAction(tr("&Play Streams"));
+    ca = new QAction(tr("&Play Streams"), player_button);
     ca->setToolTip(tr("Open RTP player dialog"));
     ca->setIcon(StockIcon("media-playback-start"));
     connect(ca, SIGNAL(triggered()), dialog, SLOT(rtpPlayerReplace()));
@@ -1425,7 +1425,7 @@ void RtpPlayerDialog::on_playButton_clicked()
 #if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
     notify_timer_start_diff_ = -1;
 #endif
-    marker_stream_->start(new AudioSilenceGenerator());
+    marker_stream_->start(new AudioSilenceGenerator(marker_stream_));
     // It may happen that stream play is finished before all others are started
     // therefore we do not use playing_streams_ there, but separate temporarly
     // list. It avoids access element/remove element race condition.
@@ -2448,7 +2448,7 @@ save_audio_t RtpPlayerDialog::selectFileAudioFormatAndName(QString *file_path)
 
     QString sel_filter;
     *file_path = WiresharkFileDialog::getSaveFileName(
-                this, tr("Save audio"), mainApp->lastOpenDir().absoluteFilePath(""),
+                this, tr("Save audio"), mainApp->openDialogInitialDir().absoluteFilePath(""),
                 ext_filter, &sel_filter);
 
     if (file_path->isEmpty()) return save_audio_none;
@@ -2471,7 +2471,7 @@ save_payload_t RtpPlayerDialog::selectFilePayloadFormatAndName(QString *file_pat
 
     QString sel_filter;
     *file_path = WiresharkFileDialog::getSaveFileName(
-                this, tr("Save payload"), mainApp->lastOpenDir().absoluteFilePath(""),
+                this, tr("Save payload"), mainApp->openDialogInitialDir().absoluteFilePath(""),
                 ext_filter, &sel_filter);
 
     if (file_path->isEmpty()) return save_payload_none;
