@@ -135,10 +135,33 @@ void SyntaxLineEdit::setSyntaxState(SyntaxState state) {
     setStyleSheet(style_sheet_);
 }
 
-QString SyntaxLineEdit::syntaxErrorMessage() {
+QString SyntaxLineEdit::syntaxErrorMessage()
+{
     return syntax_error_message_;
 }
 
+QString SyntaxLineEdit::syntaxErrorMessageFull()
+{
+    return syntax_error_message_full_;
+}
+
+QString SyntaxLineEdit::createSyntaxErrorMessageFull(
+                                const QString &filter, const QString &err_msg,
+                                qsizetype loc_start, size_t loc_length)
+{
+    QString msg = tr("Invalid filter: %1").arg(err_msg);
+
+    if (loc_start >= 0 && loc_length >= 1) {
+        // Add underlined location
+        msg = QString("<p>%1<pre>  %2\n  %3^%4</pre></p>")
+            .arg(msg)
+            .arg(filter)
+            .arg(QString(' ').repeated(static_cast<int>(loc_start)))
+            .arg(QString('~').repeated(static_cast<int>(loc_length) - 1));
+    }
+    return msg;
+}
+
 QString SyntaxLineEdit::styleSheet() const {
     return style_sheet_;
 }
@@ -178,13 +201,19 @@ bool SyntaxLineEdit::checkDisplayFilter(QString filter)
     }
 
     dfilter_t *dfp = NULL;
-    gchar *err_msg;
-    if (dfilter_compile(filter.toUtf8().constData(), &dfp, &err_msg)) {
+    df_error_t *df_err = NULL;
+    if (dfilter_compile(filter.toUtf8().constData(), &dfp, &df_err)) {
+        GSList *warn;
         GPtrArray *depr = NULL;
-        if (dfp) {
-            depr = dfilter_deprecated_tokens(dfp);
-        }
-        if (depr) {
+        if (dfp != NULL && (warn = dfilter_get_warnings(dfp)) != NULL) {
+            // FIXME Need to use a different state or rename ::Deprecated
+            setSyntaxState(SyntaxLineEdit::Deprecated);
+            /*
+            * We're being lazy and only printing the first warning.
+            * Would it be better to print all of them?
+            */
+            syntax_error_message_  = QString(static_cast<gchar *>(warn->data));
+        } else if (dfp != NULL && (depr = dfilter_deprecated_tokens(dfp)) != NULL) {
             // You keep using that word. I do not think it means what you think it means.
             // Possible alternatives: ::Troubled, or ::Problematic maybe?
             setSyntaxState(SyntaxLineEdit::Deprecated);
@@ -207,8 +236,9 @@ bool SyntaxLineEdit::checkDisplayFilter(QString filter)
         }
     } else {
         setSyntaxState(SyntaxLineEdit::Invalid);
-        syntax_error_message_ = QString::fromUtf8(err_msg);
-        g_free(err_msg);
+        syntax_error_message_ = QString::fromUtf8(df_err->msg);
+        syntax_error_message_full_ = createSyntaxErrorMessageFull(filter, syntax_error_message_, df_err->loc.col_start, df_err->loc.col_len);
+        df_error_free(&df_err);
     }
     dfilter_free(dfp);
 
@@ -353,10 +383,9 @@ void SyntaxLineEdit::completionKeyPressEvent(QKeyEvent *event)
         return;
     }
 
-    QPoint token_coords(getTokenUnderCursor());
-
-    QString token_word = text().mid(token_coords.x(), token_coords.y());
-    buildCompletionList(token_word);
+    QStringList sentence(splitLineUnderCursor());
+    Q_ASSERT(sentence.size() == 2);     // (preamble, token)
+    buildCompletionList(sentence[1] /* token */, sentence[0] /* preamble */);
 
     if (completion_model_->stringList().length() < 1) {
         completer_->popup()->hide();
@@ -409,7 +438,7 @@ void SyntaxLineEdit::paintEvent(QPaintEvent *event)
 #if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
     // Must match CaptureFilterEdit and DisplayFilterEdit stylesheets.
     int pad = style()->pixelMetric(QStyle::PM_DefaultFrameWidth) + 1;
-    QRect full_cr = cr.adjusted(-pad, 0, 0, 0);
+    QRect full_cr = cr.adjusted(-pad, 0, -1, 0);
     QBrush bg;
 
     switch (syntax_state_) {
@@ -508,3 +537,19 @@ QPoint SyntaxLineEdit::getTokenUnderCursor()
 
     return QPoint(start, len);
 }
+
+QStringList SyntaxLineEdit::splitLineUnderCursor()
+{
+    QPoint token_coords(getTokenUnderCursor());
+
+    // Split line into preamble and word under cursor.
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
+    QString preamble = text().first(token_coords.x()).trimmed();
+#else
+    QString preamble = text().mid(0, token_coords.x()).trimmed();
+#endif
+    // This should be trimmed already
+    QString token_word = text().mid(token_coords.x(), token_coords.y());
+
+    return QStringList{ preamble, token_word };
+}
