@@ -38,31 +38,17 @@ static gboolean
 candump_write_packet(wtap_rec *rec, Buffer *buf, const msg_t *msg, int *err,
                      gchar **err_info)
 {
-    static const char *can_proto_name    = "can-hostendian";
-    static const char *canfd_proto_name  = "canfd";
-    const char        *proto_name        = msg->is_fd ? canfd_proto_name : can_proto_name;
-    guint              proto_name_length = (guint)strlen(proto_name) + 1;
-    guint              header_length;
-    guint              packet_length;
-    guint              frame_length;
-    guint8            *buf_data;
-
-    /* Adjust proto name length to be aligned on 4 byte boundary */
-    proto_name_length += (proto_name_length % 4) ? (4 - (proto_name_length % 4)) : 0;
-
-    header_length = 4 + proto_name_length + 4;
-    frame_length  = msg->is_fd ? sizeof(canfd_frame_t) : sizeof(can_frame_t);
-    packet_length = header_length + frame_length;
-
+    /* Generate Exported PDU tags for the packet info */
     ws_buffer_clean(buf);
-    ws_buffer_assure_space(buf, packet_length);
-    buf_data = ws_buffer_start_ptr(buf);
-
-    memset(buf_data, 0, packet_length);
-
-    phton16(buf_data + 0, EXP_PDU_TAG_DISSECTOR_NAME);
-    phton16(buf_data + 2, proto_name_length);
-    memcpy(buf_data + 4, proto_name, strlen(proto_name));
+    if (msg->is_fd)
+    {
+        wtap_buffer_append_epdu_string(buf, EXP_PDU_TAG_DISSECTOR_NAME, "canfd");
+    }
+    else
+    {
+        wtap_buffer_append_epdu_string(buf, EXP_PDU_TAG_DISSECTOR_NAME, "can-hostendian");
+    }
+    wtap_buffer_append_epdu_end(buf);
 
     if (msg->is_fd)
     {
@@ -85,7 +71,7 @@ candump_write_packet(wtap_rec *rec, Buffer *buf, const msg_t *msg, int *err,
         canfd_frame.len    = msg->data.length;
         memcpy(canfd_frame.data, msg->data.data, msg->data.length);
 
-        memcpy(buf_data + header_length, (guint8 *)&canfd_frame, sizeof(canfd_frame));
+        ws_buffer_append(buf, (guint8 *)&canfd_frame, sizeof(canfd_frame));
     }
     else
     {
@@ -107,7 +93,7 @@ candump_write_packet(wtap_rec *rec, Buffer *buf, const msg_t *msg, int *err,
         can_frame.can_dlc = msg->data.length;
         memcpy(can_frame.data, msg->data.data, msg->data.length);
 
-        memcpy(buf_data + header_length, (guint8 *)&can_frame, sizeof(can_frame));
+        ws_buffer_append(buf, (guint8 *)&can_frame, sizeof(can_frame));
     }
 
     rec->rec_type       = REC_TYPE_PACKET;
@@ -116,8 +102,8 @@ candump_write_packet(wtap_rec *rec, Buffer *buf, const msg_t *msg, int *err,
     rec->ts             = msg->ts;
     rec->tsprec         = WTAP_TSPREC_USEC;
 
-    rec->rec_header.packet_header.caplen = packet_length;
-    rec->rec_header.packet_header.len    = packet_length;
+    rec->rec_header.packet_header.caplen = (guint32)ws_buffer_length(buf);
+    rec->rec_header.packet_header.len    = (guint32)ws_buffer_length(buf);
 
     return TRUE;
 }
