@@ -8,6 +8,8 @@
  */
 #include "config.h"
 
+#define WS_LOG_DOMAIN LOG_DOMAIN_WIRETAP
+
 #include <glib.h>
 #include <string.h>
 
@@ -19,6 +21,7 @@
 
 #include <wsutil/glib-compat.h>
 #include <wsutil/inet_ipv6.h>
+#include <wsutil/unicode-utils.h>
 
 #if 0
 #define wtap_debug(...) ws_warning(__VA_ARGS__)
@@ -194,6 +197,29 @@ void wtap_packet_verdict_free(packet_verdict_opt_t* verdict)
     }
 }
 
+static packet_hash_opt_t
+packet_hash_dup(packet_hash_opt_t* hash_src)
+{
+    packet_hash_opt_t hash_dest;
+
+    memset(&hash_dest, 0, sizeof(hash_dest));
+
+    /* Deep copy. */
+    hash_dest.type = hash_src->type;
+    /* array of octets */
+    hash_dest.hash_bytes =
+        g_byte_array_new_take((guint8 *)g_memdup2(hash_src->hash_bytes->data,
+                                                  hash_src->hash_bytes->len),
+                              hash_src->hash_bytes->len);
+    return hash_dest;
+}
+
+void wtap_packet_hash_free(packet_hash_opt_t* hash)
+{
+    /* array of bytes */
+    g_byte_array_free(hash->hash_bytes, TRUE);
+}
+
 static void wtap_opttype_block_register(wtap_blocktype_t *blocktype)
 {
     wtap_block_type_t block_type;
@@ -364,6 +390,10 @@ static void wtap_block_free_option(wtap_block_t block, wtap_option_t *opt)
         wtap_packet_verdict_free(&opt->value.packet_verdictval);
         break;
 
+    case WTAP_OPTTYPE_PACKET_HASH:
+        wtap_packet_hash_free(&opt->value.packet_hash);
+        break;
+
     default:
         break;
     }
@@ -474,6 +504,18 @@ wtap_block_copy(wtap_block_t dest_block, wtap_block_t src_block)
             wtap_block_add_uint64_option(dest_block, src_opt->option_id, src_opt->value.uint64val);
             break;
 
+        case WTAP_OPTTYPE_INT8:
+            wtap_block_add_int8_option(dest_block, src_opt->option_id, src_opt->value.int8val);
+            break;
+
+        case WTAP_OPTTYPE_INT32:
+            wtap_block_add_int32_option(dest_block, src_opt->option_id, src_opt->value.int32val);
+            break;
+
+        case WTAP_OPTTYPE_INT64:
+            wtap_block_add_int64_option(dest_block, src_opt->option_id, src_opt->value.int64val);
+            break;
+
         case WTAP_OPTTYPE_IPv4:
             wtap_block_add_ipv4_option(dest_block, src_opt->option_id, src_opt->value.ipv4val);
             break;
@@ -508,6 +550,10 @@ wtap_block_copy(wtap_block_t dest_block, wtap_block_t src_block)
         case WTAP_OPTTYPE_PACKET_VERDICT:
             wtap_block_add_packet_verdict_option(dest_block, src_opt->option_id, &src_opt->value.packet_verdictval);
             break;
+
+        case WTAP_OPTTYPE_PACKET_HASH:
+            wtap_block_add_packet_hash_option(dest_block, src_opt->option_id, &src_opt->value.packet_hash);
+            break;
         }
     }
 }
@@ -823,6 +869,123 @@ wtap_block_get_uint64_option_value(wtap_block_t block, guint option_id, guint64
     return WTAP_OPTTYPE_SUCCESS;
 }
 
+wtap_opttype_return_val
+wtap_block_add_int8_option(wtap_block_t block, guint option_id, gint8 value)
+{
+    wtap_opttype_return_val ret;
+    wtap_option_t *opt;
+
+    ret = wtap_block_add_option_common(block, option_id, WTAP_OPTTYPE_INT8, &opt);
+    if (ret != WTAP_OPTTYPE_SUCCESS)
+        return ret;
+    opt->value.int8val = value;
+    return WTAP_OPTTYPE_SUCCESS;
+}
+
+wtap_opttype_return_val
+wtap_block_set_int8_option_value(wtap_block_t block, guint option_id, gint8 value)
+{
+    wtap_opttype_return_val ret;
+    wtap_optval_t *optval;
+
+    ret = wtap_block_get_option_common(block, option_id, WTAP_OPTTYPE_INT8, &optval);
+    if (ret != WTAP_OPTTYPE_SUCCESS)
+        return ret;
+    optval->int8val = value;
+    return WTAP_OPTTYPE_SUCCESS;
+}
+
+wtap_opttype_return_val
+wtap_block_get_int8_option_value(wtap_block_t block, guint option_id, gint8* value)
+{
+    wtap_opttype_return_val ret;
+    wtap_optval_t *optval;
+
+    ret = wtap_block_get_option_common(block, option_id, WTAP_OPTTYPE_INT8, &optval);
+    if (ret != WTAP_OPTTYPE_SUCCESS)
+        return ret;
+    *value = optval->int8val;
+    return WTAP_OPTTYPE_SUCCESS;
+}
+
+wtap_opttype_return_val
+wtap_block_add_int32_option(wtap_block_t block, guint option_id, gint32 value)
+{
+    wtap_opttype_return_val ret;
+    wtap_option_t *opt;
+
+    ret = wtap_block_add_option_common(block, option_id, WTAP_OPTTYPE_INT32, &opt);
+    if (ret != WTAP_OPTTYPE_SUCCESS)
+        return ret;
+    opt->value.int32val = value;
+    return WTAP_OPTTYPE_SUCCESS;
+}
+
+wtap_opttype_return_val
+wtap_block_set_int32_option_value(wtap_block_t block, guint option_id, gint32 value)
+{
+    wtap_opttype_return_val ret;
+    wtap_optval_t *optval;
+
+    ret = wtap_block_get_option_common(block, option_id, WTAP_OPTTYPE_INT32, &optval);
+    if (ret != WTAP_OPTTYPE_SUCCESS)
+        return ret;
+    optval->int32val = value;
+    return WTAP_OPTTYPE_SUCCESS;
+}
+
+wtap_opttype_return_val
+wtap_block_get_int32_option_value(wtap_block_t block, guint option_id, gint32* value)
+{
+    wtap_opttype_return_val ret;
+    wtap_optval_t *optval;
+
+    ret = wtap_block_get_option_common(block, option_id, WTAP_OPTTYPE_INT32, &optval);
+    if (ret != WTAP_OPTTYPE_SUCCESS)
+        return ret;
+    *value = optval->int32val;
+    return WTAP_OPTTYPE_SUCCESS;
+}
+
+wtap_opttype_return_val
+wtap_block_add_int64_option(wtap_block_t block, guint option_id, gint64 value)
+{
+    wtap_opttype_return_val ret;
+    wtap_option_t *opt;
+
+    ret = wtap_block_add_option_common(block, option_id, WTAP_OPTTYPE_INT64, &opt);
+    if (ret != WTAP_OPTTYPE_SUCCESS)
+        return ret;
+    opt->value.int64val = value;
+    return WTAP_OPTTYPE_SUCCESS;
+}
+
+wtap_opttype_return_val
+wtap_block_set_int64_option_value(wtap_block_t block, guint option_id, gint64 value)
+{
+    wtap_opttype_return_val ret;
+    wtap_optval_t *optval;
+
+    ret = wtap_block_get_option_common(block, option_id, WTAP_OPTTYPE_INT64, &optval);
+    if (ret != WTAP_OPTTYPE_SUCCESS)
+        return ret;
+    optval->int64val = value;
+    return WTAP_OPTTYPE_SUCCESS;
+}
+
+wtap_opttype_return_val
+wtap_block_get_int64_option_value(wtap_block_t block, guint option_id, gint64 *value)
+{
+    wtap_opttype_return_val ret;
+    wtap_optval_t *optval;
+
+    ret = wtap_block_get_option_common(block, option_id, WTAP_OPTTYPE_INT64, &optval);
+    if (ret != WTAP_OPTTYPE_SUCCESS)
+        return ret;
+    *value = optval->int64val;
+    return WTAP_OPTTYPE_SUCCESS;
+}
+
 wtap_opttype_return_val
 wtap_block_add_ipv4_option(wtap_block_t block, guint option_id, guint32 value)
 {
@@ -911,6 +1074,21 @@ wtap_block_add_string_option(wtap_block_t block, guint option_id, const char *va
     if (ret != WTAP_OPTTYPE_SUCCESS)
         return ret;
     opt->value.stringval = g_strndup(value, value_length);
+    WS_UTF_8_CHECK(opt->value.stringval, -1);
+    return WTAP_OPTTYPE_SUCCESS;
+}
+
+wtap_opttype_return_val
+wtap_block_add_string_option_owned(wtap_block_t block, guint option_id, char *value)
+{
+    wtap_opttype_return_val ret;
+    wtap_option_t *opt;
+
+    ret = wtap_block_add_option_common(block, option_id, WTAP_OPTTYPE_STRING, &opt);
+    if (ret != WTAP_OPTTYPE_SUCCESS)
+        return ret;
+    opt->value.stringval = value;
+    WS_UTF_8_CHECK(opt->value.stringval, -1);
     return WTAP_OPTTYPE_SUCCESS;
 }
 
@@ -924,6 +1102,7 @@ wtap_block_add_string_option_vformat(wtap_block_t block, guint option_id, const
     if (ret != WTAP_OPTTYPE_SUCCESS)
         return ret;
     opt->value.stringval = ws_strdup_vprintf(format, va);
+    WS_UTF_8_CHECK(opt->value.stringval, -1);
     return WTAP_OPTTYPE_SUCCESS;
 }
 
@@ -1428,6 +1607,19 @@ wtap_block_get_nth_packet_verdict_option_value(wtap_block_t block, guint option_
     return WTAP_OPTTYPE_SUCCESS;
 }
 
+wtap_opttype_return_val
+wtap_block_add_packet_hash_option(wtap_block_t block, guint option_id, packet_hash_opt_t* value)
+{
+    wtap_opttype_return_val ret;
+    wtap_option_t *opt;
+
+    ret = wtap_block_add_option_common(block, option_id, WTAP_OPTTYPE_PACKET_HASH, &opt);
+    if (ret != WTAP_OPTTYPE_SUCCESS)
+        return ret;
+    opt->value.packet_hash = packet_hash_dup(value);
+    return WTAP_OPTTYPE_SUCCESS;
+}
+
 wtap_opttype_return_val
 wtap_block_remove_option(wtap_block_t block, guint option_id)
 {
@@ -1534,9 +1726,46 @@ static void shb_copy_mand(wtap_block_t dest_block, wtap_block_t src_block)
 
 static void nrb_create(wtap_block_t block)
 {
-    block->mandatory_data = NULL;
+    block->mandatory_data = g_new0(wtapng_nrb_mandatory_t, 1);
 }
 
+static void nrb_free_mand(wtap_block_t block)
+{
+    wtapng_nrb_mandatory_t *mand = (wtapng_nrb_mandatory_t *)block->mandatory_data;
+    g_list_free_full(mand->ipv4_addr_list, g_free);
+    g_list_free_full(mand->ipv6_addr_list, g_free);
+}
+
+#if 0
+static gpointer copy_hashipv4(gconstpointer src, gpointer user_data _U_
+{
+    hashipv4_t *src_ipv4 = (hashipv4_t*)src;
+    hashipv4_t *dst = g_new0(hashipv4_t, 1);
+    dst->addr = src_ipv4->addr;
+    (void) g_strlcpy(dst->name, src_ipv4->name, MAXNAMELEN);
+    return dst;
+}
+
+static gpointer copy_hashipv4(gconstpointer src, gpointer user_data _U_
+{
+    hashipv6_t *src_ipv6 = (hashipv6_t*)src;
+    hashipv6_t *dst = g_new0(hashipv6_t, 1);
+    dst->addr = src_ipv4->addr;
+    (void) g_strlcpy(dst->name, src_ipv4->name, MAXNAMELEN);
+    return dst;
+}
+
+static void nrb_copy_mand(wtap_block_t dest_block, wtap_block_t src_block)
+{
+    wtapng_nrb_mandatory_t *src = (wtapng_nrb_mandatory_t *)src_block->mandatory_data;
+    wtapng_nrb_mandatory_t *dst = (wtapng_nrb_mandatory_t *)dest_block->mandatory_data;
+    g_list_free_full(dst->ipv4_addr_list, g_free);
+    g_list_free_full(dst->ipv6_addr_list, g_free);
+    dst->ipv4_addr_list = g_list_copy_deep(src->ipv4_addr_list, copy_hashipv4, NULL);
+    dst->ipv6_addr_list = g_list_copy_deep(src->ipv6_addr_list, copy_hashipv6, NULL);
+}
+#endif
+
 static void isb_create(wtap_block_t block)
 {
     block->mandatory_data = g_new0(wtapng_if_stats_mandatory_t, 1);
@@ -1612,6 +1841,27 @@ static void dsb_copy_mand(wtap_block_t dest_block, wtap_block_t src_block)
     dst->secrets_data = (guint8 *)g_memdup2(src->secrets_data, src->secrets_len);
 }
 
+static void sysdig_mev_create(wtap_block_t block)
+{
+    block->mandatory_data = g_new0(wtapng_sysdig_mev_mandatory_t, 1);
+}
+
+static void sysdig_mev_free_mand(wtap_block_t block)
+{
+    wtapng_sysdig_mev_mandatory_t *mand = (wtapng_sysdig_mev_mandatory_t *)block->mandatory_data;
+    g_free(mand->mev_data);
+}
+
+static void sysdig_mev_copy_mand(wtap_block_t dest_block, wtap_block_t src_block)
+{
+    wtapng_sysdig_mev_mandatory_t *src = (wtapng_sysdig_mev_mandatory_t *)src_block->mandatory_data;
+    wtapng_sysdig_mev_mandatory_t *dst = (wtapng_sysdig_mev_mandatory_t *)dest_block->mandatory_data;
+    dst->mev_type = src->mev_type;
+    dst->mev_data_len = src->mev_data_len;
+    g_free(dst->mev_data);
+    dst->mev_data = (guint8 *)g_memdup2(src->mev_data, src->mev_data_len);
+}
+
 static void pkt_create(wtap_block_t block)
 {
     /* Commented out for now, there's no mandatory data that isn't handled by
@@ -1716,6 +1966,12 @@ void wtap_opttypes_initialize(void)
         WTAP_OPTTYPE_UINT8,
         0
     };
+    static const wtap_opttype_t if_tsoffset = {
+        "tsoffset",
+        "IDB Time Stamp Offset",
+        WTAP_OPTTYPE_INT64,
+        0
+    };
     static const wtap_opttype_t if_hardware = {
         "hardware",
         "IDB Hardware",
@@ -1738,8 +1994,18 @@ void wtap_opttypes_initialize(void)
         "NRB",                      /* name */
         "Name Resolution Block",    /* description */
         nrb_create,                 /* create */
-        NULL,                       /* free_mand */
-        NULL,                       /* copy_mand */
+        nrb_free_mand,              /* free_mand */
+        /* We eventually want to copy these, when dumper actually
+         * writes them out. If we're actually processing packets,
+         * as opposed to just reading and writing a file without
+         * printing (e.g., editcap), do we still want to copy all
+         * the pre-existing NRBs, or do we want to limit it to
+         * the actually used addresses, as currently?
+         */
+#if 0
+        nrb_copy_mand,              /* copy_mand */
+#endif
+        NULL,
         NULL                        /* options */
     };
     static const wtap_opttype_t ns_dnsname = {
@@ -1813,6 +2079,16 @@ void wtap_opttypes_initialize(void)
         0
     };
 
+    static wtap_blocktype_t sysdig_mev_block = {
+        WTAP_BLOCK_SYSDIG_META_EVENT,
+        "Sysdig MEV",
+        "Sysdig Meta Event Block",
+        sysdig_mev_create,
+        sysdig_mev_free_mand,
+        sysdig_mev_copy_mand,
+        NULL
+    };
+
     static wtap_blocktype_t pkt_block = {
         WTAP_BLOCK_PACKET,            /* block_type */
         "EPB/SPB/PB",                 /* name */
@@ -1849,7 +2125,7 @@ void wtap_opttypes_initialize(void)
     static const wtap_opttype_t pkt_hash = {
         "hash",
         "Hash of packet data",
-        WTAP_OPTTYPE_BYTES,  // TODO: replace with a pkt_hash_opt_t
+        WTAP_OPTTYPE_PACKET_HASH,
         WTAP_OPTTYPE_FLAG_MULTIPLE_ALLOWED
     };
     static const wtap_opttype_t pkt_verdict = {
@@ -1898,6 +2174,7 @@ void wtap_opttypes_initialize(void)
     wtap_opttype_option_register(&idb_block, OPT_IDB_FILTER, &if_filter);
     wtap_opttype_option_register(&idb_block, OPT_IDB_OS, &if_os);
     wtap_opttype_option_register(&idb_block, OPT_IDB_FCSLEN, &if_fcslen);
+    wtap_opttype_option_register(&idb_block, OPT_IDB_TSOFFSET, &if_tsoffset);
     wtap_opttype_option_register(&idb_block, OPT_IDB_HARDWARE, &if_hardware);
 
     /*
@@ -1925,6 +2202,11 @@ void wtap_opttypes_initialize(void)
      */
     wtap_opttype_block_register(&dsb_block);
 
+    /*
+     * Register the Sysdig MEV, currently no options are defined.
+     */
+    wtap_opttype_block_register(&sysdig_mev_block);
+
     /*
      * Register EPB/SPB/PB and the options that can appear in it/them.
      * NB: Simple Packet Blocks have no options.
